@@ -40,6 +40,25 @@
 
 #define CERT_EXPIRY_TIME (60*60*24*30) /* 30 days */
 
+#define DTLS_CT_CHANGE_CIPHER_SPEC 20
+#define DTLS_CT_ALERT 21
+#define DTLS_CT_HANDSHAKE 22
+#define DTLS_CT_APPLICATION_DATA 23
+
+// https://www.iana.org/assignments/tls-parameters/tls-parameters.xhtml#tls-parameters-7
+#define DTLS_HT_HELLO_REQUEST 0
+#define DTLS_HT_CLIENT_HELLO 1
+#define DTLS_HT_SERVER_HELLO 2
+#define DTLS_HT_HELLO_VERIFY_REQUEST 3
+#define DTLS_HT_NEW_SESSION_TICKET 4
+#define DTLS_HT_CERTIFICATE 11
+#define DTLS_HT_SERVER_KEY_EXCHANGE 12
+#define DTLS_HT_CERTIFICATE_REQUEST 13
+#define DTLS_HT_SERVER_HELLO_DONE 14
+#define DTLS_HT_CERTIFICATE_VERIFY 15
+#define DTLS_HT_CLIENT_KEY_EXCHANGE 16
+#define DTLS_HT_FINISHED 20
+
 struct dtls_connection *dtls_ptr(stream_fd *sfd) {
 	if (!sfd)
 		return NULL;
@@ -50,13 +69,9 @@ struct dtls_connection *dtls_ptr(stream_fd *sfd) {
 }
 
 
-
-
-
 static char ciphers_str[1024];
 
 
-
 static unsigned int sha_1_func(unsigned char *, X509 *);
 static unsigned int sha_224_func(unsigned char *, X509 *);
 static unsigned int sha_256_func(unsigned char *, X509 *);
@@ -64,6 +79,8 @@ static unsigned int sha_384_func(unsigned char *, X509 *);
 static unsigned int sha_512_func(unsigned char *, X509 *);
 
 
+static const char *dtls_content_type_str(unsigned char type);
+static const char *dlts_handshake_type_str(unsigned char type);
 
 
 static const struct dtls_hash_func hash_funcs[] = {
@@ -99,7 +116,7 @@ const int num_hash_funcs = G_N_ELEMENTS(hash_funcs);
 
 
 static struct dtls_cert *__dtls_cert;
-static rwlock_t __dtls_cert_lock;
+static rwlock_t __dtls_cert_lock = RWLOCK_STATIC_INIT;
 
 
 
@@ -118,9 +135,7 @@ const struct dtls_hash_func *dtls_find_hash_func(const str *s) {
 	return NULL;
 }
 
-static void cert_free(void *p) {
-	struct dtls_cert *cert = p;
-
+static void cert_free(struct dtls_cert *cert) {
 	if (cert->pkey)
 		EVP_PKEY_free(cert->pkey);
 	if (cert->x509)
@@ -321,7 +336,7 @@ static int cert_init(void) {
 
 	/* digest */
 
-	new_cert = obj_alloc0("dtls_cert", sizeof(*new_cert), cert_free);
+	new_cert = obj_alloc0(struct dtls_cert, cert_free);
 
 	for (int i = 0; i < num_hash_funcs; i++) {
 		struct dtls_fingerprint *fp = malloc(sizeof(*fp));
@@ -332,7 +347,7 @@ static int cert_init(void) {
 
 	new_cert->x509 = x509;
 	new_cert->pkey = pkey;
-	new_cert->expires = time(NULL) + CERT_EXPIRY_TIME;
+	new_cert->expires_us = now_us() + CERT_EXPIRY_TIME * 1000000LL;
 
 	dump_cert(new_cert);
 
@@ -381,7 +396,6 @@ int dtls_init(void) {
 	int i;
 	char *p;
 
-	rwlock_init(&__dtls_cert_lock);
 	if (cert_init())
 		return -1;
 
@@ -403,14 +417,14 @@ int dtls_init(void) {
 
 static enum thread_looper_action __dtls_timer(void) {
 	struct dtls_cert *c;
-	long int left;
+	int64_t left;
 
 	c = dtls_cert();
 	if (!c)
 		return TLA_BREAK;
 
-	left = c->expires - rtpe_now.tv_sec;
-	if (left > CERT_EXPIRY_TIME/2)
+	left = c->expires_us - rtpe_now;
+	if (left > CERT_EXPIRY_TIME * 1000000LL / 2)
 		goto out;
 
 	cert_init();
@@ -493,7 +507,7 @@ static int verify_callback(int ok, X509_STORE_CTX *store) {
 	d = SSL_get_app_data(ssl);
 	if (d->ssl != ssl)
 		return 0;
-	ps = d->ptr;
+	ps = d->ps;
 	if (!ps)
 		return 0;
 	if (PS_ISSET(ps, FINGERPRINT_VERIFIED))
@@ -558,7 +572,7 @@ static int try_connect(struct dtls_connection *d) {
 	int ret, code;
 	unsigned char buf[0x10000];
 
-	__DBG("try_connect(%i)", d->active);
+	__DBG("try_connect(%u)", d->active);
 
 	if (d->connected)
 		ret = SSL_read(d->ssl, buf, sizeof(buf)); /* retransmission after connected - handshake lost */
@@ -585,9 +599,12 @@ static int try_connect(struct dtls_connection *d) {
 		case SSL_ERROR_WANT_WRITE:
 			if (d->connected) {
 				ilogs(crypto, LOG_INFO, "DTLS data received after handshake, code: %i", code);
+			} else {
+				ilogs(crypto, LOG_DEBUG, "DTLS handshake not completed yet, current state: %s",
+					SSL_state_string_long(d->ssl));
 			}
 			break;
-                case SSL_ERROR_ZERO_RETURN:
+		case SSL_ERROR_ZERO_RETURN:
 			if (d->connected) {
 				ilogs(crypto, LOG_INFO, "DTLS peer has closed the connection");
 				ret = -2;
@@ -620,15 +637,15 @@ static long dtls_bio_callback(BIO *bio, int oper, const char *argp, size_t len,
 	if (!argp || len <= 0)
 		return ret;
 
-	struct packet_stream *ps = (struct packet_stream *) BIO_get_callback_arg(bio);
+	struct dtls_connection *d = (struct dtls_connection *) BIO_get_callback_arg(bio);
+	if (!d)
+		return ret;
+	struct packet_stream *ps = d->ps;
 	if (!ps)
 		return ret;
-	struct stream_fd *sfd = ps->selected_sfd;
+	struct stream_fd *sfd = d->sfd;
 	if (!sfd)
 		return ret;
-	struct dtls_connection *d = dtls_ptr(sfd);
-	if (!d)
-		return ret;
 
 	__DBG("dtls packet output: len %zu %02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x",
 		len,
@@ -637,11 +654,23 @@ static long dtls_bio_callback(BIO *bio, int oper, const char *argp, size_t len,
 		argp[8], argp[9], argp[10], argp[11],
 		argp[12], argp[13], argp[14], argp[15]);
 
-	const endpoint_t *fsin = &ps->endpoint;
+	const endpoint_t *fsin = &d->fsin;
+	if (fsin->address.family == NULL)
+		fsin = &ps->endpoint;
 	if (fsin->port == 9 || fsin->address.family == NULL)
 		return ret;
+	
+	if(len > 13 && (unsigned char)argp[0] == DTLS_CT_HANDSHAKE) {
+		ilogs(srtp, LOG_INFO, "Sending DTLS handshake %02x %s packet to %s",
+            argp[13],
+			dlts_handshake_type_str((unsigned char)argp[13]),
+			endpoint_print_buf(fsin));
+	} else {
+		ilogs(srtp, LOG_DEBUG, "Sending DTLS %s packet to %s",
+			dtls_content_type_str((unsigned char)argp[0]),
+			endpoint_print_buf(fsin));
+	}
 
-	ilogs(srtp, LOG_DEBUG, "Sending DTLS packet");
 	socket_sendto(&sfd->socket, argp, len, fsin);
 	atomic64_inc_na(&ps->stats_out->packets);
 	atomic64_add_na(&ps->stats_out->bytes, len);
@@ -649,6 +678,14 @@ static long dtls_bio_callback(BIO *bio, int oper, const char *argp, size_t len,
 	return ret;
 }
 
+#if OPENSSL_VERSION_NUMBER < 0x10101000L
+static long dtls_bio_callback_wrap(BIO *bio, int oper, const char *argp, int argi, long argl,
+		long ret)
+{
+	return dtls_bio_callback(bio, oper, argp, argi, argi, argl, ret, NULL);
+}
+#endif
+
 int dtls_connection_init(struct dtls_connection *d, struct packet_stream *ps, int active,
 		struct dtls_cert *cert)
 {
@@ -665,7 +702,7 @@ int dtls_connection_init(struct dtls_connection *d, struct packet_stream *ps, in
 		dtls_connection_cleanup(d);
 	}
 
-	d->ptr = ps;
+	d->ps = ps;
 
 	ilogs(crypto, LOG_DEBUG, "Creating %s DTLS connection context", active ? "active" : "passive");
 
@@ -701,8 +738,12 @@ int dtls_connection_init(struct dtls_connection *d, struct packet_stream *ps, in
 	if (!d->r_bio || !d->w_bio)
 		goto error;
 
+#if OPENSSL_VERSION_NUMBER >= 0x10101000L
 	BIO_set_callback_ex(d->w_bio, dtls_bio_callback);
-	BIO_set_callback_arg(d->w_bio, (char *) ps);
+#else
+	BIO_set_callback(d->w_bio, dtls_bio_callback_wrap);
+#endif
+	BIO_set_callback_arg(d->w_bio, (char *) d);
 
 #if defined(BIO_CTRL_DGRAM_SET_MTU)
 	BIO_ctrl(d->w_bio, BIO_CTRL_DGRAM_SET_MTU, rtpe_config.dtls_mtu, NULL);
@@ -714,10 +755,12 @@ int dtls_connection_init(struct dtls_connection *d, struct packet_stream *ps, in
 	d->init = 1;
 	SSL_set_mode(d->ssl, SSL_MODE_ENABLE_PARTIAL_WRITE | SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER);
 
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-	int ec_groups[1] = { NID_X9_62_prime256v1 };
-	SSL_set1_groups(d->ssl, &ec_groups, G_N_ELEMENTS(ec_groups));
-#else // <3.0
+
+        /* SSL_set1_groups_list et al. is not
+         * necessary for OpenSSL >= 1.1.1 as it has sensible defaults
+         * minimally P-521:P-384:P-256
+         */
+#if OPENSSL_VERSION_NUMBER < 0x10101000L
 	EC_KEY* ecdh = EC_KEY_new_by_curve_name(NID_X9_62_prime256v1);
 	if (ecdh == NULL)
 		goto error;
@@ -734,7 +777,7 @@ int dtls_connection_init(struct dtls_connection *d, struct packet_stream *ps, in
 #endif
 #endif
 
-	d->active = active ? -1 : 0;
+	d->active = active ? 1 : 0;
 
 	random_string(d->tls_id, sizeof(d->tls_id));
 
@@ -865,8 +908,24 @@ int dtls(stream_fd *sfd, const str *s, const endpoint_t *fsin) {
 	if (!d->init || !d->ssl)
 		return -1;
 
+	// used by the BIO callback
+	d->fsin = fsin ? *fsin : (endpoint_t) {0};
+	obj_release(d->sfd);
+	d->sfd = obj_get(sfd);
+
 	if (s) {
-		ilogs(srtp, LOG_DEBUG, "Processing incoming DTLS packet");
+		if(s->len > 13 && (unsigned char)s->s[0] == DTLS_CT_HANDSHAKE) {
+			ilogs(srtp, LOG_INFO, "Processing incoming DTLS Handshake %02x %s packet from %s",
+                    (unsigned char)s->s[13],
+					dlts_handshake_type_str((unsigned char)s->s[13]),
+					endpoint_print_buf(fsin));
+		} else {
+			ilogs(srtp, LOG_DEBUG, "Processing incoming DTLS %s packet from %s",
+					dtls_content_type_str((unsigned char)s->s[0]),
+					endpoint_print_buf(fsin));
+		}
+
+
 		BIO_write(d->r_bio, s->s, s->len);
 		/* we understand this as preference of DTLS over SDES */
 		MEDIA_CLEAR(ps->media, SDES);
@@ -960,5 +1019,53 @@ void dtls_connection_cleanup(struct dtls_connection *c) {
 		if (c->w_bio)
 			BIO_free(c->w_bio);
 	}
+	obj_release(c->sfd);
 	ZERO(*c);
 }
+
+
+static const char *dtls_content_type_str(unsigned char type) {
+	switch (type) {
+		case DTLS_CT_CHANGE_CIPHER_SPEC:
+			return "ChangeCipherSpec";
+		case DTLS_CT_ALERT:
+			return "Alert";
+		case DTLS_CT_HANDSHAKE:
+			return "Handshake";
+		case DTLS_CT_APPLICATION_DATA:
+			return "ApplicationData";
+		default:
+			return "Unknown";
+	}
+}
+
+static const char *dlts_handshake_type_str(unsigned char type) {
+	switch (type) {
+		case DTLS_HT_HELLO_REQUEST:
+			return "HelloRequest";
+		case DTLS_HT_CLIENT_HELLO:
+			return "ClientHello";
+		case DTLS_HT_SERVER_HELLO:
+			return "ServerHello";
+		case DTLS_HT_HELLO_VERIFY_REQUEST:
+			return "HelloVerifyRequest";
+		case DTLS_HT_NEW_SESSION_TICKET:
+			return "NewSessionTicket";
+		case DTLS_HT_CERTIFICATE:
+			return "Certificate";
+		case DTLS_HT_SERVER_KEY_EXCHANGE:
+			return "ServerKeyExchange";
+		case DTLS_HT_CERTIFICATE_REQUEST:
+			return "CertificateRequest";
+		case DTLS_HT_SERVER_HELLO_DONE:
+			return "ServerHelloDone";
+		case DTLS_HT_CERTIFICATE_VERIFY:
+			return "CertificateVerify";
+		case DTLS_HT_CLIENT_KEY_EXCHANGE:
+			return "ClientKeyExchange";
+		case DTLS_HT_FINISHED:
+			return "Finished";
+		default:
+			return "Unknown";
+	}
+}
