@@ -37,7 +37,7 @@ struct scheduler {
 struct looper_thread {
 	enum thread_looper_action (*f)(void);
 	const char *name;
-	long long interval_us;
+	int64_t interval_us;
 };
 
 
@@ -116,7 +116,7 @@ int pcre2_multi_match(pcre2_code *re, const char *s, unsigned int num, parse_fun
 static void thread_join_me(void) {
 	pthread_t *me;
 
-	me = g_slice_alloc(sizeof(*me));
+	me = g_new(__typeof(*me), 1);
 	*me = pthread_self();
 	mutex_lock(&threads_lists_lock);
 	threads_to_join = g_list_prepend(threads_to_join, me);
@@ -157,12 +157,12 @@ void threads_join_all(bool cancel) {
 			threads_to_join = g_list_delete_link(threads_to_join, threads_to_join);
 			l = g_list_find_custom(threads_running, t, thread_equal);
 			if (l) {
-				g_slice_free1(sizeof(*t), l->data);
+				g_free(l->data);
 				threads_running = g_list_delete_link(threads_running, l);
 			}
 			else
 				abort();
-			g_slice_free1(sizeof(*t), t);
+			g_free(t);
 		}
 
 		if ((!cancel && rtpe_shutdown) || (cancel && !threads_running)) {
@@ -196,7 +196,7 @@ void thread_waker_del(struct thread_waker *wk) {
 
 static void thread_detach_cleanup(void *dtp) {
 	struct detach_thread *dt = dtp;
-	g_slice_free1(sizeof(*dt), dt);
+	g_free(dt);
 	bufferpool_destroy(media_bufferpool);
 #ifdef HAVE_LIBURING
 	if (rtpe_config.common.io_uring)
@@ -211,7 +211,7 @@ static void *thread_detach_func(void *d) {
 
 	pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
 
-	t = g_slice_alloc(sizeof(*t));
+	t = g_new(__typeof(*t), 1);
 	*t = pthread_self();
 	mutex_lock(&threads_lists_lock);
 	threads_running = g_list_prepend(threads_running, t);
@@ -254,7 +254,7 @@ static void *thread_detach_func(void *d) {
 					dt->priority, strerror(errno));
 	}
 
-	media_bufferpool = bufferpool_new(g_malloc, g_free, 64 * 65536);
+	media_bufferpool = bufferpool_new(bufferpool_aligned_alloc, bufferpool_aligned_free);
 #ifdef HAVE_LIBURING
 	if (rtpe_config.common.io_uring)
 		uring_thread_init();
@@ -272,7 +272,7 @@ void thread_create_detach_prio(void (*f)(void *), void *d, const char *scheduler
 {
 	struct detach_thread *dt;
 
-	dt = g_slice_alloc(sizeof(*dt));
+	dt = g_new(__typeof(*dt), 1);
 	dt->func = f;
 	dt->data = d;
 	dt->scheduler = scheduler;
@@ -286,40 +286,39 @@ static void thread_looper_helper(void *fp) {
 	// move object to stack and free it, so we can be cancelled without having a leak
 	struct looper_thread *lhp = fp;
 	struct looper_thread lh = *lhp;
-	g_slice_free1(sizeof(*lhp), lhp);
+	g_free(lhp);
 
-	long long interval_us = lh.interval_us;
+	int64_t interval_us = lh.interval_us;
 #ifdef ASAN_BUILD
 	interval_us = MIN(interval_us, 100000);
 #endif
-	static const long long warn_limit_pct = 20; // 20%
-	long long warn_limit_us = interval_us * warn_limit_pct / 100;
+	static const int64_t warn_limit_pct = 20; // 20%
+	int64_t warn_limit_us = interval_us * warn_limit_pct / 100;
 	struct timespec interval_ts = {
 		.tv_sec = interval_us / 1000000,
 		.tv_nsec = (interval_us % 1000000) * 1000,
 	};
 
 	while (!rtpe_shutdown) {
-		gettimeofday(&rtpe_now, NULL);
+		rtpe_now = now_us();
 
 		enum thread_looper_action ret = lh.f();
 
-		uring_thread_loop();
+		uring_methods.thread_loop();
 
-		struct timeval stop;
-		gettimeofday(&stop, NULL);
-		long long duration_us = timeval_diff(&stop, &rtpe_now);
+		if (ret == TLA_BREAK)
+			break;
+
+		int64_t stop = now_us();
+		int64_t duration_us = stop - rtpe_now;
 		if (duration_us > warn_limit_us)
-			ilog(LOG_WARN, "Run time of timer \"%s\": %lli.%06lli sec, "
-					"exceeding limit of %lli%% (%lli.%06lli sec)",
+			ilog(LOG_WARN, "Run time of timer \"%s\": %" PRId64 ".%06" PRId64 " sec, "
+					"exceeding limit of %" PRId64 "%% (%" PRId64 ".%06" PRId64 " sec)",
 					lh.name,
 					duration_us / 1000000, duration_us % 1000000,
 					warn_limit_pct,
 					warn_limit_us / 1000000, warn_limit_us % 1000000);
 
-		if (ret == TLA_BREAK)
-			break;
-
 		struct timespec sleeptime = interval_ts;
 		struct timespec remtime;
 		while (true) {
@@ -337,9 +336,9 @@ static void thread_looper_helper(void *fp) {
 
 void thread_create_looper(enum thread_looper_action (*f)(void), const char *scheduler, int priority,
 		const char *name,
-		long long interval_us)
+		int64_t interval_us)
 {
-	struct looper_thread *lh = g_slice_alloc(sizeof(*lh));
+	struct looper_thread *lh = g_new(__typeof(*lh), 1);
 	*lh = (__typeof__(*lh)) {
 		.f = f,
 		.name = name,
