@@ -33,20 +33,16 @@
 			STR_FMT_M(&(p)->remote_candidate->foundation),		\
 			(p)->remote_candidate->component_id
 
-struct fragment_key {
-	str call_id;
-	str from_tag;
-};
 struct sdp_fragment {
 	ng_buffer *ngbuf;
-	struct timeval received;
+	int64_t received;
 	sdp_streams_q streams;
 	sdp_ng_flags flags;
 };
 
 
 
-static void __ice_agent_free(void *p);
+static void __ice_agent_free(struct ice_agent *);
 static void create_random_ice_string(call_t *call, str *s, int len);
 static void __do_ice_checks(struct ice_agent *ag);
 static struct ice_candidate_pair *__pair_lookup(struct ice_agent *, struct ice_candidate *cand,
@@ -54,8 +50,8 @@ static struct ice_candidate_pair *__pair_lookup(struct ice_agent *, struct ice_c
 static void __recalc_pair_prios(struct ice_agent *ag);
 static void __role_change(struct ice_agent *ag, int new_controlling);
 static void __get_complete_components(candidate_pair_q *out, struct ice_agent *ag, GTree *t, unsigned int);
-static void __agent_schedule(struct ice_agent *ag, unsigned long);
-static void __agent_schedule_abs(struct ice_agent *ag, const struct timeval *tv);
+static void __agent_schedule(struct ice_agent *ag, int64_t);
+static void __agent_schedule_abs(struct ice_agent *ag, int64_t tv);
 static void __agent_deschedule(struct ice_agent *ag);
 static void __ice_agent_free_components(struct ice_agent *ag);
 static void __agent_shutdown(struct ice_agent *ag);
@@ -87,27 +83,19 @@ const char * const ice_type_strings[] = {
 
 
 
-static unsigned int frag_key_hash(const struct fragment_key *A);
-static int frag_key_eq(const struct fragment_key *A, const struct fragment_key *B);
-static void fragment_key_free(struct fragment_key *);
-
-TYPED_GQUEUE(fragment, struct sdp_fragment)
-TYPED_GHASHTABLE(fragments_ht, struct fragment_key, fragment_q, frag_key_hash, frag_key_eq,
-		fragment_key_free, NULL)
-TYPED_GHASHTABLE_LOOKUP_INSERT(fragments_ht, fragment_key_free, fragment_q_new)
-
-static mutex_t sdp_fragments_lock;
-static fragments_ht sdp_fragments;
+TYPED_GHASHTABLE_LOOKUP_INSERT(fragments_ht, NULL, fragment_q_new)
 
 
 
-static void ice_update_media_streams(struct call_monologue *ml, sdp_streams_q *streams) {
+static void ice_update_media_streams(struct call_monologue *ml, sdp_streams_q *streams,
+		sdp_ng_flags *flags)
+{
 	for (__auto_type l = streams->head; l; l = l->next) {
 		struct stream_params *sp = l->data;
 		struct call_media *media = NULL;
 
 		if (sp->media_id.len)
-			media = g_hash_table_lookup(ml->media_ids, &sp->media_id);
+			media = t_hash_table_lookup(ml->media_ids, &sp->media_id);
 		else if (sp->index > 0) {
 			unsigned int arr_idx = sp->index - 1;
 			if (arr_idx < ml->medias->len)
@@ -133,45 +121,28 @@ static void ice_update_media_streams(struct call_monologue *ml, sdp_streams_q *s
 }
 
 
-static unsigned int frag_key_hash(const struct fragment_key *a) {
-	return str_hash(&a->call_id) ^ str_hash(&a->from_tag);
-}
-static int frag_key_eq(const struct fragment_key *a, const struct fragment_key *b) {
-	return str_equal(&a->call_id, &b->call_id)
-		&& str_equal(&a->from_tag, &b->from_tag);
-}
-
 static void fragment_free(struct sdp_fragment *frag) {
 	sdp_streams_clear(&frag->streams);
 	call_ng_free_flags(&frag->flags);
 	obj_put(frag->ngbuf);
-	g_slice_free1(sizeof(*frag), frag);
+	g_free(frag);
 }
-static void fragment_key_free(struct fragment_key *k) {
-	g_free(k->call_id.s);
-	g_free(k->from_tag.s);
-	g_slice_free1(sizeof(*k), k);
-}
-static void queue_sdp_fragment(ng_buffer *ngbuf, sdp_streams_q *streams, sdp_ng_flags *flags) {
+static void queue_sdp_fragment(ng_buffer *ngbuf, call_t *call, str *key, sdp_streams_q *streams, sdp_ng_flags *flags) {
 	ilog(LOG_DEBUG, "Queuing up SDP fragment for " STR_FORMAT_M "/" STR_FORMAT_M,
 			STR_FMT_M(&flags->call_id), STR_FMT_M(&flags->from_tag));
 
-	struct fragment_key *k = g_slice_alloc0(sizeof(*k));
-	str_init_dup_str(&k->call_id, &flags->call_id);
-	str_init_dup_str(&k->from_tag, &flags->from_tag);
-
-	struct sdp_fragment *frag = g_slice_alloc0(sizeof(*frag));
+	struct sdp_fragment *frag = g_new0(__typeof(*frag), 1);
 	frag->received = rtpe_now;
 	frag->ngbuf = obj_get(ngbuf);
-	frag->streams = *streams;
+	if (streams) {
+		frag->streams = *streams;
+		t_queue_init(streams);
+	}
 	frag->flags = *flags;
-	t_queue_init(streams);
 	ZERO(*flags);
 
-	mutex_lock(&sdp_fragments_lock);
-	fragment_q *frags = fragments_ht_lookup_insert(sdp_fragments, k);
+	fragment_q *frags = fragments_ht_lookup_insert(call->sdp_fragments, call_str_dup(key));
 	t_queue_push_tail(frags, frag);
-	mutex_unlock(&sdp_fragments_lock);
 }
 bool trickle_ice_update(ng_buffer *ngbuf, call_t *call, sdp_ng_flags *flags,
 		sdp_streams_q *streams)
@@ -179,47 +150,37 @@ bool trickle_ice_update(ng_buffer *ngbuf, call_t *call, sdp_ng_flags *flags,
 	if (!flags->fragment)
 		return false;
 
-	if (!call) {
-		queue_sdp_fragment(ngbuf, streams, flags);
-		return true;
-	}
 	struct call_monologue *ml = call_get_monologue(call, &flags->from_tag);
 	if (!ml) {
-		queue_sdp_fragment(ngbuf, streams, flags);
+		queue_sdp_fragment(ngbuf, call, &flags->from_tag, streams, flags);
 		return true;
 	}
 
-	ice_update_media_streams(ml, streams);
+	ice_update_media_streams(ml, streams, flags);
 
 	return true;
 }
 #define MAX_FRAG_AGE 3000000
 void dequeue_sdp_fragments(struct call_monologue *monologue) {
-	struct fragment_key k;
-	ZERO(k);
-	k.call_id = monologue->call->callid;
-	k.from_tag = monologue->tag;
+	call_t *call = monologue->call;
 
 	fragment_q *frags = NULL;
 
-	{
-		LOCK(&sdp_fragments_lock);
-		t_hash_table_steal_extended(sdp_fragments, &k, NULL, &frags);
-		if (!frags)
-			return;
+	t_hash_table_steal_extended(call->sdp_fragments, &monologue->tag, NULL, &frags);
+	if (!frags)
+		return;
 
-		// we own the queue now
-	}
+	// we own the queue now
 
 	struct sdp_fragment *frag;
 	while ((frag = t_queue_pop_head(frags))) {
-		if (timeval_diff(&rtpe_now, &frag->received) > MAX_FRAG_AGE)
+		if (rtpe_now - frag->received > MAX_FRAG_AGE)
 			goto next;
 
 		ilog(LOG_DEBUG, "Dequeuing SDP fragment for " STR_FORMAT_M "/" STR_FORMAT_M,
-				STR_FMT_M(&k.call_id), STR_FMT_M(&k.from_tag));
+				STR_FMT_M(&call->callid), STR_FMT_M(&monologue->tag));
 
-		ice_update_media_streams(monologue, &frag->streams);
+		ice_update_media_streams(monologue, &frag->streams, &frag->flags);
 
 next:
 		fragment_free(frag);
@@ -227,13 +188,13 @@ next:
 
 	t_queue_free(frags);
 }
-static gboolean fragment_check_cleanup(struct fragment_key *key, fragment_q *frags, void *p) {
+static gboolean fragment_check_cleanup(str *key, fragment_q *frags, void *p) {
 	bool all = GPOINTER_TO_INT(p);
 	if (!key || !frags)
 		return TRUE;
 	while (frags->length) {
 		struct sdp_fragment *frag = frags->head->data;
-		if (!all && timeval_diff(&rtpe_now, &frag->received) <= MAX_FRAG_AGE)
+		if (!all && rtpe_now - frag->received <= MAX_FRAG_AGE)
 			break;
 		t_queue_pop_head(frags);
 		fragment_free(frag);
@@ -244,10 +205,8 @@ static gboolean fragment_check_cleanup(struct fragment_key *key, fragment_q *fra
 	}
 	return FALSE;
 }
-static void fragments_cleanup(bool all) {
-	mutex_lock(&sdp_fragments_lock);
-	t_hash_table_foreach_remove(sdp_fragments, fragment_check_cleanup, GINT_TO_POINTER(all));
-	mutex_unlock(&sdp_fragments_lock);
+void ice_fragments_cleanup(fragments_ht ht, bool all) {
+	t_hash_table_foreach_remove(ht, fragment_check_cleanup, GINT_TO_POINTER(all));
 }
 
 
@@ -301,7 +260,7 @@ static void __new_stun_transaction(struct ice_candidate_pair *pair) {
 /* agent must be locked */
 static void __all_pairs_list(struct ice_agent *ag) {
 	t_queue_clear(&ag->all_pairs_list);
-	g_tree_get_values(&ag->all_pairs_list.q, ag->all_pairs);
+	rtpe_g_tree_get_values(&ag->all_pairs_list.q, ag->all_pairs);
 }
 
 static void __tree_coll_callback(void *oo, void *nn) {
@@ -320,7 +279,7 @@ static struct ice_candidate_pair *__pair_candidate(stream_fd *sfd, struct ice_ag
 	if (sfd->socket.family != cand->endpoint.address.family)
 		return NULL;
 
-	pair = g_slice_alloc0(sizeof(*pair));
+	pair = g_new0(__typeof(*pair), 1);
 
 	pair->agent = ag;
 	pair->remote_candidate = cand;
@@ -333,7 +292,7 @@ static struct ice_candidate_pair *__pair_candidate(stream_fd *sfd, struct ice_ag
 
 	t_queue_push_tail(&ag->candidate_pairs, pair);
 	t_hash_table_insert(ag->pair_hash, pair, pair);
-	g_tree_insert_coll(ag->all_pairs, pair, pair, __tree_coll_callback);
+	rtpe_g_tree_insert_coll(ag->all_pairs, pair, pair, __tree_coll_callback);
 
 	ilogs(ice, LOG_DEBUG, "Created candidate pair "PAIR_FORMAT" between %s and %s%s%s, type %s", PAIR_FMT(pair),
 			sockaddr_print_buf(&sfd->socket.local.address),
@@ -344,7 +303,7 @@ static struct ice_candidate_pair *__pair_candidate(stream_fd *sfd, struct ice_ag
 }
 
 static unsigned int __pair_hash(const struct ice_candidate_pair *pair) {
-	return g_direct_hash(pair->local_intf) ^ g_direct_hash(pair->remote_candidate);
+	return GPOINTER_TO_UINT(pair->local_intf) ^ GPOINTER_TO_UINT(pair->remote_candidate);
 }
 static int __pair_equal(const struct ice_candidate_pair *A, const struct ice_candidate_pair *B) {
 	return A->local_intf == B->local_intf
@@ -419,14 +378,14 @@ static void __ice_agent_initialize(struct ice_agent *ag) {
 	create_random_ice_string(call, &ag->ufrag[1], 8);
 	create_random_ice_string(call, &ag->pwd[1], 26);
 
-	atomic64_set_na(&ag->last_activity, rtpe_now.tv_sec);
+	atomic64_set_na(&ag->last_activity, rtpe_now);
 }
 
 static struct ice_agent *__ice_agent_new(struct call_media *media) {
 	struct ice_agent *ag;
 	call_t *call = media->call;
 
-	ag = obj_alloc0("ice_agent", sizeof(*ag), __ice_agent_free);
+	ag = obj_alloc0(struct ice_agent, __ice_agent_free);
 	ag->tt_obj.tt = &ice_agents_timer_thread;
 	ag->tt_obj.thread = &ice_agents_timer_thread.threads[0]; // there's only one thread
 	ag->call = obj_get(call);
@@ -451,7 +410,7 @@ void ice_agent_init(struct ice_agent **agp, struct call_media *media) {
 static int __copy_cand(call_t *call, struct ice_candidate *dst, const struct ice_candidate *src) {
 	int eq = (dst->priority == src->priority);
 	*dst = *src;
-	call_str_cpy(call, &dst->foundation, &src->foundation);
+	dst->foundation = call_str_cpy(&src->foundation);
 	return eq ? 0 : 1;
 }
 
@@ -460,8 +419,8 @@ static void __ice_reset(struct ice_agent *ag) {
 	AGENT_CLEAR3(ag, COMPLETED, NOMINATING, USABLE);
 	__ice_agent_free_components(ag);
 	ZERO(ag->active_components);
-	ZERO(ag->start_nominating);
-	ZERO(ag->tt_obj.last_run);
+	ag->start_nominating = 0;
+	ag->tt_obj.last_run = 0;
 	__ice_agent_initialize(ag);
 }
 
@@ -501,7 +460,7 @@ void ice_update(struct ice_agent *ag, struct stream_params *sp, bool allow_reset
 
 	log_info_ice_agent(ag);
 
-	atomic64_set_na(&ag->last_activity, rtpe_now.tv_sec);
+	atomic64_set_na(&ag->last_activity, rtpe_now);
 	media = ag->media;
 	call = media->call;
 
@@ -517,9 +476,9 @@ void ice_update(struct ice_agent *ag, struct stream_params *sp, bool allow_reset
 
 		/* update remote info */
 		if (sp->ice_ufrag.s)
-			call_str_cpy(call, &ag->ufrag[0], &sp->ice_ufrag);
+			ag->ufrag[0] = call_str_cpy(&sp->ice_ufrag);
 		if (sp->ice_pwd.s)
-			call_str_cpy(call, &ag->pwd[0], &sp->ice_pwd);
+			ag->pwd[0] = call_str_cpy(&sp->ice_pwd);
 
 		candidates = &sp->ice_candidates;
 	}
@@ -590,7 +549,7 @@ void ice_update(struct ice_agent *ag, struct stream_params *sp, bool allow_reset
 		else {
 			ilogs(ice, LOG_DEBUG, "Learning new ICE candidate " STR_FORMAT_M ":%lu",
 					STR_FMT_M(&cand->foundation), cand->component_id);
-			dup = g_slice_alloc(sizeof(*dup));
+			dup = g_new(__typeof(*dup), 1);
 			__copy_cand(call, dup, cand);
 			t_hash_table_insert(ag->candidate_hash, dup, dup);
 			t_hash_table_insert(ag->cand_prio_hash, GUINT_TO_POINTER(dup->priority), dup);
@@ -638,13 +597,13 @@ pair:
 
 
 static void ice_candidate_free(struct ice_candidate *p) {
-	g_slice_free1(sizeof(*p), p);
+	g_free(p);
 }
 void ice_candidates_free(candidate_q *q) {
 	t_queue_clear_full(q, ice_candidate_free);
 }
 static void ice_candidate_pair_free(struct ice_candidate_pair *p) {
-	g_slice_free1(sizeof(struct ice_candidate_pair), p);
+	g_free(p);
 }
 static void ice_candidate_pairs_free(candidate_pair_q *q) {
 	t_queue_clear_full(q, ice_candidate_pair_free);
@@ -689,9 +648,7 @@ static void __ice_agent_free_components(struct ice_agent *ag) {
 	ice_candidates_free(&ag->remote_candidates);
 	ice_candidate_pairs_free(&ag->candidate_pairs);
 }
-static void __ice_agent_free(void *p) {
-	struct ice_agent *ag = p;
-
+static void __ice_agent_free(struct ice_agent *ag) {
 	if (!ag) {
 		ilogs(ice, LOG_ERR, "ice ag is NULL");
 		return;
@@ -706,32 +663,32 @@ static void __ice_agent_free(void *p) {
 }
 
 
-static void __agent_schedule(struct ice_agent *ag, unsigned long usec) {
-	struct timeval nxt;
+static void __agent_schedule(struct ice_agent *ag, int64_t usec) {
+	int64_t nxt;
 
 	nxt = rtpe_now;
-	timeval_add_usec(&nxt, usec);
-	__agent_schedule_abs(ag, &nxt);
+	nxt += usec;
+	__agent_schedule_abs(ag, nxt);
 }
-static void __agent_schedule_abs(struct ice_agent *ag, const struct timeval *tv) {
-	struct timeval nxt;
-	long long diff;
+static void __agent_schedule_abs(struct ice_agent *ag, int64_t tv) {
+	int64_t nxt;
+	int64_t diff;
 
 	if (!ag)
 		return;
 
-	nxt = *tv;
+	nxt = tv;
 
 	struct timerthread_thread *tt = ag->tt_obj.thread;
 
 	mutex_lock(&tt->lock);
-	if (ag->tt_obj.last_run.tv_sec) {
+	if (ag->tt_obj.last_run) {
 		/* make sure we don't run more often than we should */
-		diff = timeval_diff(&nxt, &ag->tt_obj.last_run);
+		diff = nxt - ag->tt_obj.last_run;
 		if (diff < TIMER_RUN_INTERVAL * 1000)
-			timeval_add_usec(&nxt, TIMER_RUN_INTERVAL * 1000 - diff);
+			nxt += TIMER_RUN_INTERVAL * 1000 - diff;
 	}
-	timerthread_obj_schedule_abs_nl(&ag->tt_obj, &nxt);
+	timerthread_obj_schedule_abs_nl(&ag->tt_obj, nxt);
 	mutex_unlock(&tt->lock);
 }
 static void __agent_deschedule(struct ice_agent *ag) {
@@ -742,22 +699,10 @@ static void __agent_deschedule(struct ice_agent *ag) {
 void ice_init(void) {
 	random_string((void *) &tie_breaker, sizeof(tie_breaker));
 	timerthread_init(&ice_agents_timer_thread, 1, ice_agents_timer_run);
-
-	sdp_fragments = fragments_ht_new();
-	mutex_init(&sdp_fragments_lock);
 }
 
 void ice_free(void) {
 	timerthread_free(&ice_agents_timer_thread);
-
-	fragments_cleanup(true);
-	t_hash_table_destroy_ptr(&sdp_fragments);
-	mutex_destroy(&sdp_fragments_lock);
-}
-
-enum thread_looper_action ice_slow_timer(void) {
-	fragments_cleanup(false);
-	return TLA_CONTINUE;
 }
 
 static void __fail_pair(struct ice_candidate_pair *pair) {
@@ -801,8 +746,8 @@ static void __do_ice_check(struct ice_candidate_pair *pair) {
 		pair->retransmit_ms *= 2;
 		pair->retransmits++;
 	}
-	timeval_add_usec(&pair->retransmit, pair->retransmit_ms * 1000);
-	__agent_schedule_abs(pair->agent, &pair->retransmit);
+	pair->retransmit += pair->retransmit_ms * 1000; // XXX convert to micro
+	__agent_schedule_abs(pair->agent, pair->retransmit);
 	memcpy(transact, pair->stun_transaction, sizeof(transact));
 
 	pair->was_controlling = AGENT_ISSET(ag, CONTROLLING);
@@ -830,10 +775,10 @@ static int __component_find(const void *a, const void *b) {
 	return FALSE;
 }
 static struct ice_candidate_pair *__get_pair_by_component(GTree *t, unsigned int component) {
-	return g_tree_find_first(t, __component_find, GUINT_TO_POINTER(component));
+	return rtpe_g_tree_find_first(t, __component_find, GUINT_TO_POINTER(component));
 }
 static void __get_pairs_by_component(candidate_pair_q *out, GTree *t, unsigned int component) {
-	g_tree_find_all(&out->q, t, __component_find, GUINT_TO_POINTER(component));
+	rtpe_g_tree_find_all(&out->q, t, __component_find, GUINT_TO_POINTER(component));
 }
 
 static void __get_complete_succeeded_pairs(candidate_pair_q *out, struct ice_agent *ag) {
@@ -850,7 +795,7 @@ static void __nominate_pairs(struct ice_agent *ag) {
 	ilogs(ice, LOG_DEBUG, "Start nominating ICE pairs");
 
 	AGENT_SET(ag, NOMINATING);
-	ZERO(ag->start_nominating);
+	ag->start_nominating = 0;
 
 	__get_complete_succeeded_pairs(&complete, ag);
 
@@ -872,7 +817,7 @@ static void __do_ice_checks(struct ice_agent *ag) {
 	struct ice_candidate_pair *pair, *highest = NULL, *frozen = NULL, *valid;
 	stream_fd *sfd;
 	GQueue retransmits = G_QUEUE_INIT;
-	struct timeval next_run = {0,0};
+	int64_t next_run = 0;
 	int have_more = 0;
 
 	if (!ag) {
@@ -883,7 +828,7 @@ static void __do_ice_checks(struct ice_agent *ag) {
 	if (!ag->pwd[0].s)
 		return;
 
-	atomic64_set_na(&ag->last_activity, rtpe_now.tv_sec);
+	atomic64_set_na(&ag->last_activity, rtpe_now);
 
 	__DBG("running checks, call "STR_FORMAT" tag "STR_FORMAT"", STR_FMT(&ag->call->callid),
 			STR_FMT(&ag->media->monologue->tag));
@@ -891,10 +836,10 @@ static void __do_ice_checks(struct ice_agent *ag) {
 	mutex_lock(&ag->lock);
 
 	/* check if we're done and should start nominating pairs */
-	if (AGENT_ISSET(ag, CONTROLLING) && !AGENT_ISSET(ag, NOMINATING) && ag->start_nominating.tv_sec) {
-		if (timeval_cmp(&rtpe_now, &ag->start_nominating) >= 0)
+	if (AGENT_ISSET(ag, CONTROLLING) && !AGENT_ISSET(ag, NOMINATING) && ag->start_nominating) {
+		if (rtpe_now >= ag->start_nominating)
 			__nominate_pairs(ag);
-		timeval_lowest(&next_run, &ag->start_nominating);
+		next_run = timeval_lowest(next_run, ag->start_nominating);
 	}
 
 	/* triggered checks are preferred */
@@ -929,11 +874,11 @@ static void __do_ice_checks(struct ice_agent *ag) {
 			if (valid && valid->pair_priority > pair->pair_priority)
 				continue;
 
-			if (timeval_cmp(&pair->retransmit, &rtpe_now) <= 0)
+			if (pair->retransmit <= rtpe_now)
 				g_queue_push_tail(&retransmits, pair); /* can't run check directly
 									  due to locks */
 			else
-				timeval_lowest(&next_run, &pair->retransmit);
+				next_run = timeval_lowest(next_run, pair->retransmit);
 			continue;
 		}
 
@@ -982,8 +927,8 @@ check:
 	/* determine when to run next */
 	if (have_more)
 		__agent_schedule(ag, 0);
-	else if (next_run.tv_sec)
-		__agent_schedule_abs(ag, &next_run); /* for retransmits */
+	else if (next_run)
+		__agent_schedule_abs(ag, next_run); /* for retransmits */
 }
 
 static void __agent_shutdown(struct ice_agent *ag) {
@@ -1025,8 +970,8 @@ static void __cand_ice_foundation(call_t *call, struct ice_candidate *cand) {
 	int len;
 
 	len = sprintf(buf, "%x%x%x", endpoint_hash(&cand->endpoint),
-			cand->type, g_direct_hash(cand->transport));
-	call_str_cpy_len(call, &cand->foundation, buf, len);
+			cand->type, GPOINTER_TO_UINT(cand->transport));
+	cand->foundation = call_str_cpy_len(buf, len);
 }
 
 /* agent must be locked */
@@ -1038,7 +983,7 @@ static struct ice_candidate_pair *__learned_candidate(struct ice_agent *ag, stre
 	call_t *call = ag->call;
 	struct packet_stream *ps = sfd->stream;
 
-	cand = g_slice_alloc0(sizeof(*cand));
+	cand = g_new0(__typeof(*cand), 1);
 	cand->component_id = ps->component;
 	cand->transport = sfd->local_intf->spec->local_address.type; // XXX add socket type into socket_t?
 	cand->priority = priority;
@@ -1071,7 +1016,7 @@ static struct ice_candidate_pair *__learned_candidate(struct ice_agent *ag, stre
 		/* this is possible if two distinct requests are received from the same NAT IP
 		 * address, but from different ports. we cannot distinguish such candidates and
 		 * will drop the one with the lower priority */
-		g_slice_free1(sizeof(*cand), cand);
+		g_free(cand);
 		pair = __pair_lookup(ag, old_cand, sfd->local_intf);
 		if (pair)
 			goto out; /* nothing to do */
@@ -1118,10 +1063,10 @@ static void __recalc_pair_prios(struct ice_agent *ag) {
 
 	ilogs(ice, LOG_DEBUG, "Recalculating all ICE pair priorities");
 
-	g_tree_find_remove_all(&nominated, ag->nominated_pairs);
-	g_tree_find_remove_all(&succ, ag->succeeded_pairs);
-	g_tree_find_remove_all(&valid, ag->valid_pairs);
-	g_tree_find_remove_all(&all, ag->all_pairs);
+	rtpe_g_tree_find_remove_all(&nominated, ag->nominated_pairs);
+	rtpe_g_tree_find_remove_all(&succ, ag->succeeded_pairs);
+	rtpe_g_tree_find_remove_all(&valid, ag->valid_pairs);
+	rtpe_g_tree_find_remove_all(&all, ag->all_pairs);
 
 	for (__auto_type l = ag->candidate_pairs.head; l; l = l->next) {
 		pair = l->data;
@@ -1130,10 +1075,10 @@ static void __recalc_pair_prios(struct ice_agent *ag) {
 		__new_stun_transaction(pair);
 	}
 
-	g_tree_add_all(ag->nominated_pairs, &nominated, __tree_coll_callback);
-	g_tree_add_all(ag->succeeded_pairs, &succ, __tree_coll_callback);
-	g_tree_add_all(ag->valid_pairs, &valid, __tree_coll_callback);
-	g_tree_add_all(ag->all_pairs, &all, __tree_coll_callback);
+	rtpe_g_tree_add_all(ag->nominated_pairs, &nominated, __tree_coll_callback);
+	rtpe_g_tree_add_all(ag->succeeded_pairs, &succ, __tree_coll_callback);
+	rtpe_g_tree_add_all(ag->valid_pairs, &valid, __tree_coll_callback);
+	rtpe_g_tree_add_all(ag->all_pairs, &all, __tree_coll_callback);
 	__all_pairs_list(ag);
 }
 
@@ -1288,14 +1233,15 @@ int ice_request(stream_fd *sfd, const endpoint_t *src,
 	struct ice_candidate_pair *pair;
 	int ret;
 
-	__DBG("received ICE request from %s on %s", endpoint_print_buf(src),
+	ilogs(ice, LOG_DEBUG, "Received ICE/STUN request from %s on %s",
+			endpoint_print_buf(src),
 			endpoint_print_buf(&sfd->socket.local));
 
 	ag = media->ice_agent;
 	if (!ag)
 		return -1;
 
-	atomic64_set_na(&ag->last_activity, rtpe_now.tv_sec);
+	atomic64_set_na(&ag->last_activity, rtpe_now);
 
 	/* determine candidate pair */
 	{
@@ -1313,6 +1259,8 @@ int ice_request(stream_fd *sfd, const endpoint_t *src,
 			goto err;
 	}
 
+	PAIR_SET(pair, AUTHENTICATED);
+
 	if (!AGENT_ISSET(ag, LITE_SELF)) {
 		/* determine role conflict */
 		if (attrs->controlling && AGENT_ISSET(ag, CONTROLLING)) {
@@ -1345,11 +1293,11 @@ int ice_request(stream_fd *sfd, const endpoint_t *src,
 		LOCK(&ag->lock);
 
 		// coverity[use : FALSE]
-		g_tree_insert_coll(ag->nominated_pairs, pair, pair, __tree_coll_callback);
+		rtpe_g_tree_insert_coll(ag->nominated_pairs, pair, pair, __tree_coll_callback);
 
 		if (PAIR_ISSET(pair, SUCCEEDED)) {
 			PAIR_SET(pair, VALID);
-			g_tree_insert_coll(ag->valid_pairs, pair, pair, __tree_coll_callback);
+			rtpe_g_tree_insert_coll(ag->valid_pairs, pair, pair, __tree_coll_callback);
 		}
 
 		if (!AGENT_ISSET(ag, CONTROLLING))
@@ -1404,7 +1352,7 @@ int ice_response(stream_fd *sfd, const endpoint_t *src,
 	if (!ag)
 		return -1;
 
-	atomic64_set_na(&ag->last_activity, rtpe_now.tv_sec);
+	atomic64_set_na(&ag->last_activity, rtpe_now);
 
 	{
 		LOCK(&ag->lock);
@@ -1445,6 +1393,8 @@ int ice_response(stream_fd *sfd, const endpoint_t *src,
 		goto out;
 	}
 
+	PAIR_SET(pair, AUTHENTICATED);
+
 	/* we don't discover peer reflexive here (RFC 5245 7.1.3.2.1) as we don't expect to be behind NAT */
 	/* we also skip parts of 7.1.3.2.2 as we don't do server reflexive */
 
@@ -1455,7 +1405,7 @@ int ice_response(stream_fd *sfd, const endpoint_t *src,
 		if (pair->was_nominated && PAIR_CLEAR(pair, TO_USE)) {
 			ilogs(ice, LOG_DEBUG, "Setting nominated ICE candidate pair "PAIR_FORMAT" as valid", PAIR_FMT(pair));
 			PAIR_SET(pair, VALID);
-			g_tree_insert_coll(ag->valid_pairs, pair, pair, __tree_coll_callback);
+			rtpe_g_tree_insert_coll(ag->valid_pairs, pair, pair, __tree_coll_callback);
 			ret = __check_valid(ag);
 			goto out;
 		}
@@ -1464,13 +1414,13 @@ int ice_response(stream_fd *sfd, const endpoint_t *src,
 			goto out;
 
 		ilogs(ice, LOG_DEBUG, "Setting ICE candidate pair "PAIR_FORMAT" as succeeded", PAIR_FMT(pair));
-		g_tree_insert_coll(ag->succeeded_pairs, pair, pair, __tree_coll_callback);
+		rtpe_g_tree_insert_coll(ag->succeeded_pairs, pair, pair, __tree_coll_callback);
 
-		if (!ag->start_nominating.tv_sec) {
+		if (!ag->start_nominating) {
 			if (__check_succeeded_complete(ag)) {
 				ag->start_nominating = rtpe_now;
-				timeval_add_usec(&ag->start_nominating, 100000);
-				__agent_schedule_abs(ag, &ag->start_nominating);
+				ag->start_nominating += 100000;
+				__agent_schedule_abs(ag, ag->start_nominating);
 			}
 		}
 
@@ -1496,7 +1446,7 @@ int ice_response(stream_fd *sfd, const endpoint_t *src,
 		/* if this was previously nominated by the peer, it's now valid */
 		if (PAIR_ISSET(pair, NOMINATED)) {
 			PAIR_SET(pair, VALID);
-			g_tree_insert_coll(ag->valid_pairs, pair, pair, __tree_coll_callback);
+			rtpe_g_tree_insert_coll(ag->valid_pairs, pair, pair, __tree_coll_callback);
 		}
 
 		ret = __check_valid(ag);
@@ -1550,11 +1500,11 @@ static void create_random_ice_string(call_t *call, str *s, int len) {
 		return;
 
 	random_ice_string(buf, len);
-	call_str_cpy_len(call, s, buf, len);
+	*s = call_str_cpy_len(buf, len);
 }
 
 void ice_foundation(str *s) {
-	str_init_len(s, malloc(ICE_FOUNDATION_LENGTH), ICE_FOUNDATION_LENGTH);
+	*s = STR_LEN(malloc(ICE_FOUNDATION_LENGTH), ICE_FOUNDATION_LENGTH);
 	random_ice_string(s->s, ICE_FOUNDATION_LENGTH);
 }
 
@@ -1587,7 +1537,7 @@ bool ice_peer_address_known(struct ice_agent *ag, const endpoint_t *sin, struct
 	struct ice_candidate_pair *pair = __pair_lookup(ag, cand, ifa);
 	if (!pair)
 		return false;
-	if (!PAIR_ISSET(pair, VALID))
+	if (!PAIR_ISSET(pair, AUTHENTICATED))
 		return false;
 
 	return true;
