@@ -17,7 +17,6 @@
 #include "call.h"
 #include "main.h"
 #include "kernel.h"
-#include "bencode.h"
 #include "rtplib.h"
 #include "cdr.h"
 #include "log.h"
@@ -46,23 +45,23 @@ static int vappend_meta_chunk(struct recording *recording, const char *buf, unsi
 // all methods
 static int create_spool_dir_all(const char *spoolpath);
 static void init_all(call_t *call);
-static void sdp_after_all(struct recording *recording, GString *str, struct call_monologue *ml,
-		enum call_opmode opmode);
+static void sdp_after_all(struct recording *recording, const str *str, struct call_monologue *ml,
+		enum ng_opmode opmode);
 static void dump_packet_all(struct media_packet *mp, const str *s);
 static void finish_all(call_t *call, bool discard);
 
 // pcap methods
 static int rec_pcap_create_spool_dir(const char *dirpath);
 static void rec_pcap_init(call_t *);
-static void sdp_after_pcap(struct recording *, GString *str, struct call_monologue *, enum call_opmode opmode);
+static void sdp_after_pcap(struct recording *, const str *str, struct call_monologue *, enum ng_opmode opmode);
 static void dump_packet_pcap(struct media_packet *mp, const str *s);
 static void finish_pcap(call_t *, bool discard);
-static void response_pcap(struct recording *, bencode_item_t *);
+static void response_pcap(struct recording *, const ng_parser_t *, parser_arg);
 
 // proc methods
 static void proc_init(call_t *);
-static void sdp_before_proc(struct recording *, const str *, struct call_monologue *, enum call_opmode);
-static void sdp_after_proc(struct recording *, GString *str, struct call_monologue *, enum call_opmode opmode);
+static void sdp_before_proc(struct recording *, const str *, struct call_monologue *, enum ng_opmode);
+static void sdp_after_proc(struct recording *, const str *sdp, struct call_monologue *, enum ng_opmode opmode);
 static void meta_chunk_proc(struct recording *, const char *, const str *);
 static void update_flags_proc(call_t *call, bool streams);
 static void finish_proc(call_t *, bool discard);
@@ -277,7 +276,7 @@ static void update_call_field(call_t *call, str *dst_field, const str *src_field
 		return;
 
 	if (src_field && src_field->len && str_cmp_str(src_field, dst_field))
-		call_str_cpy(call, dst_field, src_field);
+		*dst_field = call_str_cpy(src_field);
 
 	if (call->recording && dst_field->len) {
 		va_list ap;
@@ -302,6 +301,9 @@ void update_metadata_call(call_t *call, const sdp_ng_flags *flags) {
 	update_call_field(call, &call->recording_path, flags ? &flags->recording_path : NULL, "RECORDING_PATH");
 	update_call_field(call, &call->recording_pattern, flags ? &flags->recording_pattern : NULL,
 			"RECORDING_PATTERN");
+
+	if (call->recording)
+		append_meta_chunk_null(call->recording, "STARTED");
 }
 
 // lock must be held
@@ -362,15 +364,15 @@ void recording_start_daemon(call_t *call) {
 	}
 	ilog(LOG_NOTICE, "Turning on call recording.");
 
-	call->recording = g_slice_alloc0(sizeof(struct recording));
+	call->recording = g_new0(struct recording, 1);
 	g_autoptr(char) escaped_callid = g_uri_escape_string(call->callid.s, NULL, 0);
 	if (!call->recording_meta_prefix.len) {
 		const int rand_bytes = 8;
 		char rand_str[rand_bytes * 2 + 1];
 		rand_hex_str(rand_str, rand_bytes);
 		g_autoptr(char) meta_prefix = g_strdup_printf("%s-%s", escaped_callid, rand_str);
-		call_str_cpy(call, &call->recording_meta_prefix, &STR_INIT(meta_prefix));
-		call_str_cpy(call, &call->recording_random_tag, &STR_CONST_INIT(rand_str));
+		call->recording_meta_prefix = call_str_cpy(STR_PTR(meta_prefix));
+		call->recording_random_tag = call_str_cpy(&STR_CONST(rand_str));
 	}
 
 	_rm(init_struct, call);
@@ -403,7 +405,22 @@ void recording_start_daemon(call_t *call) {
 }
 // lock must be held
 void recording_start(call_t *call) {
+	bool was_paused = call->recording && !CALL_ISSET(call, RECORDING_ON);
 	CALL_SET(call, RECORDING_ON);
+
+	// Write resume timestamp to metadata file if resuming a paused recording
+	if (was_paused && call->recording->pcap.meta_fp) {
+		time_t now = rtpe_now / 1000000;
+		char timebuffer[32];
+		struct tm timeinfo;
+		if (localtime_r(&now, &timeinfo) != NULL) {
+			strftime(timebuffer, sizeof(timebuffer), "%FT%T", &timeinfo);
+			fprintf(call->recording->pcap.meta_fp,
+				"\nRecording resumed at: %s (%.3lf ms)\n",
+				timebuffer, rtpe_now / 1000.0);
+		}
+	}
+
 	recording_start_daemon(call);
 }
 // lock must be held
@@ -439,6 +456,20 @@ void recording_pause(call_t *call) {
 	if (!call->recording)
 		return;
 	ilog(LOG_NOTICE, "Pausing call recording.");
+
+	// Write pause timestamp to metadata file
+	if (call->recording->pcap.meta_fp) {
+		time_t now = rtpe_now / 1000000;
+		char timebuffer[32];
+		struct tm timeinfo;
+		if (localtime_r(&now, &timeinfo) != NULL) {
+			strftime(timebuffer, sizeof(timebuffer), "%FT%T", &timeinfo);
+			fprintf(call->recording->pcap.meta_fp,
+				"\nRecording paused at: %s (%.3lf ms)\n",
+				timebuffer, rtpe_now / 1000.0);
+		}
+	}
+
 	recording_update_flags(call, true);
 }
 // lock must be held
@@ -467,8 +498,12 @@ void detect_setup_recording(call_t *call, const sdp_ng_flags *flags) {
 
 	const str *recordcall = &flags->record_call_str;
 
-	if (!str_cmp(recordcall, "yes") || !str_cmp(recordcall, "on") || flags->record_call)
-		recording_start(call);
+	if (!str_cmp(recordcall, "yes") || !str_cmp(recordcall, "on") || flags->record_call) {
+		// Don't auto-start recording if it's currently paused
+		// Only start if there's no existing recording or if recording is not paused
+		if (!call->recording || CALL_ISSET(call, RECORDING_ON))
+			recording_start(call);
+	}
 	else if (!str_cmp(recordcall, "no") || !str_cmp(recordcall, "off"))
 		recording_stop(call);
 	else if (!str_cmp(recordcall, "discard") || flags->discard_recording)
@@ -518,6 +553,7 @@ static char *meta_setup_file(struct recording *recording, const str *meta_prefix
 		g_clear_pointer(&recording->pcap.meta_filepath, free);
 		return NULL;
 	}
+	setvbuf(mfp, NULL, _IONBF, 0);
 	recording->pcap.meta_fp = mfp;
 	ilog(LOG_DEBUG, "Wrote metadata file to temporary path: %s%s%s", FMT_M(meta_filepath));
 	return meta_filepath;
@@ -526,27 +562,25 @@ static char *meta_setup_file(struct recording *recording, const str *meta_prefix
 /**
  * Write out a block of SDP to the metadata file.
  */
-static void sdp_after_pcap(struct recording *recording, GString *s, struct call_monologue *ml,
-		enum call_opmode opmode)
+static void sdp_after_pcap(struct recording *recording, const str *s, struct call_monologue *ml,
+		enum ng_opmode opmode)
 {
+	if (!recording)
+		return;
+
 	FILE *meta_fp = recording->pcap.meta_fp;
 	if (!meta_fp)
 		return;
 
-	int meta_fd = fileno(meta_fp);
-	// File pointers buffer data, whereas direct writing using the file
-	// descriptor does not. Make sure to flush any unwritten contents
-	// so the file contents appear in order.
 	if (ml->label.len) {
 		fprintf(meta_fp, "\nLabel: " STR_FORMAT, STR_FMT(&ml->label));
 	}
 	fprintf(meta_fp, "\nTimestamp started ms: ");
-	fprintf(meta_fp, "%.3lf", ml->started.tv_sec*1000.0+ml->started.tv_usec/1000.0);
+	fprintf(meta_fp, "%.3lf", ml->started / 1000.);
 	fprintf(meta_fp, "\nSDP mode: ");
 	fprintf(meta_fp, "%s", get_opmode_text(opmode));
 	fprintf(meta_fp, "\nSDP before RTP packet: %" PRIu64 "\n\n", recording->pcap.packet_num);
-	fflush(meta_fp);
-	if (write(meta_fd, s->str, s->len) <= 0)
+	if (fwrite(s->s, s->len, 1, meta_fp) < 1)
 		ilog(LOG_WARN, "Error writing SDP body to metadata file: %s", strerror(errno));
 }
 
@@ -564,13 +598,13 @@ static void rec_pcap_meta_finish_file(call_t *call) {
 
 	// Print start timestamp and end timestamp
 	// YYYY-MM-DDThh:mm:ss
-	time_t start = call->created.tv_sec;
-	time_t end = rtpe_now.tv_sec;
+	time_t start = call->created / 1000000;
+	time_t end = rtpe_now / 1000000;
 	char timebuffer[20];
 	struct tm timeinfo;
-	struct timeval *terminate;
-	terminate = &(((struct call_monologue *)call->monologues.head->data)->terminated);
-	fprintf(recording->pcap.meta_fp, "\nTimestamp terminated ms(first monologue): %.3lf", terminate->tv_sec*1000.0 + terminate->tv_usec/1000.0);
+	int64_t terminate;
+	terminate = (((struct call_monologue *)call->monologues.head->data)->terminated);
+	fprintf(recording->pcap.meta_fp, "\nTimestamp terminated ms(first monologue): %.3lf", terminate / 1000.);
 	if (localtime_r(&start, &timeinfo) == NULL) {
 		ilog(LOG_ERROR, "Cannot get start local time, while cleaning up recording meta file: %s", strerror(errno));
 	} else {
@@ -728,7 +762,7 @@ static void stream_pcap_dump(struct media_packet *mp, const str *s) {
 	// Set up PCAP packet header
 	struct pcap_pkthdr header;
 	ZERO(header);
-	header.ts = rtpe_now;
+	header.ts = timeval_from_us(rtpe_now);
 	header.caplen = pkt_len;
 	header.len = pkt_len;
 
@@ -740,6 +774,8 @@ static void stream_pcap_dump(struct media_packet *mp, const str *s) {
 static void dump_packet_pcap(struct media_packet *mp, const str *s) {
 	if (ML_ISSET(mp->media->monologue, NO_RECORDING))
 		return;
+	if (!CALL_ISSET(mp->call, RECORDING_ON))
+		return;
 	struct recording *recording = mp->call->recording;
 	mutex_lock(&recording->pcap.recording_lock);
 	stream_pcap_dump(mp, s);
@@ -755,12 +791,14 @@ static void finish_pcap(call_t *call, bool discard) {
 		rec_pcap_meta_discard_file(call);
 }
 
-static void response_pcap(struct recording *recording, bencode_item_t *output) {
+static void response_pcap(struct recording *recording, const ng_parser_t *parser, parser_arg output) {
+	if (!recording)
+		return;
 	if (!recording->pcap.recording_path)
 		return;
 
-	bencode_item_t *recordings = bencode_dictionary_add_list(output, "recordings");
-	bencode_list_add_string(recordings, recording->pcap.recording_path);
+	parser_arg recordings = parser->dict_add_list(output, "recordings");
+	parser->list_add_string(recordings, recording->pcap.recording_path);
 }
 
 
@@ -779,7 +817,7 @@ void recording_finish(call_t *call, bool discard) {
 
 	_rm(finish, call, discard);
 
-	g_slice_free1(sizeof(*(recording)), recording);
+	g_free(recording);
 	call->recording = NULL;
 }
 
@@ -883,16 +921,22 @@ static void proc_init(call_t *call) {
 }
 
 static void sdp_before_proc(struct recording *recording, const str *sdp, struct call_monologue *ml,
-		enum call_opmode opmode)
+		enum ng_opmode opmode)
 {
+	if (!recording)
+		return;
+
 	append_meta_chunk_str(recording, sdp,
 			"SDP from %u before %s", ml->unique_id, get_opmode_text(opmode));
 }
 
-static void sdp_after_proc(struct recording *recording, GString *s, struct call_monologue *ml,
-		enum call_opmode opmode)
+static void sdp_after_proc(struct recording *recording, const str *sdp, struct call_monologue *ml,
+		enum ng_opmode opmode)
 {
-	append_meta_chunk(recording, s->str, s->len,
+	if (!recording)
+		return;
+
+	append_meta_chunk_str(recording, sdp,
 			"SDP from %u after %s", ml->unique_id, get_opmode_text(opmode));
 }
 
@@ -941,6 +985,8 @@ static void setup_stream_proc(struct packet_stream *stream) {
 	struct recording *recording = call->recording;
 	char buf[128];
 	int len;
+	unsigned int media_rec_slot;
+	unsigned int media_rec_slots;
 
 	if (!recording)
 		return;
@@ -951,9 +997,26 @@ static void setup_stream_proc(struct packet_stream *stream) {
 	if (ML_ISSET(ml, NO_RECORDING))
 		return;
 
-	len = snprintf(buf, sizeof(buf), "TAG %u MEDIA %u TAG-MEDIA %u COMPONENT %u FLAGS %" PRIu64 " MEDIA-SDP-ID %i",
+	ilog(LOG_INFO, "media_rec_slot=%u, media_rec_slots=%u, stream=%u", media->media_rec_slot, call->media_rec_slots, stream->unique_id);
+
+	// If no slots have been specified or someone has tried to use slott 0 then we set the variables up so that the mix
+	// channels will be used in sequence as each SSRC is seen. (see mix.c for the algorithm)
+	if(call->media_rec_slots < 1 || media->media_rec_slot < 1) {
+		media_rec_slot = 1;
+		media_rec_slots = 1;
+	} else {
+		media_rec_slot = media->media_rec_slot;
+		media_rec_slots = call->media_rec_slots;
+	}
+
+	if(media_rec_slot > media_rec_slots) {
+		ilog(LOG_ERR, "slot %i is greater than the total number of slots available %i, setting to slot %i", media->media_rec_slot, call->media_rec_slots, media_rec_slots);
+		media_rec_slot = media_rec_slots;
+	}
+
+	len = snprintf(buf, sizeof(buf), "TAG %u MEDIA %u TAG-MEDIA %u COMPONENT %u FLAGS %" PRIu64 " MEDIA-SDP-ID %i MEDIA-REC-SLOT %i MEDIA-REC-SLOTS %i",
 				   ml->unique_id, media->unique_id, media->index, stream->component,
-				   atomic64_get_na(&stream->ps_flags), media->media_sdp_id);
+				   atomic64_get_na(&stream->ps_flags), media->media_sdp_id, media_rec_slot, media_rec_slots);
 	append_meta_chunk(recording, buf, len, "STREAM %u details", stream->unique_id);
 
 	len = snprintf(buf, sizeof(buf), "tag-%u-media-%u-component-%u-%s-id-%u",
@@ -1011,6 +1074,8 @@ static void dump_packet_proc(struct media_packet *mp, const str *s) {
 	struct packet_stream *stream = mp->stream;
 	if (stream->recording.proc.stream_idx == UNINIT_IDX)
 		return;
+	if (!CALL_ISSET(mp->call, RECORDING_ON))
+		return;
 
 	struct rtpengine_command_packet *cmd;
 	unsigned char pkt[sizeof(*cmd) + s->len + MAX_PACKET_HEADER_LEN];
@@ -1060,8 +1125,8 @@ static void init_all(call_t *call) {
 	proc_init(call);
 }
 
-static void sdp_after_all(struct recording *recording, GString *s, struct call_monologue *ml,
-		enum call_opmode opmode)
+static void sdp_after_all(struct recording *recording, const str *s, struct call_monologue *ml,
+		enum ng_opmode opmode)
 {
 	sdp_after_pcap(recording, s, ml, opmode);
 	sdp_after_proc(recording, s, ml, opmode);
