@@ -269,8 +269,10 @@ struct rtcp_handler {
 	void (*xr_rr_time)(struct rtcp_process_ctx *, const struct xr_rb_rr_time *);
 	void (*xr_voip_metrics)(struct rtcp_process_ctx *, const struct xr_rb_voip_metrics *);
 	void (*finish)(struct rtcp_process_ctx *, call_t *, const endpoint_t *, const endpoint_t *,
-			const struct timeval *);
+			int64_t);
 	void (*destroy)(struct rtcp_process_ctx *);
+
+	bool no_kernel_fw;
 };
 // collection of all handler types
 struct rtcp_handlers {
@@ -282,6 +284,9 @@ struct rtcp_handlers {
 		*homer;
 };
 
+TYPED_GQUEUE(ssrc, struct ssrc_entry_call)
+TYPED_GQUEUE(ssrc_rr, struct ssrc_receiver_report)
+
 // log handler function prototypes
 
 // scratch area (prepare/parse packet)
@@ -323,7 +328,7 @@ static void homer_sdes_item(struct rtcp_process_ctx *, const struct sdes_chunk *
 		const char *);
 static void homer_sdes_list_end(struct rtcp_process_ctx *);
 static void homer_finish(struct rtcp_process_ctx *, call_t *, const endpoint_t *, const endpoint_t *,
-		const struct timeval *);
+		int64_t);
 
 // syslog functions
 static void logging_init(struct rtcp_process_ctx *);
@@ -338,7 +343,7 @@ static void logging_xr_dlrr(struct rtcp_process_ctx *, const struct xr_rb_dlrr *
 static void logging_xr_stats(struct rtcp_process_ctx *, const struct xr_rb_stats *);
 static void logging_xr_voip_metrics(struct rtcp_process_ctx *, const struct xr_rb_voip_metrics *);
 static void logging_finish(struct rtcp_process_ctx *, call_t *, const endpoint_t *, const endpoint_t *,
-		const struct timeval *);
+		int64_t);
 static void logging_destroy(struct rtcp_process_ctx *);
 
 // structs for each handler type
@@ -362,9 +367,11 @@ static struct rtcp_handler transcode_handlers = {
 	.common = transcode_common,
 	.rr = transcode_rr,
 	.sr = transcode_sr,
+	.no_kernel_fw = true,
 };
 static struct rtcp_handler sink_handlers = {
 	.common = sink_common,
+	.no_kernel_fw = true,
 };
 static struct rtcp_handler transcode_handlers_wrap = {
 	.common = transcode_common_wrap,
@@ -506,7 +513,7 @@ static struct rtcp_header *rtcp_length_check(str *s, size_t min_len, unsigned in
 static struct rtcp_chain_element *rtcp_new_element(struct rtcp_header *p, unsigned int len) {
 	struct rtcp_chain_element *el;
 
-	el = g_slice_alloc(sizeof(*el));
+	el = g_new(__typeof(*el), 1);
 	el->type = p->pt;
 	el->len = len;
 	el->buf = p;
@@ -556,7 +563,7 @@ static int rtcp_rr(struct rtcp_chain_element *el, struct rtcp_process_ctx *log_c
 static int rtcp_sdes(struct rtcp_chain_element *el, struct rtcp_process_ctx *log_ctx) {
 	CAH(sdes_list_start, el->sdes);
 
-	str comp_s = STR_INIT_LEN(el->sdes->chunks, el->len - sizeof(el->sdes->header));
+	str comp_s = STR_LEN(el->sdes->chunks, el->len - sizeof(el->sdes->header));
 	int i = 0;
 	while (1) {
 		struct sdes_chunk *sdes_chunk = (struct sdes_chunk *) comp_s.s;
@@ -614,7 +621,7 @@ static void xr_voip_metrics(struct xr_rb_voip_metrics *rb, struct rtcp_process_c
 
 static int rtcp_xr(struct rtcp_chain_element *el, struct rtcp_process_ctx *log_ctx) {
 	CAH(common, el->rtcp_packet);
-	str comp_s = STR_INIT_LEN(el->buf + sizeof(el->xr->rtcp), el->len - sizeof(el->xr->rtcp));
+	str comp_s = STR_LEN(el->buf + sizeof(el->xr->rtcp), el->len - sizeof(el->xr->rtcp));
 	while (1) {
 		struct xr_report_block *rb = (void *) comp_s.s;
 		if (comp_s.len < sizeof(*rb))
@@ -644,7 +651,7 @@ next:
 
 
 static void rtcp_ce_free(void *p) {
-	g_slice_free1(sizeof(struct rtcp_chain_element), p);
+	g_free(p);
 }
 void rtcp_list_free(GQueue *q) {
 	g_queue_clear_full(q, rtcp_ce_free);
@@ -718,13 +725,13 @@ next:
 			abort();
 	}
 
-	CAH(finish, c, &mp->fsin, &mp->sfd->socket.local, &mp->tv);
+	CAH(finish, c, &mp->fsin, &mp->sfd->socket.local, mp->tv);
 	CAH(destroy);
 
 	return log_ctx->discard ? 1 : 0;
 
 error:
-	CAH(finish, c, &mp->fsin, &mp->sfd->socket.local, &mp->tv);
+	CAH(finish, c, &mp->fsin, &mp->sfd->socket.local, mp->tv);
 	CAH(destroy);
 	rtcp_list_free(q);
 	return -1;
@@ -774,13 +781,13 @@ INLINE int check_session_keys(struct crypto_context *c) {
 		goto error;
 
 	err = "Failed to generate SRTCP session keys";
-	str_init_len_assert(&s, c->session_key, c->params.crypto_suite->session_key_len);
+	s = STR_LEN_ASSERT(c->session_key, c->params.crypto_suite->session_key_len);
 	if (crypto_gen_session_key(c, &s, 0x03, SRTCP_R_LENGTH))
 		goto error;
-	str_init_len_assert(&s, c->session_auth_key, c->params.crypto_suite->srtcp_auth_key_len);
+	s = STR_LEN_ASSERT(c->session_auth_key, c->params.crypto_suite->srtcp_auth_key_len);
 	if (crypto_gen_session_key(c, &s, 0x04, SRTCP_R_LENGTH))
 		goto error;
-	str_init_len_assert(&s, c->session_salt, c->params.crypto_suite->session_salt_len);
+	s = STR_LEN_ASSERT(c->session_salt, c->params.crypto_suite->session_salt_len);
 	if (crypto_gen_session_key(c, &s, 0x05, SRTCP_R_LENGTH))
 		goto error;
 
@@ -840,7 +847,7 @@ error:
 }
 
 /* rfc 3711 section 3.4 */
-int rtcp_avp2savp(str *s, struct crypto_context *c, struct ssrc_ctx *ssrc_ctx) {
+int rtcp_avp2savp(str *s, struct crypto_context *c, struct ssrc_entry_call *ssrc_ctx) {
 	struct rtcp_packet *rtcp;
 	unsigned int i;
 	uint32_t *idx;
@@ -854,18 +861,18 @@ int rtcp_avp2savp(str *s, struct crypto_context *c, struct ssrc_ctx *ssrc_ctx) {
 		return -1;
 
 	i = atomic_get_na(&ssrc_ctx->stats->rtcp_seq);
-	crypto_debug_init(1);
-	crypto_debug_printf("RTCP SSRC %" PRIx32 ", idx %u, plain pl: ",
+	g_autoptr(crypto_debug_string) cds = crypto_debug_init(true);
+	crypto_debug_printf(cds, "RTCP SSRC %" PRIx32 ", idx %u, plain pl: ",
 			rtcp->ssrc, i);
-	crypto_debug_dump(&payload);
+	crypto_debug_dump(cds, &payload);
 
 	int prev_len = payload.len;
 	if (!c->params.session_params.unencrypted_srtcp && crypto_encrypt_rtcp(c, rtcp, &payload, i))
 		return -1;
 	s->len += payload.len - prev_len;
 
-	crypto_debug_printf(", enc pl: ");
-	crypto_debug_dump(&payload);
+	crypto_debug_printf(cds, ", enc pl: ");
+	crypto_debug_dump(cds, &payload);
 
 	idx = (void *) s->s + s->len;
 	*idx = htonl((c->params.session_params.unencrypted_srtcp ? 0ULL : 0x80000000ULL) | i);
@@ -874,23 +881,21 @@ int rtcp_avp2savp(str *s, struct crypto_context *c, struct ssrc_ctx *ssrc_ctx) {
 
 	to_auth = *s;
 
-	rtp_append_mki(s, c);
+	rtp_append_mki(s, c, cds);
 
 	if (c->params.crypto_suite->srtcp_auth_tag) {
 		c->params.crypto_suite->hash_rtcp(c, s->s + s->len, &to_auth);
-		crypto_debug_printf(", auth: ");
-		crypto_debug_dump_raw(s->s + s->len, c->params.crypto_suite->srtcp_auth_tag);
+		crypto_debug_printf(cds, ", auth: ");
+		crypto_debug_dump_raw(cds, s->s + s->len, c->params.crypto_suite->srtcp_auth_tag);
 		s->len += c->params.crypto_suite->srtcp_auth_tag;
 	}
 
-	crypto_debug_finish();
-
 	return 1;
 }
 
 
 /* rfc 3711 section 3.4 */
-int rtcp_savp2avp(str *s, struct crypto_context *c, struct ssrc_ctx *ssrc_ctx) {
+int rtcp_savp2avp(str *s, struct crypto_context *c, struct ssrc_entry_call *ssrc_ctx) {
 	struct rtcp_packet *rtcp;
 	str payload, to_auth, to_decrypt, auth_tag;
 	uint32_t idx;
@@ -904,16 +909,16 @@ int rtcp_savp2avp(str *s, struct crypto_context *c, struct ssrc_ctx *ssrc_ctx) {
 	if (check_session_keys(c))
 		return -1;
 
-	crypto_debug_init(1);
+	g_autoptr(crypto_debug_string) cds = crypto_debug_init(true);
 
 	if (srtp_payloads(&to_auth, &to_decrypt, &auth_tag, NULL,
 			c->params.crypto_suite->srtcp_auth_tag, c->params.mki_len,
 			s, &payload))
 		return -1;
 
-	crypto_debug_printf("RTCP SSRC %" PRIx32 ", enc pl: ",
+	crypto_debug_printf(cds, "RTCP SSRC %" PRIx32 ", enc pl: ",
 			rtcp->ssrc);
-	crypto_debug_dump(&to_decrypt);
+	crypto_debug_dump(cds, &to_decrypt);
 
 	err = "short packet";
 	if (to_decrypt.len < sizeof(idx))
@@ -922,7 +927,7 @@ int rtcp_savp2avp(str *s, struct crypto_context *c, struct ssrc_ctx *ssrc_ctx) {
 	memcpy(&idx, to_decrypt.s + to_decrypt.len, sizeof(idx));
 	idx = ntohl(idx);
 
-	crypto_debug_printf(", idx %" PRIu32, idx);
+	crypto_debug_printf(cds, ", idx %" PRIu32, idx);
 
 	if (!auth_tag.len)
 		goto decrypt;
@@ -931,10 +936,10 @@ int rtcp_savp2avp(str *s, struct crypto_context *c, struct ssrc_ctx *ssrc_ctx) {
 	assert(sizeof(hmac) >= auth_tag.len);
 	c->params.crypto_suite->hash_rtcp(c, hmac, &to_auth);
 
-	crypto_debug_printf(", rcv hmac: ");
-	crypto_debug_dump(&auth_tag);
-	crypto_debug_printf(", calc hmac: ");
-	crypto_debug_dump_raw(hmac, auth_tag.len);
+	crypto_debug_printf(cds, ", rcv hmac: ");
+	crypto_debug_dump(cds, &auth_tag);
+	crypto_debug_printf(cds, ", calc hmac: ");
+	crypto_debug_dump_raw(cds, hmac, auth_tag.len);
 
 	err = "authentication failed";
 	if (str_memcmp(&auth_tag, hmac))
@@ -946,16 +951,14 @@ decrypt:;
 		if (crypto_decrypt_rtcp(c, rtcp, &to_decrypt, idx & 0x7fffffffULL))
 			return -1;
 
-		crypto_debug_printf(", dec pl: ");
-		crypto_debug_dump(&to_decrypt);
+		crypto_debug_printf(cds, ", dec pl: ");
+		crypto_debug_dump(cds, &to_decrypt);
 	}
 
 	*s = to_auth;
 	s->len -= sizeof(idx);
 	s->len -= prev_len - to_decrypt.len;
 
-	crypto_debug_finish();
-
 	return 0;
 
 error:
@@ -1128,7 +1131,7 @@ static void homer_sdes_list_end(struct rtcp_process_ctx *ctx) {
 	g_string_append_printf(ctx->json, "],");
 }
 static void homer_finish(struct rtcp_process_ctx *ctx, call_t *c, const endpoint_t *src,
-		const endpoint_t *dst, const struct timeval *tv)
+		const endpoint_t *dst, int64_t tv)
 {
 	str_sanitize(ctx->json);
 	g_string_append(ctx->json, " }");
@@ -1256,7 +1259,7 @@ static void logging_xr_voip_metrics(struct rtcp_process_ctx *ctx, const struct x
 			ctx->scratch.xr_vm.jb_abs_max);
 }
 static void logging_finish(struct rtcp_process_ctx *ctx, call_t *c, const endpoint_t *src,
-		const endpoint_t *dst, const struct timeval *tv)
+		const endpoint_t *dst, int64_t tv)
 {
 	str_sanitize(ctx->log);
 	if (ctx->log->len > ctx->log_init_len)
@@ -1270,19 +1273,19 @@ static void logging_destroy(struct rtcp_process_ctx *ctx) {
 
 
 static void mos_sr(struct rtcp_process_ctx *ctx, struct sender_report_packet *sr) {
-	ssrc_sender_report(ctx->mp->media, &ctx->scratch.sr, &ctx->mp->tv);
+	ssrc_sender_report(ctx->mp->media, &ctx->scratch.sr, ctx->mp->tv);
 }
 static void mos_rr(struct rtcp_process_ctx *ctx, struct report_block *rr) {
-	ssrc_receiver_report(ctx->mp->media, ctx->mp->sfd, &ctx->scratch.rr, &ctx->mp->tv);
+	ssrc_receiver_report(ctx->mp->media, ctx->mp->sfd, &ctx->scratch.rr, ctx->mp->tv);
 }
 static void mos_xr_rr_time(struct rtcp_process_ctx *ctx, const struct xr_rb_rr_time *rr) {
-	ssrc_receiver_rr_time(ctx->mp->media, &ctx->scratch.xr_rr, &ctx->mp->tv);
+	ssrc_receiver_rr_time(ctx->mp->media, &ctx->scratch.xr_rr, ctx->mp->tv);
 }
 static void mos_xr_dlrr(struct rtcp_process_ctx *ctx, const struct xr_rb_dlrr *dlrr) {
-	ssrc_receiver_dlrr(ctx->mp->media, &ctx->scratch.xr_dlrr, &ctx->mp->tv);
+	ssrc_receiver_dlrr(ctx->mp->media, &ctx->scratch.xr_dlrr, ctx->mp->tv);
 }
 static void mos_xr_voip_metrics(struct rtcp_process_ctx *ctx, const struct xr_rb_voip_metrics *rb_voip_mtc) {
-	ssrc_voip_metrics(ctx->mp->media, &ctx->scratch.xr_vm, &ctx->mp->tv);
+	ssrc_voip_metrics(ctx->mp->media, &ctx->scratch.xr_vm, ctx->mp->tv);
 }
 
 
@@ -1291,7 +1294,7 @@ static void mos_xr_voip_metrics(struct rtcp_process_ctx *ctx, const struct xr_rb
 static void transcode_common(struct rtcp_process_ctx *ctx, struct rtcp_packet *common) {
 	if (!ctx->mp->ssrc_in)
 		return;
-	if (ctx->scratch_common_ssrc != ctx->mp->ssrc_in->parent->h.ssrc)
+	if (ctx->scratch_common_ssrc != ctx->mp->ssrc_in->h.ssrc)
 		return;
 	// forward SSRC mapping
 	common->ssrc = htonl(ctx->mp->ssrc_in->ssrc_map_out);
@@ -1301,23 +1304,21 @@ static void transcode_common(struct rtcp_process_ctx *ctx, struct rtcp_packet *c
 static void transcode_rr(struct rtcp_process_ctx *ctx, struct report_block *rr) {
 	if (!ctx->mp->ssrc_in)
 		return;
-	if (ctx->scratch.rr.from != ctx->mp->ssrc_in->parent->h.ssrc)
+	if (ctx->scratch.rr.from != ctx->mp->ssrc_in->h.ssrc)
 		return;
 	if (!ctx->mp->media)
 		return;
 
 	// reverse SSRC mapping
-	struct ssrc_ctx *map_ctx = get_ssrc_ctx(ctx->scratch.rr.ssrc, ctx->mp->media->monologue->ssrc_hash,
-			SSRC_DIR_OUTPUT, ctx->mp->media->monologue);
+	struct ssrc_entry_call *map_ctx = get_ssrc(ctx->scratch.rr.ssrc, &ctx->mp->media->ssrc_hash_out);
 	rr->ssrc = htonl(map_ctx->ssrc_map_out);
 
 	if (!ctx->mp->media_out)
 		return;
 
 	// for reception stats
-	struct ssrc_ctx *input_ctx = get_ssrc_ctx(map_ctx->ssrc_map_out,
-			ctx->mp->media_out->monologue->ssrc_hash,
-			SSRC_DIR_INPUT, NULL);
+	struct ssrc_entry_call *input_ctx = get_ssrc(map_ctx->ssrc_map_out,
+			&ctx->mp->media_out->ssrc_hash_in);
 	if (!input_ctx)
 		return;
 
@@ -1330,8 +1331,8 @@ static void transcode_rr(struct rtcp_process_ctx *ctx, struct report_block *rr)
 	if (!packets)
 		goto out;
 
-	unsigned int lost = input_ctx->parent->packets_lost;
-	unsigned int dupes = input_ctx->parent->duplicates;
+	unsigned int lost = input_ctx->packets_lost;
+	unsigned int dupes = input_ctx->duplicates;
 	unsigned int tot_lost = lost - dupes; // can be negative/rollover
 
 	ilogs(rtcp, LOG_DEBUG, "Substituting RTCP RR SSRC from %s%x%s to %x: %u packets, %u lost, %u duplicates",
@@ -1357,14 +1358,13 @@ static void transcode_rr(struct rtcp_process_ctx *ctx, struct report_block *rr)
 	// XXX jitter, last SR
 
 out:
-	if (input_ctx)
-		obj_put(&input_ctx->parent->h);
-	obj_put(&map_ctx->parent->h);
+	ssrc_entry_release(input_ctx);
+	ssrc_entry_release(map_ctx);
 }
 static void transcode_sr(struct rtcp_process_ctx *ctx, struct sender_report_packet *sr) {
 	if (!ctx->mp->ssrc_in)
 		return;
-	if (ctx->scratch.sr.ssrc != ctx->mp->ssrc_in->parent->h.ssrc)
+	if (ctx->scratch.sr.ssrc != ctx->mp->ssrc_in->h.ssrc)
 		return;
 	if (!ctx->mp->ssrc_out)
 		return;
@@ -1414,19 +1414,22 @@ void rtcp_init(void) {
 
 
 static GString *rtcp_sender_report(struct ssrc_sender_report *ssr,
-		uint32_t ssrc, uint32_t ssrc_out, uint32_t ts, uint32_t packets, uint32_t octets, GQueue *rrs,
-		GQueue *srrs)
+		uint32_t ssrc, uint32_t ssrc_out, uint32_t ts, uint32_t packets, uint32_t octets, ssrc_q *rrs,
+		ssrc_rr_q *srrs)
 {
 	GString *ret = g_string_sized_new(128);
 	g_string_set_size(ret, sizeof(struct sender_report_packet));
 	struct sender_report_packet *sr = (void *) ret->str;
 
+	long now_msw = rtpe_now / 1000000L + 2208988800L;
+	long now_lsw = 4294967296L * (rtpe_now % 1000000) / 1000000L;
+
 	*sr = (struct sender_report_packet) {
 		.rtcp.header.version = 2,
 		.rtcp.header.pt = RTCP_PT_SR,
 		.rtcp.ssrc = htonl(ssrc),
-		.ntp_msw = htonl(rtpe_now.tv_sec + 2208988800),
-		.ntp_lsw = htonl((4294967295ULL * rtpe_now.tv_usec) / 1000000ULL),
+		.ntp_msw = htonl(now_msw),
+		.ntp_lsw = htonl(now_lsw),
 		.timestamp = htonl(ts), // XXX calculate from rtpe_now instead
 		.packet_count = htonl(packets),
 		.octet_count = htonl(octets),
@@ -1434,8 +1437,8 @@ static GString *rtcp_sender_report(struct ssrc_sender_report *ssr,
 	if (ssr) {
 		*ssr = (struct ssrc_sender_report) {
 			.ssrc = ssrc_out,
-			.ntp_msw = rtpe_now.tv_sec + 2208988800,
-			.ntp_lsw = (4294967295ULL * rtpe_now.tv_usec) / 1000000ULL,
+			.ntp_msw = now_msw,
+			.ntp_lsw = now_lsw,
 			.timestamp = ts, // XXX calculate from rtpe_now instead
 			.packet_count = packets,
 			.octet_count = octets,
@@ -1445,7 +1448,7 @@ static GString *rtcp_sender_report(struct ssrc_sender_report *ssr,
 	// receiver reports
 	int i = 0, n = 0;
 	while (rrs->length) {
-		struct ssrc_ctx *s = g_queue_pop_head(rrs);
+		struct ssrc_entry_call *s = t_queue_pop_head(rrs);
 		if (i < 30) {
 			g_string_set_size(ret, ret->len + sizeof(struct report_block));
 			struct report_block *rr = (void *) ret->str + ret->len - sizeof(struct report_block);
@@ -1453,23 +1456,22 @@ static GString *rtcp_sender_report(struct ssrc_sender_report *ssr,
 			// XXX unify with transcode_rr
 
 			// last received SR?
-			struct ssrc_entry_call *se = s->parent;
-			long long tv_diff = 0;
+			int64_t tv_diff = 0;
 			uint32_t ntp_middle_bits = 0;
-			mutex_lock(&se->h.lock);
-			if (se->sender_reports.length) {
-				struct ssrc_time_item *si = se->sender_reports.tail->data;
-				tv_diff = timeval_diff(&rtpe_now, &si->received);
+			mutex_lock(&s->h.lock);
+			if (s->sender_reports.length) {
+				struct ssrc_time_item *si = s->sender_reports.tail->data;
+				tv_diff = rtpe_now - si->received;
 				ntp_middle_bits = si->ntp_middle_bits;
 			}
-			uint32_t jitter = se->jitter;
-			mutex_unlock(&se->h.lock);
+			uint32_t jitter = s->jitter;
+			mutex_unlock(&s->h.lock);
 
-			uint64_t lost = se->packets_lost;
+			uint64_t lost = s->packets_lost;
 			uint64_t tot = atomic64_get(&s->stats->packets);
 
 			*rr = (struct report_block) {
-				.ssrc = htonl(s->parent->h.ssrc),
+				.ssrc = htonl(s->h.ssrc),
 				.fraction_lost = lost * 256 / (tot + lost),
 				.number_lost[0] = (lost >> 16) & 0xff,
 				.number_lost[1] = (lost >> 8) & 0xff,
@@ -1481,10 +1483,10 @@ static GString *rtcp_sender_report(struct ssrc_sender_report *ssr,
 			};
 
 			if (srrs) {
-				struct ssrc_receiver_report *srr = g_slice_alloc(sizeof(*srr));
+				struct ssrc_receiver_report *srr = g_new(__typeof(*srr), 1);
 				*srr = (struct ssrc_receiver_report) {
 					.from = ssrc_out,
-					.ssrc = s->parent->h.ssrc,
+					.ssrc = s->h.ssrc,
 					.fraction_lost = lost * 256 / (tot + lost),
 					.packets_lost = lost,
 					.high_seq_received = atomic_get_na(&s->stats->ext_seq),
@@ -1492,12 +1494,12 @@ static GString *rtcp_sender_report(struct ssrc_sender_report *ssr,
 					.dlsr = tv_diff * 65536 / 1000000,
 					.jitter = jitter >> 4,
 				};
-				g_queue_push_tail(srrs, srr);
+				t_queue_push_tail(srrs, srr);
 			}
 
 			n++;
 		}
-		ssrc_ctx_put(&s);
+		ssrc_entry_release(s);
 		i++;
 	}
 
@@ -1538,26 +1540,22 @@ static GString *rtcp_sender_report(struct ssrc_sender_report *ssr,
 	return ret;
 }
 
-void rtcp_receiver_reports(GQueue *out, struct ssrc_hash *hash, struct call_monologue *ml) {
-	rwlock_lock_r(&hash->lock);
-	for (GList *l = hash->q.head; l; l = l->next) {
-		struct ssrc_entry_call *e = l->data;
-		struct ssrc_ctx *i = &e->input_ctx;
-		if (i->ref != ml)
-			continue;
+static void rtcp_receiver_reports(ssrc_q *out, struct ssrc_hash *hash) {
+	LOCK(&hash->lock);
+	for (GList *l = hash->nq.head; l; l = l->next) {
+		struct ssrc_entry_call *i = l->data;
 		if (!atomic64_get_na(&i->stats->packets))
 			continue;
 
-		ssrc_ctx_hold(i);
-		g_queue_push_tail(out, i);
+		ssrc_entry_hold(i);
+		t_queue_push_tail(out, i);
 	}
-	rwlock_unlock_r(&hash->lock);
 }
 
 
 // call must be locked in R
 // if a `ps` is locked, it must be passed as argument
-void rtcp_send_report(struct call_media *media, struct ssrc_ctx *ssrc_out,
+void rtcp_send_report(struct call_media *media, struct ssrc_entry_call *ssrc_out,
 		const struct packet_stream *locked)
 {
 	// figure out where to send it
@@ -1582,17 +1580,17 @@ void rtcp_send_report(struct call_media *media, struct ssrc_ctx *ssrc_out,
 
 	log_info_stream_fd(ps->selected_sfd);
 
-	GQueue rrs = G_QUEUE_INIT;
-	rtcp_receiver_reports(&rrs, media->monologue->ssrc_hash, ps->media->monologue);
+	ssrc_q rrs = TYPED_GQUEUE_INIT;
+	rtcp_receiver_reports(&rrs, &media->ssrc_hash_in);
 
 	ilogs(rtcp, LOG_DEBUG, "Generating and sending RTCP SR for %x and up to %i source(s)",
-			ssrc_out->parent->h.ssrc, rrs.length);
+			ssrc_out->h.ssrc, rrs.length);
 
 	struct ssrc_sender_report ssr;
-	GQueue srrs = G_QUEUE_INIT;
+	ssrc_rr_q srrs = TYPED_GQUEUE_INIT;
 
-	GString *sr = rtcp_sender_report(&ssr, ssrc_out->parent->h.ssrc,
-			ssrc_out->ssrc_map_out ? : ssrc_out->parent->h.ssrc,
+	GString *sr = rtcp_sender_report(&ssr, ssrc_out->h.ssrc,
+			ssrc_out->ssrc_map_out ? : ssrc_out->h.ssrc,
 			atomic_get_na(&ssrc_out->stats->timestamp),
 			atomic64_get_na(&ssrc_out->stats->packets),
 			atomic64_get(&ssrc_out->stats->bytes),
@@ -1600,14 +1598,14 @@ void rtcp_send_report(struct call_media *media, struct ssrc_ctx *ssrc_out,
 
 	// handle crypto
 
-	str rtcp_packet = STR_INIT_GS(sr);
+	str rtcp_packet = STR_GS(sr);
 
 	const struct streamhandler *crypt_handler = determine_handler(&transport_protocols[PROTO_RTP_AVP],
 			media, true);
 
 	if (crypt_handler && crypt_handler->out->rtcp_crypt) {
 		g_string_set_size(sr, sr->len + RTP_BUFFER_TAIL_ROOM);
-		rtcp_packet = STR_INIT_LEN(sr->str, sr->len - RTP_BUFFER_TAIL_ROOM);
+		rtcp_packet = STR_LEN(sr->str, sr->len - RTP_BUFFER_TAIL_ROOM);
 		crypt_handler->out->rtcp_crypt(&rtcp_packet, ps, ssrc_out);
 	}
 
@@ -1620,15 +1618,15 @@ void rtcp_send_report(struct call_media *media, struct ssrc_ctx *ssrc_out,
 		struct packet_stream *sink = sh->sink;
 		struct call_media *other_media = sink->media;
 
-		ssrc_sender_report(other_media, &ssr, &rtpe_now);
-		for (GList *k = srrs.head; k; k = k->next) {
+		ssrc_sender_report(other_media, &ssr, rtpe_now);
+		for (__auto_type k = srrs.head; k; k = k->next) {
 			struct ssrc_receiver_report *srr = k->data;
-			ssrc_receiver_report(other_media, sink->selected_sfd, srr, &rtpe_now);
+			ssrc_receiver_report(other_media, sink->selected_sfd, srr, rtpe_now);
 		}
 	}
 	while (srrs.length) {
-		struct ssrc_receiver_report *srr = g_queue_pop_head(&srrs);
-		g_slice_free1(sizeof(*srr), srr);
+		struct ssrc_receiver_report *srr = t_queue_pop_head(&srrs);
+		g_free(srr);
 	}
 
 out:
@@ -1642,3 +1640,10 @@ out:
 static void sink_common(struct rtcp_process_ctx *ctx, struct rtcp_packet *common) {
 	ctx->discard = 1;
 }
+
+
+bool rtcp_kernel_fw(struct call_media *media) {
+	if (!media->rtcp_handler)
+		return true;
+	return !media->rtcp_handler->no_kernel_fw;
+}
