@@ -12,11 +12,6 @@
 #include <limits.h>
 #include <stdbool.h>
 
-#define MAX_RTP_PACKET_SIZE	8192
-#define RTP_BUFFER_HEAD_ROOM	128
-#define RTP_BUFFER_TAIL_ROOM	512
-#define RTP_BUFFER_SIZE		(MAX_RTP_PACKET_SIZE + RTP_BUFFER_HEAD_ROOM + RTP_BUFFER_TAIL_ROOM)
-
 #include "compat.h"
 #include "socket.h"
 #include "media_socket.h"
@@ -59,40 +54,7 @@ enum message_type {
 	SIP_REPLY,
 };
 
-enum call_opmode {
-	OP_OFFER = 0,
-	OP_ANSWER = 1,
-	OP_REQUEST,
-	OP_REQ_ANSWER,
-	OP_PUBLISH,
-	OP_DELETE,
-	OP_QUERY,
-	OP_LIST,
-	OP_PING,
-	OP_STATISTICS,
-	OP_PLAY_DTMF,
-	OP_BLOCK_DTMF,
-	OP_UNBLOCK_DTMF,
-	OP_BLOCK_MEDIA,
-	OP_UNBLOCK_MEDIA,
-	OP_SILENCE_MEDIA,
-	OP_UNSILENCE_MEDIA,
-	OP_BLOCK_SILENCE_MEDIA,
-	OP_UNBLOCK_SILENCE_MEDIA,
-	OP_PLAY_MEDIA,
-	OP_STOP_MEDIA,
-	OP_START_FORWARDING,
-	OP_STOP_FORWARDING,
-	OP_SUBSCRIBER_REQ,
-	OP_SUBSCRIBER_ANS,
-	OP_UNSUBSCRIBE,
-	OP_START_RECORDING,
-	OP_STOP_RECORDING,
-	OP_PAUSE_RECORDING,
-	OP_OTHER,
-};
-
-#define IS_OP_OTHER(opmode)                                                              \
+#define IS_OP_OTHER(opmode)                                                                      \
 		 ((opmode == OP_DELETE || opmode == OP_QUERY)                                    \
 		 || (opmode == OP_LIST || opmode == OP_PING)                                     \
 		 || (opmode == OP_STATISTICS || opmode == OP_PLAY_DTMF)                          \
@@ -102,16 +64,24 @@ enum call_opmode {
 		 || (opmode == OP_BLOCK_SILENCE_MEDIA || opmode == OP_UNBLOCK_SILENCE_MEDIA)     \
 		 || (opmode == OP_PLAY_MEDIA || opmode == OP_STOP_MEDIA)                         \
 		 || (opmode == OP_START_FORWARDING || opmode == OP_STOP_FORWARDING)              \
-		 || (opmode == OP_SUBSCRIBER_REQ || opmode == OP_SUBSCRIBER_ANS)                 \
 		 || (opmode == OP_UNSUBSCRIBE || opmode == OP_START_RECORDING)                   \
 		 || (opmode == OP_STOP_RECORDING || opmode == OP_PAUSE_RECORDING)                \
 		 || (opmode == OP_OTHER))
 
-#define IS_OP_DIRECTIONAL(opmode)                                                        \
+#define IS_OP_DIRECTIONAL(opmode)                                                                \
 		 ((opmode == OP_BLOCK_DTMF || opmode == OP_BLOCK_MEDIA)                          \
 		 || (opmode == OP_UNBLOCK_DTMF || opmode == OP_UNBLOCK_MEDIA)                    \
 		 || (opmode == OP_START_FORWARDING || opmode == OP_STOP_FORWARDING))
 
+#define RESET_BANDWIDTH(union_var, value) \
+	do { \
+		union_var.as = value; \
+		union_var.rr = value; \
+		union_var.rs = value; \
+		union_var.ct = value; \
+		union_var.tias = value; \
+	} while(0)
+
 enum call_media_counted {
 	CMC_INCREMENT = 0,
 	CMC_DECREMENT,
@@ -147,25 +117,26 @@ enum {
 #define IS_OWN_CALL(c) !IS_FOREIGN_CALL(c)
 
 /* flags shared by several of the structs below */
-#define SHARED_FLAG_IMPLICIT_RTCP		0x00000001
-#define SHARED_FLAG_ASYMMETRIC			0x00000002
-#define SHARED_FLAG_SEND			0x00000004
-#define SHARED_FLAG_RECV			0x00000008
-#define SHARED_FLAG_RTCP_MUX			0x00000010
-#define SHARED_FLAG_SETUP_ACTIVE		0x00000020
-#define SHARED_FLAG_SETUP_PASSIVE		0x00000040
-#define SHARED_FLAG_ICE				0x00000080
-#define SHARED_FLAG_STRICT_SOURCE		0x00000100
-#define SHARED_FLAG_MEDIA_HANDOVER		0x00000200
-#define SHARED_FLAG_TRICKLE_ICE			0x00000400
-#define SHARED_FLAG_ICE_LITE_PEER		0x00000800
-#define SHARED_FLAG_UNIDIRECTIONAL		0x00001000
-#define SHARED_FLAG_RTCP_FB			0x00002000
-#define SHARED_FLAG_LEGACY_OSRTP		0x00004000
-#define SHARED_FLAG_LEGACY_OSRTP_REV		0x00008000
+#define SHARED_FLAG_IMPLICIT_RTCP		(1LL <<  0)
+#define SHARED_FLAG_ASYMMETRIC			(1LL <<  1)
+#define SHARED_FLAG_SEND			(1LL <<  2)
+#define SHARED_FLAG_RECV			(1LL <<  3)
+#define SHARED_FLAG_RTCP_MUX			(1LL <<  4)
+#define SHARED_FLAG_SETUP_ACTIVE		(1LL <<  5)
+#define SHARED_FLAG_SETUP_PASSIVE		(1LL <<  6)
+#define SHARED_FLAG_ICE				(1LL <<  7)
+#define SHARED_FLAG_STRICT_SOURCE		(1LL <<  8)
+#define SHARED_FLAG_MEDIA_HANDOVER		(1LL <<  9)
+#define SHARED_FLAG_TRICKLE_ICE			(1LL << 10)
+#define SHARED_FLAG_ICE_LITE_PEER		(1LL << 11)
+#define SHARED_FLAG_UNIDIRECTIONAL		(1LL << 12)
+#define SHARED_FLAG_RTCP_FB			(1LL << 13)
+#define SHARED_FLAG_LEGACY_OSRTP		(1LL << 14)
+#define SHARED_FLAG_LEGACY_OSRTP_REV		(1LL << 15)
+/* empty range [16 - 29] in-between for non-shared flags */
+#define SHARED_FLAG_END_OF_CANDIDATES		(1LL << 30)
 
 /* struct stream_params */
-#define SP_FLAG_NO_RTCP				0x00010000
 #define SP_FLAG_IMPLICIT_RTCP			SHARED_FLAG_IMPLICIT_RTCP
 #define SP_FLAG_RTCP_MUX			SHARED_FLAG_RTCP_MUX
 #define SP_FLAG_SEND				SHARED_FLAG_SEND
@@ -182,86 +153,100 @@ enum {
 #define SP_FLAG_RTCP_FB				SHARED_FLAG_RTCP_FB
 #define SP_FLAG_LEGACY_OSRTP			SHARED_FLAG_LEGACY_OSRTP
 #define SP_FLAG_LEGACY_OSRTP_REV		SHARED_FLAG_LEGACY_OSRTP_REV
+#define SP_FLAG_END_OF_CANDIDATES		SHARED_FLAG_END_OF_CANDIDATES
 
 /* struct packet_stream */
-#define PS_FLAG_RTP				0x00010000
-#define PS_FLAG_RTCP				0x00020000
+#define PS_FLAG_RTP				(1LL << 16)
+#define PS_FLAG_RTCP				(1LL << 17)
 #define PS_FLAG_IMPLICIT_RTCP			SHARED_FLAG_IMPLICIT_RTCP
-#define PS_FLAG_FALLBACK_RTCP			0x00040000
-#define PS_FLAG_RECEIVED			0x00080000
-#define PS_FLAG_FILLED				0x00100000
-#define PS_FLAG_CONFIRMED			0x00200000
-#define PS_FLAG_KERNELIZED			0x00400000
-#define PS_FLAG_NO_KERNEL_SUPPORT		0x00800000
-#define PS_FLAG_UNUSED				0x01000000
-#define PS_FLAG_FINGERPRINT_VERIFIED		0x02000000
+#define PS_FLAG_FALLBACK_RTCP			(1LL << 18)
+#define PS_FLAG_RECEIVED			(1LL << 19)
+#define PS_FLAG_FILLED				(1LL << 20)
+#define PS_FLAG_CONFIRMED			(1LL << 21)
+#define PS_FLAG_KERNELIZED			(1LL << 22)
+#define PS_FLAG_NO_KERNEL_SUPPORT		(1LL << 23)
+#define PS_FLAG_UNUSED				(1LL << 24)
+#define PS_FLAG_FINGERPRINT_VERIFIED		(1LL << 25)
 #define PS_FLAG_STRICT_SOURCE			SHARED_FLAG_STRICT_SOURCE
 #define PS_FLAG_MEDIA_HANDOVER			SHARED_FLAG_MEDIA_HANDOVER
 #define PS_FLAG_ICE				SHARED_FLAG_ICE
-#define PS_FLAG_ZERO_ADDR			0x04000000
-#define PS_FLAG_PIERCE_NAT			0x08000000
-#define PS_FLAG_NAT_WAIT			0x10000000
+#define PS_FLAG_ZERO_ADDR			(1LL << 26)
+#define PS_FLAG_PIERCE_NAT			(1LL << 27)
+#define PS_FLAG_NAT_WAIT			(1LL << 28)
 
 /* struct call_media */
-#define MEDIA_FLAG_INITIALIZED			0x00010000
+#define MEDIA_FLAG_INITIALIZED			(1LL << 16)
 #define MEDIA_FLAG_ASYMMETRIC			SHARED_FLAG_ASYMMETRIC
 #define MEDIA_FLAG_UNIDIRECTIONAL		SHARED_FLAG_UNIDIRECTIONAL
 #define MEDIA_FLAG_SEND				SHARED_FLAG_SEND
 #define MEDIA_FLAG_RECV				SHARED_FLAG_RECV
 #define MEDIA_FLAG_RTCP_MUX			SHARED_FLAG_RTCP_MUX
-#define MEDIA_FLAG_RTCP_MUX_OVERRIDE		0x00020000
-#define MEDIA_FLAG_DTLS				0x00040000
-#define MEDIA_FLAG_SDES				0x00080000
+#define MEDIA_FLAG_RTCP_MUX_OVERRIDE		(1LL << 17)
+#define MEDIA_FLAG_DTLS				(1LL << 18)
+#define MEDIA_FLAG_SDES				(1LL << 19)
 #define MEDIA_FLAG_SETUP_ACTIVE			SHARED_FLAG_SETUP_ACTIVE
 #define MEDIA_FLAG_SETUP_PASSIVE		SHARED_FLAG_SETUP_PASSIVE
-#define MEDIA_FLAG_PASSTHRU			0x00100000
+#define MEDIA_FLAG_PASSTHRU			(1LL << 20)
 #define MEDIA_FLAG_ICE				SHARED_FLAG_ICE
 #define MEDIA_FLAG_TRICKLE_ICE			SHARED_FLAG_TRICKLE_ICE
 #define MEDIA_FLAG_ICE_LITE_PEER		SHARED_FLAG_ICE_LITE_PEER
-#define MEDIA_FLAG_ICE_CONTROLLING		0x00200000
-#define MEDIA_FLAG_LOOP_CHECK			0x00400000
-#define MEDIA_FLAG_REORDER_FORCED		0x00800000
-#define MEDIA_FLAG_PTIME_OVERRIDE		0x01000000
+#define MEDIA_FLAG_ICE_CONTROLLING		(1LL << 21)
+#define MEDIA_FLAG_LOOP_CHECK			(1LL << 22)
+#define MEDIA_FLAG_REORDER_FORCED		(1LL << 23)
+#define MEDIA_FLAG_PTIME_OVERRIDE		(1LL << 24)
 #define MEDIA_FLAG_RTCP_FB			SHARED_FLAG_RTCP_FB
-#define MEDIA_FLAG_GENERATOR			0x02000000
-#define MEDIA_FLAG_ICE_LITE_SELF		0x04000000
-#define MEDIA_FLAG_RTCP_GEN			0x08000000
-#define MEDIA_FLAG_ECHO				0x10000000
-#define MEDIA_FLAG_BLACKHOLE			0x20000000
-#define MEDIA_FLAG_AUDIO_PLAYER			0x80000000
+#define MEDIA_FLAG_GENERATOR			(1LL << 25)
+#define MEDIA_FLAG_ICE_LITE_SELF		(1LL << 26)
+#define MEDIA_FLAG_RTCP_GEN			(1LL << 27)
+#define MEDIA_FLAG_ECHO				(1LL << 28)
+#define MEDIA_FLAG_BLACKHOLE			(1LL << 29)
+// 30 used by SHARED_FLAG_END_OF_CANDIDATES
+#define MEDIA_FLAG_AUDIO_PLAYER			(1LL << 31)
+#define MEDIA_FLAG_END_OF_CANDIDATES		SHARED_FLAG_END_OF_CANDIDATES
 #define MEDIA_FLAG_LEGACY_OSRTP			SHARED_FLAG_LEGACY_OSRTP
 #define MEDIA_FLAG_LEGACY_OSRTP_REV		SHARED_FLAG_LEGACY_OSRTP_REV
-#define MEDIA_FLAG_TRANSCODING			0x100000000LL
-#define MEDIA_FLAG_BLOCK_EGRESS			0x200000000LL
+#define MEDIA_FLAG_TRANSCODING			(1LL << 32)
+#define MEDIA_FLAG_BLOCK_EGRESS			(1LL << 33)
+/* MoH sendrecv flag handling, if set then hold is on-going */
+#define MEDIA_FLAG_FAKE_SENDRECV		(1LL << 34)
+/* in common with previous, if set, then answer has to be recvonly,
+ * if not set, then inactive.
+ */
+#define MEDIA_FLAG_REAL_SENDONLY		(1LL << 35)
+#define MEDIA_FLAG_SELECT_PT			(1LL << 36)
 #define MEDIA_FLAG_RECRYPT			(1LL << 37)
+#define MEDIA_FLAG_PUBLIC			(1LL << 38)
 
 /* struct call_monologue */
-#define ML_FLAG_REC_FORWARDING			0x00010000
-#define ML_FLAG_INJECT_DTMF			0x00020000
-#define ML_FLAG_DTMF_INJECTION_ACTIVE		0x00040000
-#define ML_FLAG_DETECT_DTMF			0x00080000
-#define ML_FLAG_NO_RECORDING			0x00100000
-/* unused					0x00200000 */
-#define ML_FLAG_BLOCK_SHORT			0x00400000
-#define ML_FLAG_BLOCK_MEDIA			0x00800000
-#define ML_FLAG_SILENCE_MEDIA			0x01000000
+#define ML_FLAG_REC_FORWARDING			(1LL << 16)
+#define ML_FLAG_INJECT_DTMF			(1LL << 17)
+#define ML_FLAG_DTMF_INJECTION_ACTIVE		(1LL << 18)
+#define ML_FLAG_DETECT_DTMF			(1LL << 19)
+#define ML_FLAG_NO_RECORDING			(1LL << 20)
+#define ML_FLAG_FINAL_RESPONSE			(1LL << 21)
+#define ML_FLAG_BLOCK_SHORT			(1LL << 22)
+#define ML_FLAG_BLOCK_MEDIA			(1LL << 23)
+#define ML_FLAG_SILENCE_MEDIA			(1LL << 24)
+#define ML_FLAG_MOH_SENDRECV			(1LL << 25)
+#define ML_FLAG_MOH_ZEROCONN			(1LL << 26)
+#define ML_FLAG_FORCE_TRANSCODING		(1LL << 27)
 
 /* call_t */
-#define CALL_FLAG_IPV4_OFFER			0x00010000
-#define CALL_FLAG_IPV6_OFFER			0x00020000
-#define CALL_FLAG_IPV4_ANSWER			0x00040000
-#define CALL_FLAG_IPV6_ANSWER			0x00080000
-#define CALL_FLAG_MEDIA_COUNTED			0x00100000
-#define CALL_FLAG_RECORDING_ON			0x00200000
-#define CALL_FLAG_REC_FORWARDING		0x00400000
-#define CALL_FLAG_DROP_TRAFFIC			0x00800000
-#define CALL_FLAG_FOREIGN			0x01000000 // created_via_redis_notify call
-#define CALL_FLAG_FOREIGN_MEDIA			0x02000000 // for calls taken over, tracks whether we have media
-#define CALL_FLAG_DISABLE_JB			0x04000000
-#define CALL_FLAG_DEBUG				0x08000000
-#define CALL_FLAG_BLOCK_MEDIA			0x10000000
-#define CALL_FLAG_SILENCE_MEDIA			0x20000000
-#define CALL_FLAG_NO_REC_DB			0x40000000
+#define CALL_FLAG_IPV4_OFFER			(1LL << 16)
+#define CALL_FLAG_IPV6_OFFER			(1LL << 17)
+#define CALL_FLAG_IPV4_ANSWER			(1LL << 18)
+#define CALL_FLAG_IPV6_ANSWER			(1LL << 19)
+#define CALL_FLAG_MEDIA_COUNTED			(1LL << 20)
+#define CALL_FLAG_RECORDING_ON			(1LL << 21)
+#define CALL_FLAG_REC_FORWARDING		(1LL << 22)
+#define CALL_FLAG_DROP_TRAFFIC			(1LL << 23)
+#define CALL_FLAG_FOREIGN			(1LL << 24) // created_via_redis_notify call
+#define CALL_FLAG_FOREIGN_MEDIA			(1LL << 25) // for calls taken over, tracks whether we have media
+#define CALL_FLAG_DISABLE_JB			(1LL << 26)
+#define CALL_FLAG_DEBUG				(1LL << 27)
+#define CALL_FLAG_BLOCK_MEDIA			(1LL << 28)
+#define CALL_FLAG_SILENCE_MEDIA			(1LL << 29)
+#define CALL_FLAG_NO_REC_DB			(1LL << 30)
 
 /* access macros */
 #define SP_ISSET(p, f)		bf_isset(&(p)->sp_flags, SP_FLAG_ ## f)
@@ -313,6 +298,8 @@ enum block_dtmf_mode {
 #include "crypto.h"
 #include "dtls.h"
 #include "dtmf.h"
+#include "arena.h"
+#include "ssrc.h"
 
 
 struct control_stream;
@@ -322,7 +309,6 @@ struct rtpengine_srtp;
 struct local_interface;
 struct call_monologue;
 struct ice_agent;
-struct ssrc_hash;
 struct codec_handler;
 struct media_player;
 struct send_timer;
@@ -335,11 +321,6 @@ struct janus_session;
 struct audio_player;
 struct media_subscription;
 
-typedef bencode_buffer_t call_buffer_t;
-#define call_buffer_alloc bencode_buffer_alloc
-#define call_buffer_init bencode_buffer_init
-#define call_buffer_free bencode_buffer_free
-
 
 
 
@@ -350,7 +331,11 @@ TYPED_GQUEUE(subscription, struct media_subscription)
 TYPED_DIRECT_FUNCS(media_direct_hash, media_direct_eq, struct call_media)
 TYPED_GHASHTABLE(subscription_ht, struct call_media, subscription_list, media_direct_hash, media_direct_eq,
 		NULL, NULL)
+TYPED_GHASHTABLE(media_id_ht, str, struct call_media, str_hash, str_equal, NULL, NULL)
 
+struct session_bandwidth {
+	long as, rr, rs, ct, tias;
+};
 
 struct codec_store {
 	codecs_ht		codecs; // int payload type -> rtp_payload_type
@@ -377,28 +362,29 @@ struct stream_params {
 	const struct transport_protocol *protocol;
 	str			format_str;
 	sdes_q			sdes_params; // slice-alloc'd
-	sdp_attr_q		attributes;	/* just some other attributes */
+	sdp_attr_q		generic_attributes;	/* just some other attributes */
+	sdp_attr_q		all_attributes;		/* all attributes */
 	str			direction[2];
-	sockfamily_t		*desired_family;
 	struct dtls_fingerprint fingerprint;
 	atomic64		sp_flags;
 	struct codec_store	codecs;
 	candidate_q		ice_candidates; /* slice-alloc'd */
 	str			ice_ufrag;
 	str			ice_pwd;
-	int			ptime;
+	int			ptime, maxptime;
 	str			media_id;
 	struct t38_options	t38_options;
 	str			tls_id;
 	int			media_sdp_id;
-	int			media_session_as, media_session_rr, media_session_rs;
+	struct session_bandwidth media_session_bandiwdth;
+	str			sdp_information;
 };
 
 struct endpoint_map {
 	unsigned int		unique_id;
 	struct endpoint		endpoint;
 	unsigned int		num_ports;
-	const struct logical_intf *logical_intf;
+	struct logical_intf	*logical_intf;
 	sfd_intf_list_q		intf_sfds; /* list of struct sfd_intf_list - contains stream_fd list */
 	unsigned int		wildcard:1;
 };
@@ -409,6 +395,8 @@ struct loop_protector {
 };
 
 
+TYPED_GPTRARRAY(rtp_stats_arr, struct rtp_stats)
+TYPED_GHASHTABLE_PROTO(rtp_stats_ht, void, struct rtp_stats)
 
 /**
  * The packet_stream itself can be marked as:
@@ -425,7 +413,7 @@ struct packet_stream {
 	mutex_t			lock;
 
 	struct call_media	*media;		/* RO */
-	call_t		*call;		/* RO */
+	call_t			*call;		/* RO */
 	unsigned int		component;	/* RO, starts with 1 */
 	unsigned int		unique_id;	/* RO */
 	struct recording_stream recording;	/* LOCK: call->master_lock */
@@ -437,25 +425,21 @@ struct packet_stream {
 	sink_handler_q		rtp_sinks;	/* LOCK: call->master_lock, ps->lock for streamhandler */
 	sink_handler_q		rtcp_sinks;	/* LOCK: call->master_lock, ps->lock for streamhandler */
 	struct packet_stream	*rtcp_sibling;	/* LOCK: call->master_lock */
-	sink_handler_q		rtp_mirrors;	/* LOCK: call->master_lock, ps->ock for streamhandler */
+	sink_handler_q		rtp_mirrors;	/* LOCK: call->master_lock, ps->lock for streamhandler */
 	struct endpoint		endpoint;	/* LOCK: ps->lock */
 	struct endpoint		detected_endpoints[4];		/* LOCK: ps->lock */
-	time_t			ep_detect_signal;		/* LOCK: ps->lock */
+	int64_t			ep_detect_signal;		/* LOCK: ps->lock */
 	struct endpoint		advertised_endpoint;		/* RO */
 	struct endpoint		learned_endpoint;		/* LOCK: ps->lock */
 	struct crypto_context	crypto;				/* OUT direction, LOCK: ps->lock */
-	struct ssrc_ctx		*ssrc_in[RTPE_NUM_SSRC_TRACKING],	/* LOCK: ps->lock */
-				*ssrc_out[RTPE_NUM_SSRC_TRACKING];	/* LOCK: ps->lock */
-	unsigned int		ssrc_in_idx,				/* LOCK: ps->lock */
-				ssrc_out_idx;				/* LOCK: ps->lock */
 	struct send_timer	*send_timer;				/* RO */
 	struct jitter_buffer	*jb;					/* RO */
-	time_t kernel_time;
+	int64_t			kernel_time_us;
 
 	struct stream_stats	*stats_in;
 	struct stream_stats	*stats_out;
-	atomic64		last_packet;				// userspace only
-	GHashTable		*rtp_stats;				/* LOCK: call->master_lock */
+	atomic64		last_packet_us;				// userspace only
+	rtp_stats_ht		rtp_stats;				/* LOCK: call->master_lock */
 	struct rtp_stats	*rtp_stats_cache;
 	enum endpoint_learning		el_flags;
 
@@ -472,9 +456,9 @@ struct packet_stream {
 	atomic64		ps_flags;
 };
 
-INLINE uint64_t packet_stream_last_packet(const struct packet_stream *ps) {
-	uint64_t lp1 = atomic64_get_na(&ps->last_packet);
-	uint64_t lp2 = atomic64_get_na(&ps->stats_in->last_packet);
+INLINE int64_t packet_stream_last_packet(const struct packet_stream *ps) {
+	uint64_t lp1 = atomic64_get_na(&ps->last_packet_us);
+	uint64_t lp2 = atomic64_get_na(&ps->stats_in->last_packet_us);
 	return MAX(lp1, lp2);
 }
 
@@ -496,7 +480,7 @@ struct call_media {
 	const struct transport_protocol *protocol;
 	str			format_str;
 	sockfamily_t		*desired_family;
-	const struct logical_intf *logical_intf;
+	struct logical_intf	*logical_intf;
 
 	struct ice_agent	*ice_agent;
 
@@ -506,13 +490,19 @@ struct call_media {
 	struct dtls_fingerprint fingerprint;			/* as received */
 	const struct dtls_hash_func *fp_hash_func;		/* outgoing */
 	str			tls_id;
+	candidate_q		ice_candidates; 		/* slice-alloc'd, as received */
+	unsigned int			media_rec_slot;
 
 	packet_stream_q		streams;			/* normally RTP + RTCP */
 	endpoint_map_q		endpoint_maps;
+	struct ssrc_hash	ssrc_hash_in;
+	struct ssrc_hash	ssrc_hash_out;
 
 	struct codec_store	codecs;
-	sdp_attr_q		sdp_attributes;			/* sdp_attr_new() */
-	sdp_attr_print_f	*sdp_attr_print;
+	struct codec_store	offered_codecs;
+	sdp_attr_q		generic_attributes;			/* sdp_attr_new() */
+	sdp_attr_q		all_attributes;			/* sdp_attr_new() */
+	sdp_media_attr_print_f	*sdp_attr_print;
 	codec_handlers_ht	codec_handlers;			/* int payload type -> struct codec_handler
 														XXX combine this with 'codecs' hash table? */
 	codec_handlers_q	codec_handlers_store;		/* storage for struct codec_handler */
@@ -542,19 +532,24 @@ struct call_media {
 	int					media_sdp_id;
 
 	/* bandwidth */
-	int bandwidth_as, bandwidth_rr, bandwidth_rs;
+	struct session_bandwidth sdp_media_bandwidth;
+
+	str sdp_information;
 
 #ifdef WITH_TRANSCODING
 	encoder_callback_t	encoder_callback;
 #endif
 
 	int			ptime;				/* either from SDP or overridden */
+	int			maxptime;			/* from SDP */
 
 	atomic64		media_flags;
+	unsigned int		update_iter;
 };
 
 TYPED_GPTRARRAY(medias_arr, struct call_media)
 TYPED_GQUEUE(medias, struct call_media)
+G_DEFINE_AUTO_CLEANUP_CLEAR_FUNC(medias_q, medias_q_clear)
 
 
 struct media_subscription {
@@ -580,33 +575,36 @@ struct call_monologue {
 
 	str			tag;
 	str			viabranch;
+	str_q			tag_aliases;
 	enum tag_type		tagtype;
 	str			label;
-	time_t			created;		/* RO */
-	time_t			deleted;
-	struct timeval		started;		/* for CDR */
-	struct timeval		terminated;		/* for CDR */
+	int64_t			created_us;		/* RO */
+	int64_t			deleted_us;
+	int64_t			started;		/* for CDR */
+	int64_t			terminated;		/* for CDR */
 	enum termination_reason	term_reason;
 	sockfamily_t		*desired_family;
-	const struct logical_intf *logical_intf;
+	struct logical_intf	*logical_intf;
 	GHashTable 		*associated_tags;
 	GHashTable		*subscribers_ht;	/* for quick lookup */
 	medias_arr		*medias;
-	GHashTable		*media_ids;
+	media_id_ht		media_ids;
 	struct media_player	*player;
 	struct media_player	*rec_player;
-	int			sdp_session_rr, sdp_session_rs;
-	str			last_in_sdp;
-	sdp_sessions_q		last_in_sdp_parsed;	/* last parsed `sdp_session` */
+	struct session_bandwidth sdp_session_bandwidth;
 	sdp_streams_q		last_in_sdp_streams;	/* last parsed `stream_params` */
 	GString			*last_out_sdp;
 
 	sdp_origin * session_sdp_orig;	/* actual origin belonging to this monologue */
 	sdp_origin * session_last_sdp_orig;	/* previously used origin by other other side */
 
-	char			*sdp_session_name;
-	char			*sdp_session_timing;
-	struct ssrc_hash	*ssrc_hash;
+	str			sdp_session_name;
+	str			sdp_session_timing;
+	str			sdp_session_group;	/* a=group: e.g. BUNDLE */
+	str			sdp_session_information;
+	str			sdp_session_uri;
+	str			sdp_session_phone;
+	str			sdp_session_email;
 	str			metadata;
 	struct janus_session	*janus_session;
 
@@ -627,8 +625,13 @@ struct call_monologue {
 	unsigned int		block_dtmf_trigger_end_ms; // unblock after this many ms
 
 	/* carry `sdp_session` attributes into resulting call monologue SDP */
-	sdp_attr_q		sdp_attributes;
-	sdp_attr_print_f	*sdp_attr_print;
+	sdp_attr_q		generic_attributes;
+	sdp_attr_q		all_attributes;
+	sdp_monologue_attr_print_f *sdp_attr_print;
+
+	long long moh_db_id;
+	str moh_blob;
+	str moh_file;
 
 	atomic64		ml_flags;
 };
@@ -637,6 +640,11 @@ TYPED_GQUEUE(monologues, struct call_monologue)
 G_DEFINE_AUTO_CLEANUP_CLEAR_FUNC(monologues_q, monologues_q_clear)
 TYPED_GHASHTABLE(tags_ht, str, struct call_monologue, str_hash, str_equal, NULL, NULL)
 
+struct sdp_fragment;
+TYPED_GQUEUE(fragment, struct sdp_fragment)
+TYPED_GHASHTABLE(fragments_ht, str, fragment_q, str_hash, str_equal, NULL, NULL)
+
+
 struct call_iterator_list {
 	call_list *first;
 	mutex_t lock; // protects .first and every entry's .data
@@ -719,8 +727,7 @@ struct call {
 	 */
 	struct obj		obj;
 
-	mutex_t			buffer_lock;
-	call_buffer_t		buffer;
+	memory_arena_t		buffer;
 
 	// use a single poller for all sockets within the call
 	struct poller		*poller;
@@ -742,6 +749,7 @@ struct call {
 	tags_ht			tags;
 	tags_ht			viabranches;
 	labels_ht		labels;
+	fragments_ht		sdp_fragments;
 	packet_stream_q		streams;
 	stream_fd_q		stream_fds;	/* stream_fd */
 	endpoint_map_q		endpoint_maps;
@@ -749,19 +757,19 @@ struct call {
 	struct mqtt_timer	*mqtt_timer;
 
 	str			callid;
-	struct timeval		created;
-	struct timeval		destroyed;
-	time_t			last_signal;
-	time_t			deleted;
-	time_t			ml_deleted;
+	str_q			callid_aliases;
+	int64_t			created;
+	int64_t			destroyed;
+	int64_t			last_signal_us;
+	int64_t			deleted_us;
+	int64_t			ml_deleted_us;
 	unsigned char		tos;
-	char			*created_from;
-	sockaddr_t		created_from_addr;
-	sockaddr_t		xmlrpc_callback;
+	str			created_from;
+	str			xmlrpc_callback;
 	endpoint_t		dtmf_log_dest;
 
 	int			redis_hosted_db;
-	atomic64		last_redis_update;
+	atomic64		last_redis_update_us;
 
 	struct recording 	*recording;
 	str			metadata;
@@ -776,6 +784,8 @@ struct call {
 	enum block_dtmf_mode	block_dtmf;
 
 	atomic64		call_flags;
+	unsigned int		update_iter;
+	unsigned int media_rec_slots;
 };
 
 
@@ -790,6 +800,7 @@ TYPED_GHASHTABLE(rtpe_calls_ht, str, struct call, str_hash, str_equal, NULL, NUL
 extern rwlock_t rtpe_callhash_lock;
 extern rtpe_calls_ht rtpe_callhash;
 extern struct call_iterator_list rtpe_call_iterators[NUM_CALL_ITERATORS];
+extern __thread call_t *call_memory_arena;
 
 
 
@@ -801,9 +812,13 @@ void __monologue_free(struct call_monologue *m);
 void __monologue_tag(struct call_monologue *ml, const str *tag);
 void __monologue_viabranch(struct call_monologue *ml, const str *viabranch);
 struct packet_stream *__packet_stream_new(call_t *call);
-void __add_media_subscription(struct call_media * which, struct call_media * to,
+__attribute__((nonnull(1, 2)))
+struct media_subscription *__add_media_subscription(struct call_media * which, struct call_media * to,
 		const struct sink_attrs *attrs);
-struct media_subscription *call_get_top_media_subscription(struct call_monologue *ml);
+bool __unsubscribe_media(struct call_media * which, struct call_media * from);
+struct media_subscription *call_ml_get_top_ms(struct call_monologue *ml);
+bool call_ml_sendonly_inactive(struct call_monologue *ml);
+struct media_subscription *call_media_get_top_ms(struct call_media * cm);
 struct media_subscription *call_get_media_subscription(subscription_ht ht, struct call_media * cm);
 struct call_monologue * ml_medias_subscribed_to_single_ml(struct call_monologue *ml);
 
@@ -815,15 +830,25 @@ void media_subscriptions_clear(subscription_q *q);
 G_DEFINE_AUTO_CLEANUP_CLEAR_FUNC(subscription_q, media_subscriptions_clear)
 
 call_t *call_get_or_create(const str *callid, bool exclusive);
-call_t *call_get_opmode(const str *callid, enum call_opmode opmode);
+call_t *call_get_opmode(const str *callid, enum ng_opmode opmode);
 void call_make_own_foreign(call_t *c, bool foreign);
 int call_get_mono_dialogue(struct call_monologue *monologues[2], call_t *call,
 		const str *fromtag,
 		const str *totag,
-		const str *viabranch);
+		const str *viabranch,
+		sdp_ng_flags *);
 struct call_monologue *call_get_monologue(call_t *call, const str *fromtag);
 struct call_monologue *call_get_or_create_monologue(call_t *call, const str *fromtag);
+__attribute__((nonnull(1, 2, 4, 5, 6)))
+struct call_media *call_make_transform_media(struct call_monologue *ml, const str *type, enum media_type type_id,
+		const str *media_id, const endpoint_t *remote, const str *interface);
+__attribute__((nonnull(1)))
 call_t *call_get(const str *callid);
+typedef enum { CG2_OK, CG2_NF1, CG2_NF2, CG2_SAME } call_get2_ret_t;
+__attribute__((nonnull(1, 2, 3, 4)))
+call_get2_ret_t call_get2(call_t **, call_t **, const str *, const str *);
+__attribute__((nonnull(1, 2)))
+bool call_merge(call_t *, call_t **);
 __attribute__((nonnull(2, 3)))
 int monologue_offer_answer(struct call_monologue *monologues[2], sdp_streams_q *streams, sdp_ng_flags *flags);
 __attribute__((nonnull(1, 2, 3, 4)))
@@ -834,11 +859,13 @@ int monologue_subscribe_request(const subscription_q *srms, struct call_monologu
 int monologue_subscribe_answer(struct call_monologue *dst, sdp_ng_flags *flags,
 		sdp_streams_q *streams);
 int monologue_unsubscribe(struct call_monologue *dst, sdp_ng_flags *);
+void dialogue_connect(struct call_monologue *, struct call_monologue *, sdp_ng_flags *);
+bool monologue_transform(struct call_monologue *, sdp_ng_flags *, medias_q *);
 void monologue_destroy(struct call_monologue *ml);
 int call_delete_branch_by_id(const str *callid, const str *branch,
-	const str *fromtag, const str *totag, bencode_item_t *output, int delete_delay);
+	const str *fromtag, const str *totag, ng_command_ctx_t *, int64_t delete_delay);
 int call_delete_branch(call_t *, const str *branch,
-	const str *fromtag, const str *totag, bencode_item_t *output, int delete_delay);
+	const str *fromtag, const str *totag, ng_command_ctx_t *, int64_t delete_delay);
 void call_destroy(call_t *);
 struct call_media *call_media_new(call_t *call);
 void call_media_free(struct call_media **mdp);
@@ -848,72 +875,34 @@ void call_media_unkernelize(struct call_media *media, const char *reason);
 void dialogue_unconfirm(struct call_monologue *ml, const char *);
 void __monologue_unconfirm(struct call_monologue *monologue, const char *);
 void __media_unconfirm(struct call_media *media, const char *);
-void update_init_subscribers(struct call_monologue *ml, enum call_opmode opmode);
+__attribute__((nonnull(1)))
+void update_init_monologue_subscribers(struct call_monologue *ml, enum ng_opmode opmode);
 
-int call_stream_address46(char *o, struct packet_stream *ps, enum stream_address_format format,
-		int *len, const struct local_intf *ifa, bool keep_unspec);
+int call_stream_address(GString *, struct packet_stream *ps, enum stream_address_format format,
+		const struct local_intf *ifa, bool keep_unspec);
 
-void add_total_calls_duration_in_interval(struct timeval *interval_tv);
+void add_total_calls_duration_in_interval(int64_t interval_tv);
 enum thread_looper_action call_timer(void);
 
-void __rtp_stats_update(GHashTable *dst, struct codec_store *);
-int __init_stream(struct packet_stream *ps);
-void call_stream_crypto_reset(struct packet_stream *ps);
+void __rtp_stats_update(rtp_stats_ht dst, struct codec_store *);
+bool __init_stream(struct packet_stream *ps);
 
 const rtp_payload_type *__rtp_stats_codec(struct call_media *m);
 
 #include "str.h"
 #include "rtp.h"
 
-INLINE void *call_malloc(call_t *c, size_t l) {
-	void *ret;
-	mutex_lock(&c->buffer_lock);
-	ret = call_buffer_alloc(&c->buffer, l);
-	mutex_unlock(&c->buffer_lock);
-	return ret;
-}
+#define call_malloc memory_arena_alloc
+#define call_dup memory_arena_dup
+#define call_ref memory_arena_ref
 
-INLINE char *call_strdup_len(call_t *c, const char *s, unsigned int len) {
-	char *r;
-	if (!s)
-		return NULL;
-	r = call_malloc(c, len + 1);
-	memcpy(r, s, len);
-	r[len] = 0;
-	return r;
-}
+#define call_strdup memory_arena_strdup
+#define call_strdup_str memory_arena_strdup_str
+#define call_str_cpy_len memory_arena_str_cpy_len
+#define call_str_cpy memory_arena_str_cpy
+#define call_str_cpy_c memory_arena_str_cpy_c
+#define call_str_dup memory_arena_str_dup
 
-INLINE char *call_strdup(call_t *c, const char *s) {
-	if (!s)
-		return NULL;
-	return call_strdup_len(c, s, strlen(s));
-}
-INLINE str *call_str_cpy_len(call_t *c, str *out, const char *in, int len) {
-	if (!in) {
-		*out = STR_NULL;
-		return out;
-	}
-	out->s = call_strdup_len(c, in, len);
-	out->len = len;
-	return out;
-}
-INLINE str *call_str_cpy(call_t *c, str *out, const str *in) {
-	return call_str_cpy_len(c, out, in ? in->s : NULL, in ? in->len : 0);
-}
-INLINE str *call_str_cpy_c(call_t *c, str *out, const char *in) {
-	return call_str_cpy_len(c, out, in, in ? strlen(in) : 0);
-}
-INLINE str *call_str_dup(call_t *c, const str *in) {
-	str *out;
-	out = call_malloc(c, sizeof(*out));
-	call_str_cpy_len(c, out, in->s, in->len);
-	return out;
-}
-INLINE str *call_str_init_dup(call_t *c, char *s) {
-	str t;
-	str_init(&t, s);
-	return call_str_dup(c, &t);
-}
 INLINE void __call_unkernelize(call_t *call, const char *reason) {
 	for (__auto_type l = call->monologues.head; l; l = l->next) {
 		struct call_monologue *ml = l->data;
@@ -937,4 +926,17 @@ INLINE endpoint_t *packet_stream_local_addr(struct packet_stream *ps) {
 	return &dummy;
 }
 
+INLINE void call_memory_arena_release(void) {
+	if (!call_memory_arena)
+		return;
+	obj_put(call_memory_arena);
+	call_memory_arena = NULL;
+	memory_arena = &rtpe_arena;
+}
+INLINE void call_memory_arena_set(call_t *c) {
+	call_memory_arena_release();
+	call_memory_arena = obj_get(c);
+	memory_arena = &c->buffer;
+}
+
 #endif
