@@ -11,11 +11,11 @@
 #include "rtplib.h"
 #include "timerthread.h"
 #include "types.h"
+#include "ssrc.h"
 
 struct call_media;
 struct codec_handler;
 struct media_packet;
-struct ssrc_hash;
 struct codec_ssrc_handler;
 struct rtp_header;
 struct stream_params;
@@ -31,25 +31,41 @@ struct sink_handler;
 typedef int codec_handler_func(struct codec_handler *, struct media_packet *);
 
 
+struct codec_handler_index {
+	int payload_type;
+	struct call_media *sink;
+};
+
+struct codec_pipeline_index {
+	rtp_payload_type src;
+	rtp_payload_type dst;
+};
+
 struct codec_handler {
-	rtp_payload_type source_pt; // source_pt.payload_type = hashtable index
-	rtp_payload_type dest_pt;
+	union {
+		struct codec_pipeline_index pi;
+		struct {
+			rtp_payload_type source_pt;
+			rtp_payload_type dest_pt;
+		};
+	};
+	struct codec_handler_index i;
 	int dtmf_payload_type;
 	int real_dtmf_payload_type;
 	int cn_payload_type;
 	codec_handler_func *handler_func;
-	unsigned int passthrough:1;
-	unsigned int kernelize:1;
-	unsigned int transcoder:1;
-	unsigned int pcm_dtmf_detect:1;
+	bool passthrough:1;
+	bool kernelize:1;
+	bool transcoder:1;
+	bool pcm_dtmf_detect:1;
+	bool blackhole:1;
 
 	size_t payload_len; // for short-packet blocking
 
-	struct ssrc_hash *ssrc_hash;
+	struct ssrc_hash ssrc_hash;
 	struct codec_handler *input_handler; // == main handler for supp codecs
 	struct codec_handler *output_handler; // == self, or other PT handler
 	struct call_media *media;
-	struct call_media *sink;
 #ifdef WITH_TRANSCODING
 	int (*packet_encoded)(encoder_t *enc, void *u1, void *u2);
 	int (*packet_decoded)(decoder_t *, AVFrame *, void *, void *);
@@ -60,20 +76,26 @@ struct codec_handler {
 	// for DTMF injection
 	struct codec_handler *dtmf_injector;
 	struct delay_buffer *delay_buffer;
+	// for remote transcoding
+	struct transcode_config *tcc;
+	struct transform_handler *transform;
 
 	// stats entry
+	const char *stats_chain_suffix;
+	const char *stats_chain_suffix_brief;
 	char *stats_chain;
 	struct codec_stats *stats_entry;
 };
 
 struct codec_packet {
 	struct timerthread_queue_entry ttq_entry;
+	codec_packet_list link;
 	str s;
 	str plain;
 	struct rtp_header *rtp;
 	unsigned long ts;
 	unsigned int clockrate;
-	struct ssrc_ctx *ssrc_out;
+	struct ssrc_entry_call *ssrc_out;
 	void (*free_func)(void *);
 	void (*plain_free_func)(void *);
 };
@@ -81,9 +103,29 @@ struct codec_packet {
 struct codec_scheduler {
 	unsigned long first_ts; // for output TS scaling
 	unsigned long last_ts; // to detect input lag and handle lost packets
-	struct timeval first_send;
+	int64_t first_send;
 	unsigned long first_send_ts;
 	long output_skew;
+	bool is_media_playback; // used to track media playback so we can skip skew calculation
+};
+
+struct transcode_config {
+	char *src; // unparsed
+	char *dst; // unparsed
+
+	char *name; // of the config section
+
+	struct codec_pipeline_index i; // parsed
+
+	int preference; // default: 0
+
+	// transform verdict
+	endpoint_t transform;
+	str local_interface;
+	str remote_interface;
+
+	// codec-chain verdict
+	bool codec_chain:1;
 };
 
 typedef union {
@@ -105,13 +147,16 @@ struct codec_handler *codec_handler_get(struct call_media *, int payload_type, s
 		struct sink_handler *);
 void codec_handlers_free(struct call_media *);
 struct codec_handler *codec_handler_make_playback(const rtp_payload_type *src_pt,
-		const rtp_payload_type *dst_pt, unsigned long ts, struct call_media *, uint32_t ssrc);
+		const rtp_payload_type *dst_pt, unsigned long ts, struct call_media *, uint32_t ssrc,
+		str_case_value_ht codec_set);
 struct codec_handler *codec_handler_make_media_player(const rtp_payload_type *src_pt,
-		const rtp_payload_type *dst_pt, unsigned long ts, struct call_media *, uint32_t ssrc);
-struct codec_handler *codec_handler_make_dummy(const rtp_payload_type *dst_pt, struct call_media *media);
-void codec_calc_jitter(struct ssrc_ctx *, unsigned long ts, unsigned int clockrate, const struct timeval *);
+		const rtp_payload_type *dst_pt, unsigned long ts, struct call_media *, uint32_t ssrc,
+		str_case_value_ht codec_set);
+struct codec_handler *codec_handler_make_dummy(const rtp_payload_type *dst_pt, struct call_media *media,
+		str_case_value_ht codec_set);
+void codec_calc_jitter(struct ssrc_entry_call *, unsigned long ts, unsigned int clockrate, int64_t);
 void codec_update_all_handlers(struct call_monologue *ml);
-void codec_update_all_source_handlers(struct call_monologue *ml, const sdp_ng_flags *flags);
+void codec_update_all_source_handlers(struct call_monologue *ml, const sdp_ng_flags *flags, bool clear_delay_buffer);
 
 struct codec_store_args {
 	str_case_value_ht codec_set;
@@ -133,13 +178,14 @@ void __codec_store_populate_reuse(struct codec_store *, struct codec_store *, st
 #define codec_store_populate_reuse(dst, src, ...) \
 	__codec_store_populate_reuse(dst, src, (struct codec_store_args) {__VA_ARGS__})
 __attribute__((nonnull(1, 2)))
+void codec_store_copy(struct codec_store *, struct codec_store *);
 void codec_store_add_raw(struct codec_store *cs, rtp_payload_type *pt);
 __attribute__((nonnull(1, 2)))
 void codec_store_strip(struct codec_store *, str_q *strip, str_case_ht except);
 __attribute__((nonnull(1, 2, 3)))
 void codec_store_offer(struct codec_store *, str_q *, struct codec_store *);
-__attribute__((nonnull(1, 2)))
-void codec_store_check_empty(struct codec_store *, struct codec_store *);
+__attribute__((nonnull(1, 2, 3)))
+void codec_store_check_empty(struct codec_store *, struct codec_store *, sdp_ng_flags *);
 __attribute__((nonnull(1, 2)))
 void codec_store_accept(struct codec_store *, str_q *, struct codec_store *);
 __attribute__((nonnull(1, 2)))
@@ -159,7 +205,7 @@ __attribute__((nonnull(1, 2)))
 bool codec_store_is_full_answer(const struct codec_store *src, const struct codec_store *dst);
 
 void codec_add_raw_packet(struct media_packet *mp, unsigned int clockrate);
-void codec_packet_free(void *);
+void codec_packet_free(struct codec_packet *);
 struct codec_packet *codec_packet_dup(struct codec_packet *p);
 bool codec_packet_copy(struct codec_packet *p);
 
@@ -169,8 +215,8 @@ rtp_payload_type *rtp_payload_type_dup(const rtp_payload_type *pt);
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(rtp_payload_type, payload_type_free)
 
 
-// special return value `(void *) 0x1` to signal type mismatch
 rtp_payload_type *codec_make_payload_type(const str *codec_str, enum media_type);
+bool codec_parse_payload_type(rtp_payload_type *pt, const str *codec_str);
 
 // handle string allocation
 void codec_init_payload_type(rtp_payload_type *, enum media_type);
@@ -182,10 +228,15 @@ struct chu_args {
 	const struct stream_params *sp;
 	bool allow_asymmetric;
 	bool reset_transcoding;
+	// when false (default if not passed) the delay buffer is flushed as normal
+	// used by play-media to ensure media packets aren't sent alongside delayed packets
+	bool clear_delay_buffer;
 };
 #define codec_handlers_update(r, s, ...) \
 	__codec_handlers_update(r, s, (struct chu_args) {__VA_ARGS__})
 
+bool codec_handler_transform(struct call_media *r, ng_codecs_q *);
+
 #ifdef WITH_TRANSCODING
 
 void ensure_codec_def(rtp_payload_type *pt, struct call_media *media);
@@ -194,17 +245,18 @@ __attribute__((nonnull(1, 2)))
 void __codec_handlers_update(struct call_media *receiver, struct call_media *sink, struct chu_args);
 void codec_add_dtmf_event(struct codec_ssrc_handler *ch, int code, int level, uint64_t ts, bool injected);
 uint64_t codec_last_dtmf_event(struct codec_ssrc_handler *ch);
-uint64_t codec_encoder_pts(struct codec_ssrc_handler *ch, struct ssrc_ctx *);
+uint64_t codec_encoder_pts(struct codec_ssrc_handler *ch, struct ssrc_entry_call *);
 void codec_decoder_skip_pts(struct codec_ssrc_handler *ch, uint64_t);
 uint64_t codec_decoder_unskip_pts(struct codec_ssrc_handler *ch);
 void codec_tracker_update(struct codec_store *, struct codec_store *);
-void codec_handlers_stop(codec_handlers_q *, struct call_media *sink);
+void codec_handlers_stop(codec_handlers_q *, struct call_media *sink, bool clear_delay_buffer);
 
 
 void packet_encoded_packetize(AVPacket *pkt, struct codec_ssrc_handler *ch, struct media_packet *mp,
 		packetizer_f pkt_f, void *pkt_f_data, const struct fraction *cr_fact,
-		void (*tx_f)(AVPacket *, struct codec_ssrc_handler *, struct media_packet *, str *,
-			char *, unsigned int, const struct fraction *cr_fact));
+		void (*tx_f)(struct codec_ssrc_handler *, struct media_packet *, str *,
+			char *, unsigned int, int64_t pts, int64_t duration,
+			const struct fraction *cr_fact));
 void codec_output_rtp(struct media_packet *mp, struct codec_scheduler *,
 		struct codec_handler *handler, // normally == ch->handler except for DTMF
 		char *buf, // malloc'd, room for rtp_header + filled-in payload
@@ -214,22 +266,19 @@ void codec_output_rtp(struct media_packet *mp, struct codec_scheduler *,
 		unsigned long ts_delay);
 
 
-INLINE struct codec_handler __codec_handler_lookup_struct(int pt, struct call_media *sink) {
-	struct codec_handler lookup = {
-		.source_pt = {
-			.payload_type = pt,
-		},
+INLINE struct codec_handler_index __codec_handler_lookup_struct(int pt, struct call_media *sink) {
+	struct codec_handler_index lookup = {
+		.payload_type = pt,
 		.sink = sink,
 	};
 	return lookup;
 }
 INLINE struct codec_handler *codec_handler_lookup(codec_handlers_ht ht, int pt, struct call_media *sink) {
-	struct codec_handler lookup = __codec_handler_lookup_struct(pt, sink);
+	struct codec_handler_index lookup = __codec_handler_lookup_struct(pt, sink);
 	return t_hash_table_lookup(ht, &lookup);
 }
 
 
-
 #else
 
 INLINE void __codec_handlers_update(struct call_media *receiver, struct call_media *sink, struct chu_args a)
@@ -237,7 +286,7 @@ INLINE void __codec_handlers_update(struct call_media *receiver, struct call_med
 }
 INLINE void codec_handler_free(struct codec_handler **handler) { }
 INLINE void codec_tracker_update(struct codec_store *cs, struct codec_store *ocs) { }
-INLINE void codec_handlers_stop(codec_handlers_q *q, struct call_media *sink) { }
+INLINE void codec_handlers_stop(codec_handlers_q *q, struct call_media *sink, bool clear_delay_buffer) { }
 INLINE void ensure_codec_def(rtp_payload_type *pt, struct call_media *media) { }
 
 #endif
