@@ -14,6 +14,9 @@
 #include <netinet/in.h>
 #include <sys/resource.h>
 #include <sys/epoll.h>
+#ifdef HAVE_CODEC_CHAIN
+#include <codec-chain/types.h>
+#endif
 #include "log.h"
 #include "loglib.h"
 
@@ -22,7 +25,7 @@ struct thread_buf {
 };
 
 struct rtpengine_common_config *rtpe_common_config_ptr;
-__thread struct timeval rtpe_now;
+__thread int64_t rtpe_now;
 volatile bool rtpe_shutdown;
 
 static __thread struct thread_buf t_bufs[NUM_THREAD_BUFS];
@@ -152,13 +155,67 @@ void config_load_free(struct rtpengine_common_config *cconfig) {
 	g_free(cconfig->pidfile);
 }
 
+static void section_keys_callback(GKeyFile *kf,
+		const char *section_name,
+		void (*callback)(const char *key, char *value, union rtpenging_config_callback_arg),
+		union rtpenging_config_callback_arg arg)
+{
+	if (!section_name)
+		return;
+
+	g_autoptr(GError) err = NULL;
+	g_autoptr(char_p) keys = g_key_file_get_keys(kf, section_name, NULL, &err);
+	if (err)
+		die("Failed to load keys from given config file section '%s': %s", section_name, err->message);
+	if (!keys)
+		return; // empty config section
+
+	for (char **key = keys; *key; key++) {
+		char *val = g_key_file_get_string(kf, section_name, *key, &err);
+		if (err)
+			die("Failed to read config value '%s' (section '%s') from config file: %s", *key, section_name, err->message);
+		callback(*key, val, arg);
+	}
+}
+
+static void file_groups_callback(GKeyFile *kf,
+		const char *group_prefix,
+		void (*callback)(const char *name, charp_ht, union rtpenging_config_callback_arg),
+		union rtpenging_config_callback_arg arg)
+{
+	if (!group_prefix)
+		return;
+
+	size_t pref_len = strlen(group_prefix);
+
+	g_autoptr(char_p) groups = g_key_file_get_groups(kf, NULL);
+
+	for (char **group = groups; *group; group++) {
+		// check for groups starting with "PREFIX-.."
+		if (memcmp(*group, group_prefix, pref_len))
+			continue;
+		char *ident = *group + pref_len;
+		if (*ident != '-')
+			continue;
+		ident++;
+		if (*ident == '\0')
+			continue;
+
+		// read all keys and put them in a hash table
+		g_auto(charp_ht) ht = charp_ht_new();
+		section_keys_callback(kf, *group, add_c_str_to_ht, ht);
+		callback(ident, ht, arg);
+	}
+}
+
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GOptionEntry, free)
 typedef char *char_p_shallow;
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(char_p_shallow, g_free)
 
-void config_load(int *argc, char ***argv, GOptionEntry *app_entries, const char *description,
+void config_load_ext(int *argc, char ***argv, GOptionEntry *app_entries, const char *description,
 		char *default_config, char *default_section,
-		struct rtpengine_common_config *cconfig)
+		struct rtpengine_common_config *cconfig,
+		const struct rtpenging_config_callback *callbacks)
 {
 	g_autoptr(GOptionContext) c = NULL;
 	g_autoptr(GError) er = NULL;
@@ -169,6 +226,7 @@ void config_load(int *argc, char ***argv, GOptionEntry *app_entries, const char
 	g_autoptr(char_p_shallow) saved_argv = __g_memdup(saved_argv_arr, sizeof(char *) * (*argc + 1));
 	int saved_argc = *argc;
 	gboolean version = false;
+	g_autoptr(char) opus_application = NULL;
 
 	rtpe_common_config_ptr = cconfig;
 
@@ -187,7 +245,7 @@ void config_load(int *argc, char ***argv, GOptionEntry *app_entries, const char
 
 	GOptionEntry shared_options[] = {
 		{ "version",		'v', 0, G_OPTION_ARG_NONE,	&version,	"Print build time and exit",		NULL		},
-		{ "config-file",	0,   0, G_OPTION_ARG_STRING,	&rtpe_common_config_ptr->config_file,	"Load config from this file",		"FILE"		},
+		{ "config-file",	0,   0, G_OPTION_ARG_FILENAME,	&rtpe_common_config_ptr->config_file,	"Load config from this file",		"FILE"		},
 		{ "config-section",	0,   0, G_OPTION_ARG_STRING,	&rtpe_common_config_ptr->config_section,"Config file section to use",		"STRING"	},
 		{ "log-facility",	0,   0,	G_OPTION_ARG_STRING,	&rtpe_common_config_ptr->log_facility,	"Syslog facility to use for logging",	"daemon|local0|...|local7"},
 		{ "log-level",		'L', 0, G_OPTION_ARG_INT,	&rtpe_common_config_ptr->default_log_level,"Default log level",			"INT"		},
@@ -213,6 +271,7 @@ void config_load(int *argc, char ***argv, GOptionEntry *app_entries, const char
 		{ "codec-chain-runners",0,0,	G_OPTION_ARG_INT,	&rtpe_common_config_ptr->codec_chain_runners,"Number of chain runners per codec","INT"		},
 		{ "codec-chain-concurrency",0,0,G_OPTION_ARG_INT,	&rtpe_common_config_ptr->codec_chain_concurrency,"Max concurrent codec jobs per runner","INT"	},
 		{ "codec-chain-async",0,0,	G_OPTION_ARG_INT,	&rtpe_common_config_ptr->codec_chain_async,"Number of background callback threads","INT"	},
+		{ "codec-chain-opus-application",0,0,G_OPTION_ARG_STRING,&opus_application,			"Opus application",			"default|VoIP|audio|low-delay"	},
 		{ "codec-chain-opus-complexity",0,0,G_OPTION_ARG_INT,	&rtpe_common_config_ptr->codec_chain_opus_complexity,"Opus encoding complexity (0..10)","INT"	},
 #endif
 		{ NULL, }
@@ -290,6 +349,7 @@ void config_load(int *argc, char ***argv, GOptionEntry *app_entries, const char
 				break;
 			}
 
+			case G_OPTION_ARG_FILENAME_ARRAY:
 			case G_OPTION_ARG_STRING_ARRAY: {
 				char ***s = e->arg_data;
 				g_strfreev(*s);
@@ -333,6 +393,7 @@ void config_load(int *argc, char ***argv, GOptionEntry *app_entries, const char
 				break;
 			}
 
+			case G_OPTION_ARG_FILENAME_ARRAY:
 			case G_OPTION_ARG_STRING_ARRAY: {
 				char ***s = e->arg_data;
 				if (!*s && e->description)
@@ -355,6 +416,22 @@ void config_load(int *argc, char ***argv, GOptionEntry *app_entries, const char
 		}
 	}
 
+	for (const struct rtpenging_config_callback *cb = callbacks; cb; cb++) {
+		switch (cb->type) {
+			case RCC_END:
+				break;
+
+			case RCC_SECTION_KEYS:
+				section_keys_callback(kf, *cb->section_keys.name, cb->section_keys.callback, cb->arg);
+				continue;
+
+			case RCC_FILE_GROUPS:
+				file_groups_callback(kf, *cb->file_groups.prefix, cb->file_groups.callback, cb->arg);
+				continue;
+		}
+		break;
+	}
+
 out:
 	// default common values, if not configured
 	if (rtpe_common_config_ptr->log_name == NULL)
@@ -412,6 +489,18 @@ out:
 		rtpe_common_config_ptr->codec_chain_opus_complexity = 10;
 	if (rtpe_common_config_ptr->codec_chain_opus_complexity < 0 || rtpe_common_config_ptr->codec_chain_opus_complexity > 10)
 		die("Invalid value for --codec-chain-opus-complexity");
+	if (opus_application) {
+		if (!strcmp(opus_application, "default") || !strcmp(opus_application, ""))
+			rtpe_common_config_ptr->codec_chain_opus_application = 0;
+		else if (!strcmp(opus_application, "voip") || !strcmp(opus_application, "speech"))
+			rtpe_common_config_ptr->codec_chain_opus_application = CC_OPUS_APP_VOIP;
+		else if (!strcmp(opus_application, "audio") || !strcmp(opus_application, "music"))
+			rtpe_common_config_ptr->codec_chain_opus_application = CC_OPUS_APP_AUDIO;
+		else if (!strcmp(opus_application, "low delay") || !strcmp(opus_application, "low-delay") || !strcmp(opus_application, "lowdelay"))
+			rtpe_common_config_ptr->codec_chain_opus_application = CC_OPUS_APP_LOWDELAY;
+		else
+			die("Invalid value for --codec-chain-opus-application");
+	}
 #endif
 
 #if HAVE_LIBURING
@@ -448,43 +537,6 @@ int in6_addr_eq(const void *a, const void *b) {
 	return !memcmp(A, B, sizeof(*A));
 }
 
-unsigned int uint32_hash(const void *p) {
-	const uint32_t *a = p;
-	return *a;
-}
-int uint32_eq(const void *a, const void *b) {
-	const uint32_t *A = a, *B = b;
-	return (*A == *B) ? TRUE : FALSE;
-}
-
-int timeval_cmp_zero(const void *a, const void *b) {
-	const struct timeval *A = a, *B = b;
-
-	/* zero timevals go last */
-	if (A->tv_sec == 0 && B->tv_sec != 0)
-		return 1;
-	if (B->tv_sec == 0 && A->tv_sec == 0)
-		return -1;
-	if (A->tv_sec == 0 && B->tv_sec == 0)
-		return 0;
-	/* earlier timevals go first */
-	return timeval_cmp(A, B);
-}
-
-int timeval_cmp_ptr(const void *a, const void *b) {
-	const struct timeval *A = a, *B = b;
-	int ret;
-	ret = timeval_cmp_zero(A, B);
-	if (ret)
-		return ret;
-	/* equal timeval, so use pointer as tie breaker */
-	if (A < B)
-		return -1;
-	if (A > B)
-		return 1;
-	return 0;
-}
-
 int rtpe_tree_find_first_cmp(void *k, void *v, void *d) {
 	struct rtpe_g_tree_find_helper *h = d;
 	if (!h->func || h->func(v, h->data)) {
