@@ -11,6 +11,12 @@
 #include "compat.h"
 
 
+#define MAX_RTP_PACKET_SIZE	8192
+#define RTP_BUFFER_HEAD_ROOM	128
+#define RTP_BUFFER_TAIL_ROOM	512
+#define RTP_BUFFER_SIZE		(MAX_RTP_PACKET_SIZE + RTP_BUFFER_HEAD_ROOM + RTP_BUFFER_TAIL_ROOM)
+
+
 struct obj;
 struct sockaddr;
 
@@ -22,8 +28,7 @@ struct poller_item {
 	struct obj			*obj;
 
 	poller_func_t			readable;
-	void				(*recv)(struct obj *, char *b, size_t len, struct sockaddr *,
-						struct timeval *);
+	void				(*recv)(struct obj *, char *b, size_t len, struct sockaddr *, int64_t);
 	poller_func_t			writeable;
 	poller_func_t			closed;
 };
@@ -50,11 +55,4 @@ extern bool (*rtpe_poller_isblocked)(struct poller *, void *);
 extern void (*rtpe_poller_error)(struct poller *, void *);
 
 
-#ifdef HAVE_LIBURING
-extern __thread unsigned int (*uring_thread_loop)(void);
-#else
-INLINE unsigned int uring_thread_loop(void) { return 0; }
-#endif
-
-
 #endif
