@@ -12,38 +12,39 @@
 #include "xt_RTPENGINE.h"
 #include "log.h"
 
-static int __ip4_addr_parse(sockaddr_t *dst, const char *src);
-static int __ip6_addr_parse(sockaddr_t *dst, const char *src);
-static int __ip4_addr_print(const sockaddr_t *a, char *buf, size_t len);
-static int __ip6_addr_print(const sockaddr_t *a, char *buf, size_t len);
-static int __ip6_addr_print_p(const sockaddr_t *a, char *buf, size_t len);
+static bool __ip4_addr_parse(sockaddr_t *dst, const char *src);
+static bool __ip6_addr_parse(sockaddr_t *dst, const char *src);
+static bool __ip4_addr_print(const sockaddr_t *a, char *buf, size_t len);
+static bool __ip6_addr_print(const sockaddr_t *a, char *buf, size_t len);
+static bool __ip6_addr_print_p(const sockaddr_t *a, char *buf, size_t len);
 static unsigned int __ip4_hash(const sockaddr_t *a);
 static unsigned int __ip6_hash(const sockaddr_t *a);
-static int __ip4_eq(const sockaddr_t *a, const sockaddr_t *b);
-static int __ip6_eq(const sockaddr_t *a, const sockaddr_t *b);
-static int __ip4_is_specified(const sockaddr_t *a);
-static int __ip6_is_specified(const sockaddr_t *a);
-static int __ip_bind(socket_t *s, unsigned int, const sockaddr_t *);
-static int __ip_connect(socket_t *s, const endpoint_t *);
-static int __ip_listen(socket_t *s, int backlog);
-static int __ip_accept(socket_t *s, socket_t *new_sock);
-static int __ip_timestamping(socket_t *s);
-static int __ip4_pktinfo(socket_t *s);
-static int __ip6_pktinfo(socket_t *s);
-static int __ip4_sockaddr2endpoint(endpoint_t *, const void *);
-static int __ip6_sockaddr2endpoint(endpoint_t *, const void *);
-static int __ip4_endpoint2sockaddr(void *, const endpoint_t *);
-static int __ip6_endpoint2sockaddr(void *, const endpoint_t *);
-static int __ip4_addrport2sockaddr(void *, const sockaddr_t *, unsigned int);
-static int __ip6_addrport2sockaddr(void *, const sockaddr_t *, unsigned int);
+static bool __ip4_eq(const sockaddr_t *a, const sockaddr_t *b);
+static bool __ip6_eq(const sockaddr_t *a, const sockaddr_t *b);
+static bool __ip4_is_specified(const sockaddr_t *a);
+static bool __ip6_is_specified(const sockaddr_t *a);
+static bool __ip_bind(socket_t *s, unsigned int, const sockaddr_t *);
+static bool __ip_connect(socket_t *s, const endpoint_t *);
+static bool __ip_listen(socket_t *s, int backlog);
+static bool __ip_accept(socket_t *s, socket_t *new_sock);
+static bool __ip_getsockname(socket_t *s);
+static bool __ip_timestamping(socket_t *s);
+static bool __ip4_pktinfo(socket_t *s);
+static bool __ip6_pktinfo(socket_t *s);
+static bool __ip4_sockaddr2endpoint(endpoint_t *, const void *);
+static bool __ip6_sockaddr2endpoint(endpoint_t *, const void *);
+static bool __ip4_endpoint2sockaddr(void *, const endpoint_t *);
+static bool __ip6_endpoint2sockaddr(void *, const endpoint_t *);
+static bool __ip4_addrport2sockaddr(void *, const sockaddr_t *, unsigned int);
+static bool __ip6_addrport2sockaddr(void *, const sockaddr_t *, unsigned int);
 static ssize_t __ip_recvfrom(socket_t *s, void *buf, size_t len, endpoint_t *ep);
-static ssize_t __ip_recvfrom_ts(socket_t *s, void *buf, size_t len, endpoint_t *ep, struct timeval *);
+static ssize_t __ip_recvfrom_ts(socket_t *s, void *buf, size_t len, endpoint_t *ep, int64_t *);
 static ssize_t __ip4_recvfrom_to(socket_t *s, void *buf, size_t len, endpoint_t *ep, sockaddr_t *to);
 static ssize_t __ip6_recvfrom_to(socket_t *s, void *buf, size_t len, endpoint_t *ep, sockaddr_t *to);
 static ssize_t __ip_sendmsg(socket_t *s, struct msghdr *mh, const endpoint_t *ep);
 static ssize_t __ip_sendto(socket_t *s, const void *buf, size_t len, const endpoint_t *ep);
-static int __ip4_tos(socket_t *, unsigned int);
-static int __ip6_tos(socket_t *, unsigned int);
+static bool __ip4_tos(socket_t *, unsigned int);
+static bool __ip6_tos(socket_t *, unsigned int);
 static int __ip_error(socket_t *s);
 static void __ip4_pmtu_disc(socket_t *, int);
 static void __ip4_endpoint2kernel(struct re_address *, const endpoint_t *);
@@ -87,6 +88,7 @@ static struct socket_family __socket_families[__SF_LAST] = {
 		.connect		= __ip_connect,
 		.listen			= __ip_listen,
 		.accept			= __ip_accept,
+		.getsockname		= __ip_getsockname,
 		.timestamping		= __ip_timestamping,
 		.pktinfo		= __ip4_pktinfo,
 		.recvfrom		= __ip_recvfrom,
@@ -122,6 +124,7 @@ static struct socket_family __socket_families[__SF_LAST] = {
 		.connect		= __ip_connect,
 		.listen			= __ip_listen,
 		.accept			= __ip_accept,
+		.getsockname		= __ip_getsockname,
 		.timestamping		= __ip_timestamping,
 		.pktinfo		= __ip6_pktinfo,
 		.recvfrom		= __ip_recvfrom,
@@ -145,22 +148,22 @@ socktype_t *socktype_udp;
 
 
 
-static int __ip4_addr_parse(sockaddr_t *dst, const char *src) {
+static bool __ip4_addr_parse(sockaddr_t *dst, const char *src) {
 	if (inet_pton(AF_INET, src, &dst->ipv4) == 1)
-		return 0;
-	return -1;
+		return true;
+	return false;
 }
-static int __ip6_addr_parse(sockaddr_t *dst, const char *src) {
+static bool __ip6_addr_parse(sockaddr_t *dst, const char *src) {
 	if (src[0] != '[') {
 		if (inet_pton(AF_INET6, src, &dst->ipv6) == 1)
-			return 0;
-		return -1;
+			return true;
+		return false;
 	}
 
 	const char *ep;
 	ep = strchr(src, ']');
 	if (!ep)
-		return -1;
+		return false;
 
 	unsigned int len = ep - src - 1;
 	char buf[64];
@@ -168,28 +171,28 @@ static int __ip6_addr_parse(sockaddr_t *dst, const char *src) {
 	buf[len] = '\0';
 
 	if (inet_pton(AF_INET6, buf, &dst->ipv6) == 1)
-		return 0;
-	return -1;
+		return true;
+	return false;
 }
-static int __ip4_addr_print(const sockaddr_t *a, char *buf, size_t len) {
+static bool __ip4_addr_print(const sockaddr_t *a, char *buf, size_t len) {
 	buf[0] = '\0';
 	if (!inet_ntop(AF_INET, &a->ipv4, buf, len))
-		return -1;
-	return 0;
+		return false;
+	return true;
 }
-static int __ip6_addr_print(const sockaddr_t *a, char *buf, size_t len) {
+static bool __ip6_addr_print(const sockaddr_t *a, char *buf, size_t len) {
 	buf[0] = '\0';
 	if (!inet_ntop(AF_INET6, &a->ipv6, buf, len))
-		return -1;
-	return 0;
+		return false;
+	return true;
 }
-static int __ip6_addr_print_p(const sockaddr_t *a, char *buf, size_t len) {
+static bool __ip6_addr_print_p(const sockaddr_t *a, char *buf, size_t len) {
 	buf[0] = '\0';
 	if (!inet_ntop(AF_INET6, &a->ipv6, buf+1, len-2))
-		return -1;
+		return false;
 	buf[0] = '[';
 	strcpy(buf + strlen(buf), "]");
-	return 0;
+	return true;
 }
 static unsigned int __ip4_hash(const sockaddr_t *a) {
 	return a->ipv4.s_addr;
@@ -197,40 +200,40 @@ static unsigned int __ip4_hash(const sockaddr_t *a) {
 static unsigned int __ip6_hash(const sockaddr_t *a) {
 	return in6_addr_hash(&a->ipv6);
 }
-static int __ip4_eq(const sockaddr_t *a, const sockaddr_t *b) {
-	return !memcmp(&a->ipv4, &b->ipv4, sizeof(a->ipv4));
+static bool __ip4_eq(const sockaddr_t *a, const sockaddr_t *b) {
+	return memcmp(&a->ipv4, &b->ipv4, sizeof(a->ipv4)) == 0;
 }
-static int __ip6_eq(const sockaddr_t *a, const sockaddr_t *b) {
-	return !memcmp(&a->ipv6, &b->ipv6, sizeof(a->ipv6));
+static bool __ip6_eq(const sockaddr_t *a, const sockaddr_t *b) {
+	return memcmp(&a->ipv6, &b->ipv6, sizeof(a->ipv6)) == 0;
 }
-static int __ip4_is_specified(const sockaddr_t *a) {
+static bool __ip4_is_specified(const sockaddr_t *a) {
 	return a->ipv4.s_addr != 0;
 }
-static int __ip6_is_specified(const sockaddr_t *a) {
+static bool __ip6_is_specified(const sockaddr_t *a) {
 	return a->ipv6.s6_addr32[0] != 0
 		|| a->ipv6.s6_addr32[1] != 0
 		|| a->ipv6.s6_addr32[2] != 0
 		|| a->ipv6.s6_addr32[3] != 0;
 }
-static int __ip4_sockaddr2endpoint(endpoint_t *ep, const void *p) {
+static bool __ip4_sockaddr2endpoint(endpoint_t *ep, const void *p) {
 	const struct sockaddr_in *sin = p;
 	if (sin->sin_family != AF_INET)
-		return -1;
+		return false;
 	ZERO(*ep);
 	ep->address.family = &__socket_families[SF_IP4];
 	ep->address.ipv4 = sin->sin_addr;
 	ep->port = ntohs(sin->sin_port);
-	return 0;
+	return true;
 }
-static int __ip6_sockaddr2endpoint(endpoint_t *ep, const void *p) {
+static bool __ip6_sockaddr2endpoint(endpoint_t *ep, const void *p) {
 	const struct sockaddr_in6 *sin = p;
 	if (sin->sin6_family != AF_INET6)
-		return -1;
+		return false;
 	ZERO(*ep);
 	ep->address.family = &__socket_families[SF_IP6];
 	ep->address.ipv6 = sin->sin6_addr;
 	ep->port = ntohs(sin->sin6_port);
-	return 0;
+	return true;
 }
 void endpoint_parse_sockaddr_storage(endpoint_t *ep, struct sockaddr_storage *sa) {
 	if (sa->ss_family == AF_INET)
@@ -240,59 +243,59 @@ void endpoint_parse_sockaddr_storage(endpoint_t *ep, struct sockaddr_storage *sa
 	else
 		abort();
 }
-static int __ip4_endpoint2sockaddr(void *p, const endpoint_t *ep) {
+static bool __ip4_endpoint2sockaddr(void *p, const endpoint_t *ep) {
 	return __ip4_addrport2sockaddr(p, &ep->address, ep->port);
 }
-static int __ip6_endpoint2sockaddr(void *p, const endpoint_t *ep) {
+static bool __ip6_endpoint2sockaddr(void *p, const endpoint_t *ep) {
 	return __ip6_addrport2sockaddr(p, &ep->address, ep->port);
 }
-static int __ip4_addrport2sockaddr(void *p, const sockaddr_t *sa, unsigned int port) {
+static bool __ip4_addrport2sockaddr(void *p, const sockaddr_t *sa, unsigned int port) {
 	struct sockaddr_in *sin = p;
 	ZERO(*sin);
 	sin->sin_family = AF_INET;
 	sin->sin_port = htons(port);
 	if (sa)
 		sin->sin_addr = sa->ipv4;
-	return 0;
+	return true;
 }
-static int __ip6_addrport2sockaddr(void *p, const sockaddr_t *sa, unsigned int port) {
+static bool __ip6_addrport2sockaddr(void *p, const sockaddr_t *sa, unsigned int port) {
 	struct sockaddr_in6 *sin = p;
 	ZERO(*sin);
 	sin->sin6_family = AF_INET6;
 	sin->sin6_port = htons(port);
 	if (sa)
 		sin->sin6_addr = sa->ipv6;
-	return 0;
+	return true;
 }
-static int __ip_bind(socket_t *s, unsigned int port, const sockaddr_t *a) {
+static bool __ip_bind(socket_t *s, unsigned int port, const sockaddr_t *a) {
 	struct sockaddr_storage sin;
 
 	s->family->addrport2sockaddr(&sin, a, port);
 	if (bind(s->fd, (struct sockaddr *) &sin, s->family->sockaddr_size)) {
 		__C_DBG("bind fail, fd=%d, port=%d", s->fd, s->local.port);
-		return -1;
+		return false;
 	} else {
 		__C_DBG("bind success, fd=%d, port=%d", s->fd, s->local.port);
 	}
 
-	return 0;
+	return true;
 }
-static int __ip_connect(socket_t *s, const endpoint_t *ep) {
+static bool __ip_connect(socket_t *s, const endpoint_t *ep) {
 	struct sockaddr_storage sin;
 
 	s->family->endpoint2sockaddr(&sin, ep);
 	if (connect(s->fd, (struct sockaddr *) &sin, s->family->sockaddr_size)) {
 		__C_DBG("connect fail, fd=%d, port=%d", s->fd, s->local.port);
-		return -1;
+		return false;
 	} else {
 		__C_DBG("connect success, fd=%d, port=%d", s->fd, s->local.port);
 	}
-	return 0;
+	return true;
 }
-static int __ip_listen(socket_t *s, int backlog) {
-	return listen(s->fd, backlog);
+static bool __ip_listen(socket_t *s, int backlog) {
+	return listen(s->fd, backlog) == 0;
 }
-static int __ip_accept(socket_t *s, socket_t *newsock) {
+static bool __ip_accept(socket_t *s, socket_t *newsock) {
 	int nfd;
 	struct sockaddr_storage sin;
 	socklen_t sinlen;
@@ -303,7 +306,7 @@ static int __ip_accept(socket_t *s, socket_t *newsock) {
 	nfd = accept(s->fd, (struct sockaddr *) &sin, &sinlen);
 	if (nfd == -1) {
 		__C_DBG("accept fail, fd=%d, port=%d", s->fd, s->local.port);
-		return -1;
+		return false;
 	}
 
 	newsock->fd = nfd;
@@ -311,9 +314,18 @@ static int __ip_accept(socket_t *s, socket_t *newsock) {
 	newsock->local = s->local;
 	s->family->sockaddr2endpoint(&newsock->remote, &sin);
 
-	return 0;
+	return true;
 }
-INLINE ssize_t __ip_recvfrom_options(socket_t *s, void *buf, size_t len, endpoint_t *ep, struct timeval *tv,
+static bool __ip_getsockname(socket_t *s) {
+	struct sockaddr_storage sin;
+	socklen_t sinlen = sizeof(sin);
+	int ret = getsockname(s->fd, (struct sockaddr *) &sin, &sinlen);
+	if (ret != 0)
+		return false;
+	s->family->sockaddr2endpoint(&s->local, &sin);
+	return true;
+}
+INLINE ssize_t __ip_recvfrom_options(socket_t *s, void *buf, size_t len, endpoint_t *ep, int64_t *tv,
 		sockaddr_t *to, bool (*parse)(struct cmsghdr *, sockaddr_t *))
 {
 	ssize_t ret;
@@ -323,8 +335,10 @@ INLINE ssize_t __ip_recvfrom_options(socket_t *s, void *buf, size_t len, endpoin
 	char ctrl[64];
 
 	ZERO(msg);
-	msg.msg_name = &sin;
-	msg.msg_namelen = s->family->sockaddr_size;
+	if (ep) {
+		msg.msg_name = &sin;
+		msg.msg_namelen = s->family->sockaddr_size;
+	}
 	msg.msg_iov = &iov;
 	msg.msg_iovlen = 1;
 	msg.msg_control = ctrl;
@@ -336,13 +350,15 @@ INLINE ssize_t __ip_recvfrom_options(socket_t *s, void *buf, size_t len, endpoin
 	ret = recvmsg(s->fd, &msg, 0);
 	if (ret < 0)
 		return ret;
-	s->family->sockaddr2endpoint(ep, &sin);
+
+	if (ep)
+		s->family->sockaddr2endpoint(ep, &sin);
 
 	socket_recvfrom_parse_cmsg(&tv, &to, parse, &msg, CMSG_FIRSTHDR(&msg), CMSG_NXTHDR(&msg, cm));
 
 	return ret;
 }
-static ssize_t __ip_recvfrom_ts(socket_t *s, void *buf, size_t len, endpoint_t *ep, struct timeval *tv) {
+static ssize_t __ip_recvfrom_ts(socket_t *s, void *buf, size_t len, endpoint_t *ep, int64_t *tv) {
 	return __ip_recvfrom_options(s, buf, len, ep, tv, NULL, NULL);
 }
 static ssize_t __ip_recvfrom(socket_t *s, void *buf, size_t len, endpoint_t *ep) {
@@ -373,9 +389,11 @@ static ssize_t __ip6_recvfrom_to(socket_t *s, void *buf, size_t len, endpoint_t
 static ssize_t __ip_sendmsg(socket_t *s, struct msghdr *mh, const endpoint_t *ep) {
 	struct sockaddr_storage sin;
 
-	s->family->endpoint2sockaddr(&sin, ep);
-	mh->msg_name = &sin;
-	mh->msg_namelen = s->family->sockaddr_size;
+	if (ep) {
+		s->family->endpoint2sockaddr(&sin, ep);
+		mh->msg_name = &sin;
+		mh->msg_namelen = s->family->sockaddr_size;
+	}
 
 	return sendmsg(s->fd, mh, 0);
 }
@@ -387,17 +405,17 @@ static ssize_t __ip_sendto(socket_t *s, const void *buf, size_t len, const endpo
 	ep->address.family->endpoint2sockaddr(&sin, ep);
 	return sendto(s->fd, buf, len, 0, (void *) &sin, ep->address.family->sockaddr_size);
 }
-static int __ip4_tos(socket_t *s, unsigned int tos) {
+static bool __ip4_tos(socket_t *s, unsigned int tos) {
 	unsigned char ctos;
 	ctos = tos;
 	if (setsockopt(s->fd, IPPROTO_IP, IP_TOS, &ctos, sizeof(ctos)))
 		ilog(LOG_ERR, "Failed to set TOS on IPv4 socket: %s", strerror(errno));
-	return 0;
+	return true;
 }
-static int __ip6_tos(socket_t *s, unsigned int tos) {
+static bool __ip6_tos(socket_t *s, unsigned int tos) {
 	if (setsockopt(s->fd, IPPROTO_IPV6, IPV6_TCLASS, &tos, sizeof(tos)))
 		ilog(LOG_ERR, "Failed to set TOS on IPv6 socket: %s", strerror(errno));
-	return 0;
+	return true;
 }
 static int __ip_error(socket_t *s) {
 	int optval;
@@ -410,17 +428,17 @@ static void __ip4_pmtu_disc(socket_t *s, int opt) {
 	if (setsockopt(s->fd, IPPROTO_IP, IP_MTU_DISCOVER, &opt, sizeof(opt)))
 		ilog(LOG_ERR, "Failed to set PMTU discovery option on IPv4 socket: %s", strerror(errno));
 }
-static int __ip_timestamping(socket_t *s) {
+static bool __ip_timestamping(socket_t *s) {
 	int one = 1;
 	if (setsockopt(s->fd, SOL_SOCKET, SO_TIMESTAMP, &one, sizeof(one)))
-		return -1;
-	return 0;
+		return false;
+	return true;
 }
-static int __ip4_pktinfo(socket_t *s) {
+static bool __ip4_pktinfo(socket_t *s) {
 	int one = 1;
 	if (setsockopt(s->fd, IPPROTO_IP, IP_PKTINFO, &one, sizeof(one)))
-		return -1;
-	return 0;
+		return false;
+	return true;
 }
 static void __ip4_cmsg_pktinfo(struct cmsghdr *cm, const sockaddr_t *addr) {
 	cm->cmsg_level = IPPROTO_IP;
@@ -430,11 +448,11 @@ static void __ip4_cmsg_pktinfo(struct cmsghdr *cm, const sockaddr_t *addr) {
 	pi->ipi_spec_dst = addr->ipv4;
 	cm->cmsg_len = CMSG_LEN(sizeof(*pi));
 }
-static int __ip6_pktinfo(socket_t *s) {
+static bool __ip6_pktinfo(socket_t *s) {
 	int one = 1;
 	if (setsockopt(s->fd, IPPROTO_IPV6, IPV6_RECVPKTINFO, &one, sizeof(one)))
-		return -1;
-	return 0;
+		return false;
+	return true;
 }
 static void __ip6_cmsg_pktinfo(struct cmsghdr *cm, const sockaddr_t *addr) {
 	cm->cmsg_level = IPPROTO_IPV6;
@@ -526,7 +544,7 @@ static unsigned int __ip6_packet_header(unsigned char *out, const endpoint_t *sr
 
 
 unsigned int sockaddr_hash(const sockaddr_t *a) {
-	return a->family->hash(a) ^ g_direct_hash(a->family);
+	return a->family->hash(a) ^ GPOINTER_TO_UINT(a->family);
 }
 bool sockaddr_eq(const sockaddr_t *a, const sockaddr_t *b) {
 	return a->family == b->family && a->family->eq(a, b);
@@ -548,34 +566,34 @@ gboolean endpoint_eq(const endpoint_t *a, const endpoint_t *b) {
 
 
 
-int sockaddr_parse_any(sockaddr_t *dst, const char *src) {
+bool sockaddr_parse_any(sockaddr_t *dst, const char *src) {
 	int i;
 	sockfamily_t *fam;
 
 	for (i = 0; i < __SF_LAST; i++) {
 		fam = &__socket_families[i];
-		if (!fam->addr_parse(dst, src)) {
+		if (fam->addr_parse(dst, src)) {
 			dst->family = fam;
-			return 0;
+			return true;
 		}
 	}
-	return -1;
+	return false;
 }
-int sockaddr_parse_any_str(sockaddr_t *dst, const str *src) {
+bool sockaddr_parse_any_str(sockaddr_t *dst, const str *src) {
 	char buf[64];
 	if (!src || !src->s)
-		return -1;
+		return false;
 	if (src->len >= sizeof(buf))
-		return -1;
+		return false;
 	sprintf(buf, STR_FORMAT, STR_FMT(src));
 	return sockaddr_parse_any(dst, buf);
 }
-int sockaddr_parse_str(sockaddr_t *dst, sockfamily_t *fam, const str *src) {
+bool sockaddr_parse_str(sockaddr_t *dst, sockfamily_t *fam, const str *src) {
 	char buf[64];
 	if (src->len >= sizeof(buf))
-		return -1;
+		return false;
 	if (!fam)
-		return -1;
+		return false;
 	sprintf(buf, STR_FORMAT, STR_FMT(src));
 	dst->family = fam;
 	return fam->addr_parse(dst, buf);
@@ -594,7 +612,7 @@ sockfamily_t *get_socket_family_rfc(const str *s) {
 sockfamily_t *__get_socket_family_enum(enum socket_families i) {
 	return &__socket_families[i];
 }
-int endpoint_parse_any(endpoint_t *d, const char *s) {
+bool endpoint_parse_any(endpoint_t *d, const char *s) {
 	int i;
 	sockfamily_t *fam;
 	unsigned int len;
@@ -604,29 +622,29 @@ int endpoint_parse_any(endpoint_t *d, const char *s) {
 	ep = strrchr(s, ':');
 	if (!ep) {
 		if (strchr(s, '.'))
-			return -1;
+			return false;
 		/* just a port number */
 		d->port = atoi(s);
 		ZERO(d->address);
 		d->address.family = __get_socket_family_enum(SF_IP4);
-		return 0;
+		return true;
 	}
 	len = ep - s;
 	if (len >= sizeof(buf))
-		return -1;
+		return false;
 	d->port = atoi(ep+1);
 	if (d->port > 0xffff)
-		return -1;
+		return false;
 	sprintf(buf, "%.*s", len, s);
 
 	for (i = 0; i < __SF_LAST; i++) {
 		fam = &__socket_families[i];
-		if (!fam->addr_parse(&d->address, buf)) {
+		if (fam->addr_parse(&d->address, buf)) {
 			d->address.family = fam;
-			return 0;
+			return true;
 		}
 	}
-	return -1;
+	return false;
 }
 
 static int socket_addrinfo_convert(sockaddr_t *a, struct addrinfo *res) {
@@ -644,10 +662,10 @@ static int socket_addrinfo_convert(sockaddr_t *a, struct addrinfo *res) {
 		return -1;
 	return 0;
 }
-int sockaddr_getaddrinfo_alt(sockaddr_t *a, sockaddr_t *a2, const char *s) {
+bool sockaddr_getaddrinfo_alt(sockaddr_t *a, sockaddr_t *a2, const char *s) {
 	struct addrinfo hints, *res;
 	int status;
-	int ret;
+	bool ret;
 
 	memset(&hints, 0, sizeof hints);
 	hints.ai_family = AF_UNSPEC;
@@ -656,15 +674,15 @@ int sockaddr_getaddrinfo_alt(sockaddr_t *a, sockaddr_t *a2, const char *s) {
 
 	if ((status = getaddrinfo(s, NULL, &hints, &res)) != 0) {
 		__C_DBG("getaddrinfo failed for %s, status is \"%s\"\n", s, gai_strerror(status));
-		return -1;
+		return false;
 	}
 
-	ret = socket_addrinfo_convert(a, res);
+	ret = socket_addrinfo_convert(a, res) == 0 ? true : false;
 
 	if (a2) {
-		if (ret == 0 && res->ai_next) {
+		if (ret == true && res->ai_next) {
 			struct addrinfo *next = res->ai_next;
-			ret = socket_addrinfo_convert(a2, next);
+			ret = socket_addrinfo_convert(a2, next) == 0 ? true : false;
 		}
 		else
 			ZERO(*a2);
@@ -674,7 +692,7 @@ int sockaddr_getaddrinfo_alt(sockaddr_t *a, sockaddr_t *a2, const char *s) {
 	return ret;
 }
 
-int endpoint_parse_any_getaddrinfo_alt(endpoint_t *d, endpoint_t *d2, const char *s) {
+bool endpoint_parse_any_getaddrinfo_alt(endpoint_t *d, endpoint_t *d2, const char *s) {
 	unsigned int len;
 	const char *ep;
 	char buf[64];
@@ -682,7 +700,7 @@ int endpoint_parse_any_getaddrinfo_alt(endpoint_t *d, endpoint_t *d2, const char
 	ep = strrchr(s, ':');
 	if (!ep) {
 		if (strchr(s, '.'))
-			return -1;
+			return false;
 		/* just a port number */
 		d->port = atoi(s);
 		ZERO(d->address);
@@ -692,14 +710,14 @@ int endpoint_parse_any_getaddrinfo_alt(endpoint_t *d, endpoint_t *d2, const char
 			*d2 = *d;
 			ipv46_any_convert(d2);
 		}
-		return 0;
+		return true;
 	}
 	len = ep - s;
 	if (len >= sizeof(buf))
-		return -1;
+		return false;
 	d->port = atoi(ep+1);
 	if (d->port > 0xffff)
-		return -1;
+		return false;
 
 	/* original s was [IPv6]:port */
 	if ((len > 2) && (s[0] == '[') && (s[len - 1] == ']')) {
@@ -708,8 +726,8 @@ int endpoint_parse_any_getaddrinfo_alt(endpoint_t *d, endpoint_t *d2, const char
 		sprintf(buf, "%.*s", len, s);
 	}
 
-	if (sockaddr_getaddrinfo_alt(&d->address, d2 ? &d2->address : NULL, buf))
-		return -1;
+	if (!sockaddr_getaddrinfo_alt(&d->address, d2 ? &d2->address : NULL, buf))
+		return false;
 
 	if (d2) {
 		if (d2->address.family)
@@ -718,31 +736,41 @@ int endpoint_parse_any_getaddrinfo_alt(endpoint_t *d, endpoint_t *d2, const char
 			ZERO(*d2);
 	}
 
-	return 0;
+	return true;
 }
 
-static int __socket(socket_t *r, int type, sockfamily_t *fam) {
+static bool __socket(socket_t *r, int type, sockfamily_t *fam) {
 	ZERO(*r);
 	r->family = fam;
 	r->fd = socket(fam->af, type, 0);
 	if (r->fd == -1) {
 		__C_DBG("socket() syscall fail, fd=%d", r->fd);
-		return -1;
+		return false;
 	} else {
 		__C_DBG("socket() syscall success, fd=%d", r->fd);
 	}
 
-	return 0;
+	return true;
 }
 
-int open_socket(socket_t *r, int type, unsigned int port, const sockaddr_t *sa) {
+INLINE void reuseaddr(int fd) {
+	int one = 1;
+	// coverity[check_return : FALSE]
+	setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &one, sizeof(one));
+}
+INLINE void ipv6only(int fd, int yn) {
+	// coverity[check_return : FALSE]
+	setsockopt(fd, IPPROTO_IPV6, IPV6_V6ONLY, &yn, sizeof(yn));
+}
+
+bool open_socket(socket_t *r, int type, unsigned int port, const sockaddr_t *sa) {
 	sockfamily_t *fam;
 
 	fam = sa->family;
 
-	if (__socket(r, type, fam)) {
+	if (!__socket(r, type, fam)) {
 		__C_DBG("open socket fail, fd=%d", r->fd);
-		return -1;
+		return false;
 	}
 
 	nonblock(r->fd);
@@ -755,7 +783,7 @@ int open_socket(socket_t *r, int type, unsigned int port, const sockaddr_t *sa)
 		goto fail;
 	}
 
-	if (fam->bind(r, port, sa)) {
+	if (!fam->bind(r, port, sa)) {
 		__C_DBG("open socket fail, fd=%d, port=%d", r->fd, port);
 		goto fail;
 	}
@@ -765,21 +793,21 @@ int open_socket(socket_t *r, int type, unsigned int port, const sockaddr_t *sa)
 
 	__C_DBG("open socket success, fd=%d, port=%d", r->fd, port);
 
-	return 0;
+	return true;
 
 fail:
 	close_socket(r);
-	return -1;
+	return false;
 }
 
-int open_v46_socket(socket_t *r, int type) {
-	int ret = __socket(r, type, &__socket_families[SF_IP6]);
-	if (ret) {
+bool open_v46_socket(socket_t *r, int type) {
+	bool ret = __socket(r, type, &__socket_families[SF_IP6]);
+	if (!ret) {
 		if (errno == EAFNOSUPPORT)
 			ret = __socket(r, type, &__socket_families[SF_IP4]);
-		if (ret) {
+		if (!ret) {
 			__C_DBG("open socket fail");
-			return -1;
+			return false;
 		}
 	}
 	else
@@ -787,7 +815,7 @@ int open_v46_socket(socket_t *r, int type) {
 
 	nonblock(r->fd);
 
-	return 0;
+	return true;
 }
 
 void dummy_socket(socket_t *r, const sockaddr_t *sa) {
@@ -798,29 +826,29 @@ void dummy_socket(socket_t *r, const sockaddr_t *sa) {
 	r->remote.address.family = sa->family;
 }
 
-int connect_socket(socket_t *r, int type, const endpoint_t *ep) {
+bool connect_socket(socket_t *r, int type, const endpoint_t *ep) {
 	sockfamily_t *fam;
 
 	fam = ep->address.family;
 
-	if (__socket(r, type, fam))
-		return -1;
-	if (fam->connect(r, ep))
+	if (!__socket(r, type, fam))
+		return false;
+	if (!fam->connect(r, ep))
 		goto fail;
 
 	r->remote = *ep;
 
-	return 0;
+	return true;
 
 fail:
 	close_socket(r);
-	return -1;
+	return false;
 }
 
 int connect_socket_retry(socket_t *r) {
 	int ret = 0;
 
-	if (r->family->connect(r, &r->remote)) {
+	if (!r->family->connect(r, &r->remote)) {
 		if (errno != EINPROGRESS && errno != EALREADY && errno != EISCONN)
 			goto fail;
 		if (errno != EISCONN)
@@ -839,7 +867,7 @@ int connect_socket_nb(socket_t *r, int type, const endpoint_t *ep) {
 
 	fam = ep->address.family;
 
-	if (__socket(r, type, fam))
+	if (!__socket(r, type, fam))
 		return -1;
 	nonblock(r->fd);
 	r->remote = *ep;
@@ -847,36 +875,36 @@ int connect_socket_nb(socket_t *r, int type, const endpoint_t *ep) {
 	return connect_socket_retry(r);
 }
 
-int reset_socket(socket_t *r) {
+bool reset_socket(socket_t *r) {
 	if (!r)
-		return -1;
+		return false;
 
 	r->fd = -1;
 	ZERO(r->local);
 	ZERO(r->remote);
 
-	return 0;
+	return true;
 }
-int close_socket(socket_t *r) {
+bool close_socket(socket_t *r) {
 	if (!r) {
 		__C_DBG("close() syscall not called, no socket");
-		return -1;
+		return false;
 	}
 	if (r->fd == -1) {
 		__C_DBG("close() syscall not called, fd=%d", r->fd);
-		return -1;
+		return false;
 	}
 
 	if (close(r->fd) != 0) {
 		__C_DBG("close() syscall fail, fd=%d", r->fd);
-		return -1;
+		return false;
 	}
 
 	__C_DBG("close() syscall success, fd=%d", r->fd);
 
 	reset_socket(r);
 
-	return 0;
+	return true;
 }
 
 // moves the contents of the socket object:
