@@ -13,7 +13,6 @@
 
 
 
-
 enum socket_families {
 	SF_IP4 = 0,
 	SF_IP6,
@@ -62,26 +61,27 @@ struct socket_family {
 	const char			*rfc_name; /* "IP4" */
 	const char			*unspec_string; /* 0.0.0.0 or :: */
 	unsigned int			(*hash)(const sockaddr_t *);
-	int				(*eq)(const sockaddr_t *, const sockaddr_t *);
-	int				(*addr_parse)(sockaddr_t *, const char *);
-	int				(*addr_print)(const sockaddr_t *, char *, size_t);
-	int				(*addr_print_p)(const sockaddr_t *, char *, size_t);
-	int				(*is_specified)(const sockaddr_t *);
-	int				(*sockaddr2endpoint)(endpoint_t *, const void *);
-	int				(*endpoint2sockaddr)(void *, const endpoint_t *);
-	int				(*addrport2sockaddr)(void *, const sockaddr_t *, unsigned int);
-	int				(*bind)(socket_t *, unsigned int, const sockaddr_t *);
-	int				(*connect)(socket_t *, const endpoint_t *);
-	int				(*listen)(socket_t *, int);
-	int				(*accept)(socket_t *, socket_t *);
-	int				(*timestamping)(socket_t *);
-	int				(*pktinfo)(socket_t *);
+	bool				(*eq)(const sockaddr_t *, const sockaddr_t *);
+	bool				(*addr_parse)(sockaddr_t *, const char *);
+	bool				(*addr_print)(const sockaddr_t *, char *, size_t);
+	bool				(*addr_print_p)(const sockaddr_t *, char *, size_t);
+	bool				(*is_specified)(const sockaddr_t *);
+	bool				(*sockaddr2endpoint)(endpoint_t *, const void *);
+	bool				(*endpoint2sockaddr)(void *, const endpoint_t *);
+	bool				(*addrport2sockaddr)(void *, const sockaddr_t *, unsigned int);
+	bool				(*bind)(socket_t *, unsigned int, const sockaddr_t *);
+	bool				(*connect)(socket_t *, const endpoint_t *);
+	bool				(*listen)(socket_t *, int);
+	bool				(*accept)(socket_t *, socket_t *);
+	bool				(*getsockname)(socket_t *);
+	bool				(*timestamping)(socket_t *);
+	bool				(*pktinfo)(socket_t *);
 	ssize_t				(*recvfrom)(socket_t *, void *, size_t, endpoint_t *);
-	ssize_t				(*recvfrom_ts)(socket_t *, void *, size_t, endpoint_t *, struct timeval *);
+	ssize_t				(*recvfrom_ts)(socket_t *, void *, size_t, endpoint_t *, int64_t *);
 	ssize_t				(*recvfrom_to)(socket_t *, void *, size_t, endpoint_t *, sockaddr_t *);
 	ssize_t				(*sendmsg)(socket_t *, struct msghdr *, const endpoint_t *);
 	ssize_t				(*sendto)(socket_t *, const void *, size_t, const endpoint_t *);
-	int				(*tos)(socket_t *, unsigned int);
+	bool				(*tos)(socket_t *, unsigned int);
 	void				(*pmtu_disc)(socket_t *, int);
 	int				(*error)(socket_t *);
 	void				(*endpoint2kernel)(struct re_address *, const endpoint_t *);
@@ -118,10 +118,10 @@ extern socktype_t *socktype_udp;
 #include "auxlib.h"
 
 
-INLINE int sockaddr_print(const sockaddr_t *a, char *buf, size_t len) {
+INLINE bool sockaddr_print(const sockaddr_t *a, char *buf, size_t len) {
 	if (!a->family) {
 		buf[0] = '\0';
-		return 0;
+		return true;
 	}
 	return a->family->addr_print(a, buf, len);
 }
@@ -134,10 +134,19 @@ INLINE char *sockaddr_print_buf(const sockaddr_t *a) {
 	sockaddr_print(a, buf, THREAD_BUF_SIZE);
 	return buf;
 }
-INLINE int sockaddr_print_p(const sockaddr_t *a, char *buf, size_t len) {
+INLINE bool sockaddr_print_gstring(GString *s, const sockaddr_t *a) {
+	if (!a->family)
+		return true;
+	char buf[THREAD_BUF_SIZE];
+	if (!sockaddr_print(a, buf, THREAD_BUF_SIZE))
+		return false;
+	g_string_append(s, buf);
+	return true;
+}
+INLINE bool sockaddr_print_p(const sockaddr_t *a, char *buf, size_t len) {
 	if (!a->family) {
 		buf[0] = '\0';
-		return 0;
+		return true;
 	}
 	return a->family->addr_print_p(a, buf, len);
 }
@@ -146,30 +155,30 @@ INLINE char *sockaddr_print_p_buf(const sockaddr_t *a) {
 	sockaddr_print_p(a, buf, THREAD_BUF_SIZE);
 	return buf;
 }
-INLINE int sockaddr_print_port(const sockaddr_t *a, unsigned int port, char *buf, size_t len) {
+INLINE bool sockaddr_print_port(const sockaddr_t *a, unsigned int port, char *buf, size_t len) {
 	if (!a->family) {
 		buf[0] = '\0';
-		return 0;
+		return true;
 	}
-	if (a->family->addr_print_p(a, buf, len-6))
-		return -1;
+	if (!a->family->addr_print_p(a, buf, len-6))
+		return false;
 	sprintf(buf + strlen(buf), ":%u", port);
-	return 0;
+	return true;
 }
 INLINE char *sockaddr_print_port_buf(const sockaddr_t *a, unsigned int port) {
 	char *buf = get_thread_buf();
 	sockaddr_print_port(a, port, buf, THREAD_BUF_SIZE);
 	return buf;
 }
-INLINE int endpoint_print(const endpoint_t *ep, char *buf, size_t len) {
+INLINE bool endpoint_print(const endpoint_t *ep, char *buf, size_t len) {
 	return sockaddr_print_port(&ep->address, ep->port, buf, len);
 }
 INLINE char *endpoint_print_buf(const endpoint_t *ep) {
 	return sockaddr_print_port_buf(&ep->address, ep->port);
 }
-INLINE int is_addr_unspecified(const sockaddr_t *a) {
+INLINE bool is_addr_unspecified(const sockaddr_t *a) {
 	if (!a || !a->family)
-		return 1;
+		return true;
 	return !a->family->is_specified(a);
 }
 #define socket_recvfrom(s,a...) (s)->family->recvfrom((s), a)
@@ -180,6 +189,7 @@ INLINE int is_addr_unspecified(const sockaddr_t *a) {
 #define socket_error(s) (s)->family->error((s))
 #define socket_timestamping(s) (s)->family->timestamping((s))
 #define socket_pktinfo(s) (s)->family->pktinfo((s))
+#define socket_getsockname(s) (s)->family->getsockname((s))
 INLINE ssize_t socket_sendiov(socket_t *s, const struct iovec *v, unsigned int len, const endpoint_t *dst,
 		const sockaddr_t *src)
 {
@@ -213,19 +223,23 @@ struct timeval32 {
 	int32_t tv_usec;
 };
 
+#ifndef SO_TIMESTAMP_OLD
+#define SO_TIMESTAMP_OLD SO_TIMESTAMP // pointless, compiler should remove it
+#endif
+
 #define socket_recvfrom_parse_cmsg(tv, to, parse_to, msgh, firsthdr, nexthdr) do { \
 	if ((*tv) || (*to)) { \
 		struct cmsghdr *cm; \
 		for (cm = firsthdr; cm; cm = nexthdr) { \
 			if ((*tv) && cm->cmsg_level == SOL_SOCKET) { \
 				if (cm->cmsg_type == SO_TIMESTAMP) { \
-					*(*tv) = *((struct timeval *) CMSG_DATA(cm)); \
+					*(*tv) = timeval_us(*((struct timeval *) CMSG_DATA(cm))); \
 					(*tv) = NULL; \
 				} \
 				else if (cm->cmsg_type == SO_TIMESTAMP_OLD) { \
 					struct timeval32 *tc = (struct timeval32 *) CMSG_DATA(cm); \
 					struct timeval tvv = { .tv_sec = tc->tv_sec, .tv_usec = tc->tv_usec }; \
-					*(*tv) = tvv; \
+					*(*tv) = timeval_us(tvv); \
 					(*tv) = NULL; \
 				} \
 			} \
@@ -248,23 +262,18 @@ struct timeval32 {
 } while (0)
 
 
-/* XXX obsolete these? */
-INLINE void usertimeout(int fd, unsigned int val) {
+INLINE void usertimeout(socket_t *s, unsigned int val) {
 	// coverity[check_return : FALSE]
-	setsockopt(fd, IPPROTO_TCP, TCP_USER_TIMEOUT, &val, sizeof(val));
+	setsockopt(s->fd, IPPROTO_TCP, TCP_USER_TIMEOUT, &val, sizeof(val));
 }
 INLINE void nonblock(int fd) {
 	// coverity[check_return : FALSE]
 	fcntl(fd, F_SETFL, O_NONBLOCK);
 }
-INLINE void reuseaddr(int fd) {
-	int one = 1;
+INLINE void socket_rcvtimeout(socket_t *s, int64_t us) {
+	struct timeval tv = timeval_from_us(us);
 	// coverity[check_return : FALSE]
-	setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &one, sizeof(one));
-}
-INLINE void ipv6only(int fd, int yn) {
-	// coverity[check_return : FALSE]
-	setsockopt(fd, IPPROTO_IPV6, IPV6_V6ONLY, &yn, sizeof(yn));
+	setsockopt(s->fd, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv));
 }
 INLINE int socket_cpu_affinity(socket_t *s, int cpu) {
 #ifndef SO_INCOMING_CPU
@@ -279,25 +288,25 @@ INLINE int socket_cpu_affinity(socket_t *s, int cpu) {
 
 void socket_init(void);
 
-int open_socket(socket_t *r, int type, unsigned int port, const sockaddr_t *);
-int open_v46_socket(socket_t *r, int type);
-int connect_socket(socket_t *r, int type, const endpoint_t *ep);
+bool open_socket(socket_t *r, int type, unsigned int port, const sockaddr_t *);
+bool open_v46_socket(socket_t *r, int type);
+bool connect_socket(socket_t *r, int type, const endpoint_t *ep);
 int connect_socket_nb(socket_t *r, int type, const endpoint_t *ep); // 1 == in progress
 int connect_socket_retry(socket_t *r); // retries connect() while in progress
-int close_socket(socket_t *r);
-int reset_socket(socket_t *r);
+bool close_socket(socket_t *r);
+bool reset_socket(socket_t *r);
 void move_socket(socket_t *dst, socket_t *src);
 void dummy_socket(socket_t *r, const sockaddr_t *);
 
 sockfamily_t *get_socket_family_rfc(const str *s);
 sockfamily_t *__get_socket_family_enum(enum socket_families);
-int sockaddr_parse_any(sockaddr_t *dst, const char *src);
-int sockaddr_parse_any_str(sockaddr_t *dst, const str *src);
-int sockaddr_parse_str(sockaddr_t *dst, sockfamily_t *fam, const str *src);
-int endpoint_parse_any(endpoint_t *, const char *); // address (ip) optional
-int sockaddr_getaddrinfo_alt(sockaddr_t *a, sockaddr_t *a2, const char *s);
-int endpoint_parse_any_getaddrinfo_alt(endpoint_t *d, endpoint_t *d2, const char *s); // address (ip or hostname) optional
-INLINE int endpoint_parse_any_getaddrinfo(endpoint_t *d, const char *s);
+bool sockaddr_parse_any(sockaddr_t *dst, const char *src);
+bool sockaddr_parse_any_str(sockaddr_t *dst, const str *src);
+bool sockaddr_parse_str(sockaddr_t *dst, sockfamily_t *fam, const str *src);
+bool endpoint_parse_any(endpoint_t *, const char *); // address (ip) optional
+bool sockaddr_getaddrinfo_alt(sockaddr_t *a, sockaddr_t *a2, const char *s);
+bool endpoint_parse_any_getaddrinfo_alt(endpoint_t *d, endpoint_t *d2, const char *s); // address (ip or hostname) optional
+INLINE bool endpoint_parse_any_getaddrinfo(endpoint_t *d, const char *s);
 void endpoint_parse_sockaddr_storage(endpoint_t *, struct sockaddr_storage *);
 void kernel2endpoint(endpoint_t *ep, const struct re_address *ra);
 
@@ -314,52 +323,42 @@ INLINE sockfamily_t *get_socket_family_enum(enum socket_families i) {
 		return NULL;
 	return __get_socket_family_enum(i);
 }
-INLINE int endpoint_parse_port_any(endpoint_t *e, const char *p, unsigned int port) {
+INLINE bool endpoint_parse_port_any(endpoint_t *e, const char *p, unsigned int port) {
 	if (port > 0xffff)
-		return -1;
+		return false;
 	e->port = port;
 	return sockaddr_parse_any(&e->address, p);
 }
-// address (ip) required
-INLINE int endpoint_parse_any_full(endpoint_t *d, const char *s) {
-	int ret;
-	ret = endpoint_parse_any(d, s);
-	if (ret)
-		return ret;
-	if (is_addr_unspecified(&d->address))
-		return -1;
-	return 0;
-}
 // address (ip or hostname) required
-INLINE int endpoint_parse_any_getaddrinfo_full(endpoint_t *d, const char *s) {
-	int ret;
+INLINE bool endpoint_parse_any_getaddrinfo_full(endpoint_t *d, const char *s) {
+	bool ret;
 	ret = endpoint_parse_any_getaddrinfo(d, s);
-	if (ret)
+	if (!ret)
 		return ret;
 	if (is_addr_unspecified(&d->address))
-		return -1;
-	return 0;
+		return false;
+	return true;
 }
-INLINE int sockaddr_getaddrinfo(sockaddr_t *a, const char *s) {
+INLINE bool sockaddr_getaddrinfo(sockaddr_t *a, const char *s) {
 	return sockaddr_getaddrinfo_alt(a, NULL, s);
 }
-INLINE int endpoint_parse_any_getaddrinfo(endpoint_t *d, const char *s) {
+INLINE bool endpoint_parse_any_getaddrinfo(endpoint_t *d, const char *s) {
 	return endpoint_parse_any_getaddrinfo_alt(d, NULL, s);
 }
-INLINE int ipv46_any_convert(endpoint_t *ep) {
+INLINE bool ipv46_any_convert(endpoint_t *ep) {
 	if (ep->address.family->af != AF_INET)
-		return 0;
+		return false;
 	if (!is_addr_unspecified(&ep->address))
-		return 0;
+		return false;
 	ep->address.family = __get_socket_family_enum(SF_IP6);
 	ZERO(ep->address.ipv6);
-	return 1;
+	return true;
 }
 // needs a writeable str
-INLINE int endpoint_parse_any_str(endpoint_t *d, str *s) {
+INLINE bool endpoint_parse_any_str(endpoint_t *d, str *s) {
 	char tmp = s->s[s->len];
 	s->s[s->len] = '\0';
-	int ret = endpoint_parse_any(d, s->s);
+	bool ret = endpoint_parse_any(d, s->s);
 	s->s[s->len] = tmp;
 	return ret;
 }
