@@ -40,19 +40,15 @@ gboolean str_case_equal(const str *a, const str *b) {
 	return str_casecmp_str((str *) a, (str *) b) == 0;
 }
 
-str *__str_sprintf(const char *fmt, ...) {
-	str *ret;
+str str_sprintf(const char *fmt, ...) {
+	str ret;
 	va_list ap;
 	va_start(ap, fmt);
-	ret = __str_vsprintf(fmt, ap);
+	ret = str_vsprintf(fmt, ap);
 	va_end(ap);
 	return ret;
 }
 
-void str_slice_free(void *p) {
-	g_slice_free1(sizeof(str), p);
-}
-
 
 /**
  * Generates a random hexadecimal string representing n random bytes.
@@ -69,7 +65,7 @@ char *rand_hex_str(char *rand_str, int num_bytes) {
 
 
 static const char *hex_chars = "0123456789abcdef";
-size_t str_uri_encode_len(char *out, const char *in, size_t len) {
+str str_uri_encode_len(char *out, const char *in, size_t len) {
 	const char *end = in + len;
 	char *ori_out = out;
 
@@ -86,7 +82,7 @@ size_t str_uri_encode_len(char *out, const char *in, size_t len) {
 	}
 
 	*out = 0;
-	return out - ori_out;
+	return STR_LEN(ori_out, out - ori_out);
 }
 
 str *str_uri_decode_len(const char *in, size_t in_len) {
