@@ -6,7 +6,6 @@
 #include "socket.h"
 #include "poller.h"
 #include "bufferpool.h"
-#include "call.h"
 
 
 #define BUFFER_SIZE RTP_BUFFER_SIZE	// size of individual buffer
@@ -54,9 +53,22 @@ static ssize_t __socket_sendmsg(socket_t *s, struct msghdr *m, const endpoint_t
 	r->handler(r, 0, 0);
 	return ret;
 }
+static unsigned int __dummy_thread_loop(void) {
+	return 0;
+}
+static void *__dummy_alloc(void *stack_storage, size_t len) {
+	return stack_storage;
+}
+static void __dummy_free(struct uring_req *dummy) {
+}
 
 
-__thread __typeof(__socket_sendmsg) (*uring_sendmsg) = __socket_sendmsg;
+__thread struct uring_methods uring_methods = {
+	.sendmsg = __socket_sendmsg,
+	.thread_loop = __dummy_thread_loop,
+	.free = __dummy_free,
+	.__alloc_req = __dummy_alloc,
+};
 
 
 #ifdef HAVE_LIBURING
@@ -64,6 +76,17 @@ __thread __typeof(__socket_sendmsg) (*uring_sendmsg) = __socket_sendmsg;
 #include <liburing.h>
 
 
+#define uring_alloc_req(T, fn) ({ \
+			T *__ret = g_new0(T, 1); \
+			__ret->req.handler = (fn); \
+			__ret; \
+		})
+
+
+struct uring_buffer_req {
+	struct uring_req req;
+};
+
 static __thread struct io_uring rtpe_uring;
 
 
@@ -97,14 +120,24 @@ static unsigned int __uring_thread_loop(void) {
 	return num;
 }
 
+static void *__uring_alloc(void *dummy, size_t len) {
+	return g_malloc(len);
+}
+
+static void __uring_free(struct uring_req *r) {
+	g_free(r);
+}
+
 void uring_thread_init(void) {
 	struct io_uring_params params = {0};
 	int ret = io_uring_queue_init_params(rtpe_common_config_ptr->io_uring_buffers, &rtpe_uring, &params);
 	if (ret)
 		die("io_uring init failed (%s)", strerror(-ret));
 
-	uring_sendmsg = __uring_sendmsg;
-	uring_thread_loop = __uring_thread_loop;
+	uring_methods.sendmsg = __uring_sendmsg;
+	uring_methods.thread_loop = __uring_thread_loop;
+	uring_methods.__alloc_req = __uring_alloc;
+	uring_methods.free = __uring_free;
 }
 
 void uring_thread_cleanup(void) {
@@ -143,7 +176,9 @@ struct poller *uring_poller_new(void) {
 	ret->evs = g_ptr_array_new();
 	ret->blocked = g_array_new(false, true, sizeof(char));
 
-	ret->bufferpool = bufferpool_new(g_malloc, g_free, BUFFER_SIZE * BUFFERS_COUNT);
+	static_assert(BUFFERPOOL_SHARD_SIZE - BUFFERPOOL_OVERHEAD >= BUFFER_SIZE * BUFFERS_COUNT,
+			"BUFFERPOOL_SHARD_SIZE too small");
+	ret->bufferpool = bufferpool_new(bufferpool_aligned_alloc, bufferpool_aligned_free);
 	for (int i = 0; i < BUFFER_POOLS; i++) {
 		ret->buffers[i] = g_new0(__typeof(*ret->buffers[i]), 1);
 		ret->buffers[i]->buf = bufferpool_reserve(ret->bufferpool, BUFFERS_COUNT,
@@ -307,7 +342,7 @@ static void uring_poll_event(struct uring_req *req, int32_t res, uint32_t flags)
 			if (p->evs->len > ereq->it.fd && p->evs->pdata[ereq->it.fd] == ereq)
 				p->evs->pdata[ereq->it.fd] = NULL;
 		}
-		uring_req_free(&ereq->req);
+		g_free(&ereq->req);
 	}
 }
 
@@ -325,7 +360,7 @@ static void uring_poll_removed(struct uring_req *req, int32_t res, uint32_t flag
 		rreq->callback(rreq->arg);
 	else
 		close(rreq->fd);
-	uring_req_free(req);
+	g_free(req);
 }
 
 struct uring_poll_unblocked {
@@ -360,7 +395,7 @@ static void uring_poll_unblocked(struct uring_req *req, int32_t res, uint32_t fl
 
 	if (ureq->it.obj)
 		obj_put_o(ureq->it.obj);
-	uring_req_free(req);
+	g_free(req);
 }
 
 struct uring_poll_recv {
@@ -371,7 +406,7 @@ struct uring_poll_recv {
 	struct poller *poller;
 	bool closed:1;
 };
-INLINE void uring_recvmsg_parse_cmsg(struct timeval *tv,
+INLINE void uring_recvmsg_parse_cmsg(int64_t *tv,
 		sockaddr_t *to, bool (*parse)(struct cmsghdr *, sockaddr_t *),
 		struct io_uring_recvmsg_out *out, struct msghdr *mh)
 {
@@ -407,10 +442,10 @@ static void uring_poll_recv(struct uring_req *req, int32_t res, uint32_t flags)
 		void *payload = io_uring_recvmsg_payload(out, &rreq->msg);
 		struct sockaddr *sa = io_uring_recvmsg_name(out);
 
-		struct timeval tv = {0};
+		int64_t tv = 0;
 		uring_recvmsg_parse_cmsg(&tv, NULL, NULL, out, &rreq->msg);
 
-		rreq->it.recv(rreq->it.obj, payload, out->payloadlen, sa, &tv);
+		rreq->it.recv(rreq->it.obj, payload, out->payloadlen, sa, tv);
 	}
 
 	if (!(flags & IORING_CQE_F_MORE))
@@ -426,7 +461,7 @@ static void uring_poll_recv(struct uring_req *req, int32_t res, uint32_t flags)
 		//ilog(LOG_INFO, "last uring recv event for fd %i for %p (%i)", rreq->it.fd, rreq->it.obj, rreq->it.obj->ref);
 		if (rreq->it.obj)
 			obj_put_o(rreq->it.obj);
-		uring_req_free(&rreq->req);
+		g_free(&rreq->req);
 	}
 }
 
@@ -435,7 +470,7 @@ static void uring_poller_do_add(struct poller *p, struct poller_req *preq) {
 	if (p->evs->len > preq->it.fd && p->evs->pdata[preq->it.fd])
 		abort(); // XXX handle gracefully?
 	struct uring_poll_event *ereq
-		= uring_alloc_req(sizeof(*ereq), uring_poll_event);
+		= uring_alloc_req(struct uring_poll_event, uring_poll_event);
 	ereq->it = preq->it;
 	ereq->poller = p;
 	struct io_uring_sqe *sqe = io_uring_get_sqe(&rtpe_uring);
@@ -452,7 +487,7 @@ static void uring_poller_do_blocked(struct poller *p, struct poller_req *preq) {
 		abort(); // XXX handle gracefully?
 	struct uring_poll_event *ereq = p->evs->pdata[preq->fd];
 	struct uring_poll_unblocked *ureq
-		= uring_alloc_req(sizeof(*ureq), uring_poll_unblocked);
+		= uring_alloc_req(struct uring_poll_unblocked, uring_poll_unblocked);
 	ureq->it = ereq->it;
 	ureq->poller = p;
 	if (ureq->it.obj)
@@ -467,7 +502,7 @@ static void uring_poller_do_error(struct poller *p, struct poller_req *preq) {
 static void uring_poller_do_del(struct poller *p, struct poller_req *preq) {
 	//ilog(LOG_INFO, "del fd %i on %p", preq->fd, p);
 	struct uring_poll_removed *rreq
-		= uring_alloc_req(sizeof(*rreq), uring_poll_removed);
+		= uring_alloc_req(struct uring_poll_removed, uring_poll_removed);
 	rreq->fd = preq->fd;
 	rreq->poller = p;
 	rreq->callback = preq->callback;
@@ -481,13 +516,13 @@ static void uring_poller_do_buffers(struct poller *p, struct poller_req *preq) {
 	struct io_uring_sqe *sqe = io_uring_get_sqe(&rtpe_uring);
 	io_uring_prep_provide_buffers(sqe, preq->buf, BUFFER_SIZE, BUFFERS_COUNT, 0,
 			preq->num * BUFFERS_COUNT);
-	struct uring_req *breq = uring_alloc_buffer_req(sizeof(*breq));
+	struct uring_buffer_req *breq = uring_alloc_req(struct uring_buffer_req, uring_req_free);
 	io_uring_sqe_set_data(sqe, breq); // XXX no content? not needed?
 }
 static void uring_poller_do_recv(struct poller *p, struct poller_req *preq) {
 	//ilog(LOG_INFO, "adding recv fd %i on %p for %p", preq->it.fd, p, preq->it.obj);
 	struct uring_poll_recv *rreq
-		= uring_alloc_req(sizeof(*rreq), uring_poll_recv);
+		= uring_alloc_req(struct uring_poll_recv, uring_poll_recv);
 	rreq->it = preq->it;
 	rreq->poller = p;
 	struct io_uring_sqe *sqe = io_uring_get_sqe(&rtpe_uring);
@@ -572,7 +607,7 @@ void uring_poller_poll(struct poller *p) {
 }
 
 void uring_poller_clear(struct poller *p) {
-	struct uring_req *req = uring_alloc_buffer_req(sizeof(*req));
+	struct uring_buffer_req *req = uring_alloc_req(struct uring_buffer_req, uring_req_free);
 	struct io_uring_sqe *sqe = io_uring_get_sqe(&rtpe_uring);
 	io_uring_prep_cancel(sqe, 0, IORING_ASYNC_CANCEL_ANY);
 	io_uring_sqe_set_data(sqe, req);
