@@ -17,6 +17,7 @@
 #include "db.h"
 #include "forward.h"
 #include "tag.h"
+#include "tls_send.h"
 
 static pthread_mutex_t metafiles_lock = PTHREAD_MUTEX_INITIALIZER;
 static GHashTable *metafiles;
@@ -26,8 +27,9 @@ static void meta_free(void *ptr) {
 	metafile_t *mf = ptr;
 
 	dbg("freeing metafile info for %s%s%s", FMT_M(mf->name));
-	output_close(mf, mf->mix_out, NULL, mf->discard);
 	mix_destroy(mf->mix);
+	mix_destroy(mf->tls_mix);
+	tls_fwd_free(&mf->mix_tls_fwd);
 	db_close_call(mf);
 	g_string_chunk_free(mf->gsc);
 	// SSRCs first as they have linked outputs which need to be closed first
@@ -47,7 +49,7 @@ static void meta_free(void *ptr) {
 	t_hash_table_destroy(mf->metadata_parsed);
 	g_ptr_array_free(mf->tags, TRUE);
 	g_ptr_array_free(mf->streams, TRUE);
-	g_slice_free1(sizeof(*mf), mf);
+	g_free(mf);
 }
 
 
@@ -72,8 +74,8 @@ static void meta_destroy(metafile_t *mf) {
 	//close forward socket
 	if (mf->forward_fd >= 0) {
 		dbg("call [%s%s%s] forwarded %d packets. %d failed sends.", FMT_M(mf->call_id),
-				(int )g_atomic_int_get(&mf->forward_count),
-				(int )g_atomic_int_get(&mf->forward_failed));
+				__atomic_load_n(&mf->forward_count, __ATOMIC_RELAXED),
+				__atomic_load_n(&mf->forward_failed, __ATOMIC_RELAXED));
 		close(mf->forward_fd);
 		mf->forward_fd = -1;
 	}
@@ -84,23 +86,77 @@ static void meta_destroy(metafile_t *mf) {
 		mf->ssrc_hash = NULL;
 	}
 	db_close_call(mf);
+	output_close(mf, mf->mix_out, NULL, mf->discard);
+	mf->mix_out = NULL;
+	mix_close(mf->mix);
+}
+
+
+// mf is locked
+// mix is locked
+static void meta_mix_file_output(metafile_t *mf) {
+	if ((output_storage & OUTPUT_STORAGE_MASK) == 0 || !output_mixed || !mf->recording_on
+			|| !mf->random_tag)
+	{
+		mix_destroy(mf->mix);
+		mf->mix = NULL;
+		return;
+	}
+
+	if (!mf->started)
+		return;
+
+	if (!mf->mix) {
+		mf->mix_out = output_new_ext(mf, "mix", "mixed", "mix");
+		mf->mix = mix_new(&mf->mix_lock, &mf->mix_out->sink, mf->media_rec_slots);
+	}
+
+	db_do_stream(mf, mf->mix_out, NULL, 0);
+}
+
+
+static void meta_mix_tls_output(metafile_t *mf) {
+	if (!tls_mixed) {
+		mix_destroy(mf->tls_mix);
+		mf->tls_mix = NULL;
+		tls_fwd_free(&mf->mix_tls_fwd);
+		return;
+	}
+
+	if (!mf->mix_tls_fwd) {
+		mix_destroy(mf->tls_mix);
+		mf->tls_mix = NULL;
+	}
+
+	if (!mf->started)
+		return;
+	if (!mf->forwarding_on)
+		return;
+
+	if (!tls_fwd_new(&mf->mix_tls_fwd))
+		return;
+	if (mix_method == MM_CHANNELS)
+		mf->mix_tls_fwd->sink.format.channels = mix_num_inputs;
+	if (!mf->tls_mix)
+		mf->tls_mix = mix_new(&mf->mix_lock, &mf->mix_tls_fwd->sink, mf->media_rec_slots);
+
+	mf->mix_tls_fwd->metafile = mf;
+}
+
+
+// mf is locked
+static void meta_mix_output(metafile_t *mf) {
+	LOCK(&mf->mix_lock);
+
+	meta_mix_file_output(mf);
+	meta_mix_tls_output(mf);
 }
 
 
 // mf is locked
 static void meta_stream_interface(metafile_t *mf, unsigned long snum, char *content) {
 	db_do_call(mf);
-	if (output_enabled && output_mixed && mf->recording_on) {
-		pthread_mutex_lock(&mf->mix_lock);
-		if (!mf->mix) {
-			mf->mix_out = output_new_ext(mf, "mix", "mixed", "mix");
-			if (mix_method == MM_CHANNELS)
-				mf->mix_out->channel_mult = mix_num_inputs;
-			mf->mix = mix_new();
-			db_do_stream(mf, mf->mix_out, NULL, 0);
-		}
-		pthread_mutex_unlock(&mf->mix_lock);
-	}
+	meta_mix_output(mf);
 	dbg("stream %lu interface %s%s%s", snum, FMT_M(content));
 	stream_open(mf, snum, content);
 }
@@ -109,12 +165,16 @@ static void meta_stream_interface(metafile_t *mf, unsigned long snum, char *cont
 // mf is locked
 static void meta_stream_details(metafile_t *mf, unsigned long snum, char *content) {
 	dbg("stream %lu details %s", snum, content);
-	unsigned int tag, media, tm, cmp, media_sdp_id;
+	unsigned int tag, media, tm, cmp, media_sdp_id, media_rec_slot, media_rec_slots;
 	uint64_t flags;
-	if (sscanf_match(content, "TAG %u MEDIA %u TAG-MEDIA %u COMPONENT %u FLAGS %" PRIu64 " MEDIA-SDP-ID %i",
-				&tag, &media, &tm, &cmp, &flags, &media_sdp_id) != 6)
+	if (sscanf_match(content, "TAG %u MEDIA %u TAG-MEDIA %u COMPONENT %u FLAGS %" PRIu64 " MEDIA-SDP-ID %i MEDIA-REC-SLOT %i MEDIA-REC-SLOTS %i",
+				&tag, &media, &tm, &cmp, &flags, &media_sdp_id, &media_rec_slot, &media_rec_slots) != 8)
 		return;
-	stream_details(mf, snum, tag, media_sdp_id);
+
+	mf->media_rec_slots = media_rec_slots;
+	mix_set_channel_slots(mf->mix, media_rec_slots);
+	mix_set_channel_slots(mf->tls_mix, media_rec_slots);
+	stream_details(mf, snum, tag, media_sdp_id, media_rec_slot-1);
 }
 
 
@@ -175,7 +235,7 @@ static void meta_ptime(metafile_t *mf, unsigned long mnum, int ptime)
 static void meta_metadata_parse(metafile_t *mf) {
 	// XXX offload this parsing to proxy module -> bencode list/dictionary
 	t_hash_table_remove_all(mf->metadata_parsed);
-	str all_meta = STR_INIT(mf->metadata);
+	str all_meta = STR(mf->metadata);
 	while (all_meta.len > 1) {
 		str token;
 		if (!str_token_sep(&token, &all_meta, '|'))
@@ -257,6 +317,11 @@ static void meta_section(metafile_t *mf, char *section, char *content, unsigned
 		mf->output_pattern = g_string_chunk_insert(mf->gsc, content);
 	else if (!strcmp(section, "SKIP_DATABASE"))
 		mf->skip_db = 1;
+	else if (!strcmp(section, "STARTED"))
+		mf->started = 1;
+
+	db_do_call(mf);
+	meta_mix_output(mf);
 }
 
 
@@ -270,7 +335,7 @@ static metafile_t *metafile_get(char *name) {
 
 	ilog(LOG_INFO, "New call for recording: '%s%s%s'", FMT_M(name));
 
-	mf = g_slice_alloc0(sizeof(*mf));
+	mf = g_new0(__typeof(*mf), 1);
 	mf->gsc = g_string_chunk_new(0);
 	mf->name = g_string_chunk_insert(mf->gsc, name);
 	pthread_mutex_init(&mf->lock, NULL);
@@ -280,7 +345,7 @@ static metafile_t *metafile_get(char *name) {
 	mf->forward_count = 0;
 	mf->forward_failed = 0;
 	mf->recording_on = 1;
-	mf->start_time = now_double();
+	mf->start_time_us = now_us();
 	mf->metadata_parsed = metadata_ht_new();
 
 	if (decoding_enabled) {
