@@ -12,9 +12,14 @@ void output_init(const char *format);
 
 output_t *output_new_ext(metafile_t *, const char *type, const char *kind, const char *label);
 void output_close(metafile_t *, output_t *, tag_t *, bool discard);
+content_t *output_get_content(output_t *);
+void output_content_failure(content_t *);
 
-int output_config(output_t *output, const format_t *requested_format, format_t *actual_format);
-int output_add(output_t *output, AVFrame *frame);
+
+void sink_init(sink_t *);
+void sink_close(sink_t *sink);
+
+bool sink_add(sink_t *, AVFrame *frame);
 
 
 #endif
