@@ -8,7 +8,7 @@ use NGCP::Rtpengine::AutoTest;
 use Test::More;
 use Test2::Tools::Compare qw();
 use Socket qw(AF_INET SOCK_STREAM sockaddr_in pack_sockaddr_in inet_aton);
-use JSON;
+use Bencode;
 use Data::Dumper;
 
 $Data::Dumper::Sortkeys = 1;
@@ -63,8 +63,8 @@ autotest_start(qw(--config-file=none -t -1 -i foo/203.0.113.1 -i foo/2001:db8:43
 my $json_exp;
 $NGCP::Rtpengine::req_cb = sub {
 	redis_io("*1\r\n\$4\r\nPING\r\n", "+PONG\r\n", "req PING");
-	redis_i("*3\r\n\$3\r\nSET\r\n\$" . length(cid()) . "\r\n" . cid() . "\r\n\$", "req intro");
-	# dumbly expect 4-digit number
+	redis_i("*5\r\n\$3\r\nSET\r\n\$" . length(cid()) . "\r\n" . cid() . "\r\n\$", "req intro");
+	# dumbly expect 4-digit number as length
 	my $buf;
 	alarm(1);
 	recv($redis_fd, $buf, 6, 0) or die;
@@ -74,12 +74,12 @@ $NGCP::Rtpengine::req_cb = sub {
 	alarm(1);
 	recv($redis_fd, $buf, $len, 0) or die;
 	alarm(0);
-	my $json = decode_json($buf);
+	my $json = Bencode::bdecode($buf, 1);
 	#print Dumper($json);
 	Test2::Tools::Compare::like($json, $json_exp, "JSON");
-	redis_io("\r\n*3\r\n\$6\r\nEXPIRE\r\n\$" . length(cid()) . "\r\n" . cid() . "\r\n\$5\r\n86400\r\n",
-		"+OK\r\n+OK\r\n",
-		"req EXPIRE");
+	redis_io("\r\n\$2\r\nEX\r\n\$5\r\n86400\r\n",
+		"+OK\r\n",
+		"req outro");
 };
 
 
@@ -98,7 +98,6 @@ $json_exp = {
 	      'call_flags' => 65536,
 	      'created' => qr/^\d+$/,
 	      'created_from' => qr//,
-	      'created_from_addr' => qr//,
 	      'deleted' => '0',
 	      'destroyed' => '0',
 	      'last_signal' => qr/^\d+$/,
@@ -136,24 +135,24 @@ $json_exp = {
 		    '2',
 		    '3'
 		  ],
-  'maps-0' => [
+  'maps-1' => [
 		'0'
 	      ],
-  'maps-1' => [
+  'maps-0' => [
 		'1'
 	      ],
-  'media-0' => {
+  'media-1' => {
 		 'desired_family' => 'IP4',
 		 'format_str' => '0 8',
 		 'index' => '1',
 		 'logical_intf' => 'foo',
-		 'media_flags' => '2228236',
+		 'media_flags' => '274880135180',
 		 'protocol' => 'RTP/AVP',
 		 'ptime' => '0',
 		 'tag' => '1',
 		 'type' => 'audio'
 	       },
-  'media-1' => {
+  'media-0' => {
 		 'desired_family' => 'IP4',
 		 'format_str' => '0 8',
 		 'index' => '1',
@@ -164,10 +163,10 @@ $json_exp = {
 		 'tag' => '0',
 		 'type' => 'audio'
 	       },
-  'medias-0' => [
+  'medias-1' => [
 		  '1'
 		],
-  'medias-1' => [
+  'medias-0' => [
 		  '0'
 		],
   'payload_types-0' => [
@@ -232,8 +231,7 @@ $json_exp = {
 		  'advertised_endpoint' => '',
 		  'component' => '1',
 		  'endpoint' => '',
-		  'last_packet' => qr/^\d+$/,
-		  'media' => '0',
+		  'media' => '1',
 		  'ps_flags' => '65536',
 		  'rtcp_sibling' => '1',
 		  'sfd' => '0',
@@ -245,8 +243,7 @@ $json_exp = {
 		  'advertised_endpoint' => '',
 		  'component' => '2',
 		  'endpoint' => '',
-		  'last_packet' => qr/^\d+$/,
-		  'media' => '0',
+		  'media' => '1',
 		  'ps_flags' => '131072',
 		  'rtcp_sibling' => '4294967295',
 		  'sfd' => '1',
@@ -258,8 +255,7 @@ $json_exp = {
 		  'advertised_endpoint' => '198.51.100.1:3000',
 		  'component' => '1',
 		  'endpoint' => '198.51.100.1:3000',
-		  'last_packet' => qr/^\d+$/,
-		  'media' => '1',
+		  'media' => '0',
 		  'ps_flags' => '68222976',
 		  'rtcp_sibling' => '3',
 		  'sfd' => '2',
@@ -271,9 +267,8 @@ $json_exp = {
 		  'advertised_endpoint' => '198.51.100.1:3001',
 		  'component' => '2',
 		  'endpoint' => '198.51.100.1:3001',
-		  'last_packet' => qr/^\d+$/,
-		  'media' => '1',
-		  'ps_flags' => '68288513',
+		  'media' => '0',
+		  'ps_flags' => '1179649',
 		  'rtcp_sibling' => '4294967295',
 		  'sfd' => '3',
 		  'stats-bytes' => '0',
@@ -292,11 +287,11 @@ $json_exp = {
   'stream_sfds-3' => [
 		       '3'
 		     ],
-  'streams-0' => [
+  'streams-1' => [
 		   '0',
 		   '1'
 		 ],
-  'streams-1' => [
+  'streams-0' => [
 		   '2',
 		   '3'
 		 ],
@@ -347,6 +342,8 @@ a=rtcp:PORT
 SDP
 
 $json_exp = {
+          'tag_aliases-0' => [],
+          'tag_aliases-1' => [],
   'associated_tags-0' => [
 			   '1'
 			 ],
@@ -358,7 +355,6 @@ $json_exp = {
 	      'call_flags' => 1376256,
 	      'created' => qr/^\d+$/,
 	      'created_from' => qr//,
-	      'created_from_addr' => qr//,
 	      'deleted' => '0',
 	      'destroyed' => '0',
 	      'last_signal' => qr/^\d+$/,
@@ -396,38 +392,38 @@ $json_exp = {
 		    '2',
 		    '3'
 		  ],
-  'maps-0' => [
+  'maps-1' => [
 		'0'
 	      ],
-  'maps-1' => [
+  'maps-0' => [
 		'1'
 	      ],
-  'media-0' => {
-		 'desired_family' => 'IP4',
-		 'format_str' => '8',
-		 'index' => '1',
-		 'logical_intf' => 'foo',
-		 'media_flags' => '2293772',
-		 'protocol' => 'RTP/AVP',
-		 'ptime' => '0',
-		 'tag' => '1',
-		 'type' => 'audio'
-	       },
   'media-1' => {
 		 'desired_family' => 'IP4',
 		 'format_str' => '8',
 		 'index' => '1',
 		 'logical_intf' => 'foo',
-		 'media_flags' => '65548',
+		 'media_flags' => '274880200716',
+		 'protocol' => 'RTP/AVP',
+		 'ptime' => '0',
+		 'tag' => '1',
+		 'type' => 'audio'
+	       },
+  'media-0' => {
+		 'desired_family' => 'IP4',
+		 'format_str' => '8',
+		 'index' => '1',
+		 'logical_intf' => 'foo',
+		 'media_flags' => '274877972492',
 		 'protocol' => 'RTP/AVP',
 		 'ptime' => '0',
 		 'tag' => '0',
 		 'type' => 'audio'
 	       },
-  'medias-0' => [
+  'medias-1' => [
 		  '1'
 		],
-  'medias-1' => [
+  'medias-0' => [
 		  '0'
 		],
   'payload_types-0' => [
@@ -490,8 +486,7 @@ $json_exp = {
 		  'advertised_endpoint' => '198.51.100.4:3000',
 		  'component' => '1',
 		  'endpoint' => '198.51.100.4:3000',
-		  'last_packet' => qr/^\d+$/,
-		  'media' => '0',
+		  'media' => '1',
 		  'ps_flags' => '1114112',
 		  'rtcp_sibling' => '1',
 		  'sfd' => '0',
@@ -503,8 +498,7 @@ $json_exp = {
 		  'advertised_endpoint' => '198.51.100.4:3001',
 		  'component' => '2',
 		  'endpoint' => '198.51.100.4:3001',
-		  'last_packet' => qr/^\d+$/,
-		  'media' => '0',
+		  'media' => '1',
 		  'ps_flags' => '1179649',
 		  'rtcp_sibling' => '4294967295',
 		  'sfd' => '1',
@@ -516,8 +510,7 @@ $json_exp = {
 		  'advertised_endpoint' => '198.51.100.1:3000',
 		  'component' => '1',
 		  'endpoint' => '198.51.100.1:3000',
-		  'last_packet' => qr/^\d+$/,
-		  'media' => '1',
+		  'media' => '0',
 		  'ps_flags' => '1114112',
 		  'rtcp_sibling' => '3',
 		  'sfd' => '2',
@@ -529,8 +522,7 @@ $json_exp = {
 		  'advertised_endpoint' => '198.51.100.1:3001',
 		  'component' => '2',
 		  'endpoint' => '198.51.100.1:3001',
-		  'last_packet' => qr/^\d+$/,
-		  'media' => '1',
+		  'media' => '0',
 		  'ps_flags' => '1179649',
 		  'rtcp_sibling' => '4294967295',
 		  'sfd' => '3',
@@ -550,11 +542,11 @@ $json_exp = {
   'stream_sfds-3' => [
 		       '3'
 		     ],
-  'streams-0' => [
+  'streams-1' => [
 		   '0',
 		   '1'
 		 ],
-  'streams-1' => [
+  'streams-0' => [
 		   '2',
 		   '3'
 		 ],
@@ -604,6 +596,266 @@ a=sendrecv
 a=rtcp:PORT
 SDP
 
+my $old_tt = tt();
+new_tt();
+
+$json_exp = {
+          'tag_aliases-0' => [],
+          'tag_aliases-1' => [
+		            $old_tt
+	                 ],
+  'associated_tags-0' => [
+			   '1'
+			 ],
+  'associated_tags-1' => [
+			   '0'
+			 ],
+  'json' => {
+	      'block_dtmf' => '0',
+	      'call_flags' => 1376256,
+	      'created' => qr/^\d+$/,
+	      'created_from' => qr//,
+	      'deleted' => '0',
+	      'destroyed' => '0',
+	      'last_signal' => qr/^\d+$/,
+	      'ml_deleted' => '0',
+	      'num_maps' => '2',
+	      'num_medias' => '2',
+	      'num_sfds' => '4',
+	      'num_streams' => '4',
+	      'num_tags' => '2',
+	      'recording_metadata' => '',
+	      'redis_hosted_db' => '2',
+	      'tos' => '0'
+	    },
+  'map-0' => {
+	       'endpoint' => '198.51.100.1:3000',
+	       'intf_preferred_family' => 'IP4',
+	       'logical_intf' => 'foo',
+	       'num_ports' => '2',
+	       'wildcard' => '0'
+	     },
+  'map-1' => {
+	       'endpoint' => '198.51.100.4:3000',
+	       'intf_preferred_family' => 'IP4',
+	       'logical_intf' => 'foo',
+	       'num_ports' => '2',
+	       'wildcard' => '0'
+	     },
+  'map_sfds-0' => [
+		    'loc-0',
+		    '0',
+		    '1'
+		  ],
+  'map_sfds-1' => [
+		    'loc-0',
+		    '2',
+		    '3'
+		  ],
+  'maps-1' => [
+		'0'
+	      ],
+  'maps-0' => [
+		'1'
+	      ],
+  'media-1' => {
+		 'desired_family' => 'IP4',
+		 'format_str' => '8',
+		 'index' => '1',
+		 'logical_intf' => 'foo',
+		 'media_flags' => '274880200716',
+		 'protocol' => 'RTP/AVP',
+		 'ptime' => '0',
+		 'tag' => '1',
+		 'type' => 'audio'
+	       },
+  'media-0' => {
+		 'desired_family' => 'IP4',
+		 'format_str' => '8',
+		 'index' => '1',
+		 'logical_intf' => 'foo',
+		 'media_flags' => '274877972492',
+		 'protocol' => 'RTP/AVP',
+		 'ptime' => '0',
+		 'tag' => '0',
+		 'type' => 'audio'
+	       },
+  'medias-1' => [
+		  '1'
+		],
+  'medias-0' => [
+		  '0'
+		],
+  'payload_types-0' => [
+			 '8/PCMA/8000//0/20//'
+		       ],
+  'payload_types-1' => [
+			 '8/PCMA/8000//0/20//'
+		       ],
+  'rtcp_sinks-0' => [],
+  'rtcp_sinks-1' => [
+		      '3'
+		    ],
+  'rtcp_sinks-2' => [],
+  'rtcp_sinks-3' => [
+		      '1'
+		    ],
+  'rtp_sinks-0' => [
+		     '2'
+		   ],
+  'rtp_sinks-1' => [],
+  'rtp_sinks-2' => [
+		     '0'
+		   ],
+  'rtp_sinks-3' => [],
+  'sfd-0' => {
+	       'fd' => qr/^\d+$/,
+	       'local_intf_uid' => '0',
+	       'localport' => qr/^\d+$/,
+	       'logical_intf' => 'foo',
+	       'pref_family' => 'IP4',
+	       'stream' => '0'
+	     },
+  'sfd-1' => {
+	       'fd' => qr/^\d+$/,
+	       'local_intf_uid' => '0',
+	       'localport' => qr/^\d+$/,
+	       'logical_intf' => 'foo',
+	       'pref_family' => 'IP4',
+	       'stream' => '1'
+	     },
+  'sfd-2' => {
+	       'fd' => qr/^\d+$/,
+	       'local_intf_uid' => '0',
+	       'localport' => qr/^\d+$/,
+	       'logical_intf' => 'foo',
+	       'pref_family' => 'IP4',
+	       'stream' => '2'
+	     },
+  'sfd-3' => {
+	       'fd' => qr/^\d+$/,
+	       'local_intf_uid' => '0',
+	       'localport' => qr/^\d+$/,
+	       'logical_intf' => 'foo',
+	       'pref_family' => 'IP4',
+	       'stream' => '3'
+	     },
+  'ssrc_table-0' => [],
+  'ssrc_table-1' => [],
+  'stream-0' => {
+		  'advertised_endpoint' => '198.51.100.4:3000',
+		  'component' => '1',
+		  'endpoint' => '198.51.100.4:3000',
+		  'media' => '1',
+		  'ps_flags' => '1114112',
+		  'rtcp_sibling' => '1',
+		  'sfd' => '0',
+		  'stats-bytes' => '0',
+		  'stats-errors' => '0',
+		  'stats-packets' => '0'
+		},
+  'stream-1' => {
+		  'advertised_endpoint' => '198.51.100.4:3001',
+		  'component' => '2',
+		  'endpoint' => '198.51.100.4:3001',
+		  'media' => '1',
+		  'ps_flags' => '1179649',
+		  'rtcp_sibling' => '4294967295',
+		  'sfd' => '1',
+		  'stats-bytes' => '0',
+		  'stats-errors' => '0',
+		  'stats-packets' => '0'
+		},
+  'stream-2' => {
+		  'advertised_endpoint' => '198.51.100.1:3000',
+		  'component' => '1',
+		  'endpoint' => '198.51.100.1:3000',
+		  'media' => '0',
+		  'ps_flags' => '1114112',
+		  'rtcp_sibling' => '3',
+		  'sfd' => '2',
+		  'stats-bytes' => '0',
+		  'stats-errors' => '0',
+		  'stats-packets' => '0'
+		},
+  'stream-3' => {
+		  'advertised_endpoint' => '198.51.100.1:3001',
+		  'component' => '2',
+		  'endpoint' => '198.51.100.1:3001',
+		  'media' => '0',
+		  'ps_flags' => '1179649',
+		  'rtcp_sibling' => '4294967295',
+		  'sfd' => '3',
+		  'stats-bytes' => '0',
+		  'stats-errors' => '0',
+		  'stats-packets' => '0'
+		},
+  'stream_sfds-0' => [
+		       '0'
+		     ],
+  'stream_sfds-1' => [
+		       '1'
+		     ],
+  'stream_sfds-2' => [
+		       '2'
+		     ],
+  'stream_sfds-3' => [
+		       '3'
+		     ],
+  'streams-1' => [
+		   '0',
+		   '1'
+		 ],
+  'streams-0' => [
+		   '2',
+		   '3'
+		 ],
+  'media-subscriptions-0' => [
+			 '1/1/0/0'
+		       ],
+  'media-subscriptions-1' => [
+			 '0/1/0/0'
+		       ],
+  'tag-0' => {
+	       'block_dtmf' => '0',
+	       'created' => qr/^\d+$/,
+	       'deleted' => '0',
+	       'desired_family' => 'IP4',
+	       'logical_intf' => 'foo',
+	       'ml_flags' => 0,
+	       'tag' => ft()
+	     },
+  'tag-1' => {
+	       'block_dtmf' => '0',
+	       'created' => qr/^\d+$/,
+	       'deleted' => '0',
+	       'desired_family' => 'IP4',
+	       'logical_intf' => 'foo',
+	       'ml_flags' => 0,
+	       'tag' => tt()
+	     }
+};
+
+answer('alias to-tag',
+	{ }, <<SDP);
+v=0
+o=- 1545997027 1 IN IP4 198.51.101.40
+s=tester
+t=0 0
+m=audio 3000 RTP/AVP 8
+c=IN IP4 198.51.100.4
+----------------------------------
+v=0
+o=- 1545997027 1 IN IP4 198.51.101.40
+s=tester
+t=0 0
+m=audio PORT RTP/AVP 8
+c=IN IP4 203.0.113.1
+a=rtpmap:8 PCMA/8000
+a=sendrecv
+a=rtcp:PORT
+SDP
+
 
 
 
@@ -623,7 +875,6 @@ $json_exp = {
 	      'call_flags' => 65536,
 	      'created' => qr/^\d+$/,
 	      'created_from' => qr//,
-	      'created_from_addr' => qr//,
 	      'deleted' => '0',
 	      'destroyed' => '0',
 	      'last_signal' => qr/^\d+$/,
@@ -661,24 +912,24 @@ $json_exp = {
 		    '2',
 		    '3'
 		  ],
-  'maps-0' => [
+  'maps-1' => [
 		'0'
 	      ],
-  'maps-1' => [
+  'maps-0' => [
 		'1'
 	      ],
-  'media-0' => {
+  'media-1' => {
 		 'desired_family' => 'IP4',
 		 'format_str' => '0 8',
 		 'index' => '1',
 		 'logical_intf' => 'foo',
-		 'media_flags' => '2228236',
+		 'media_flags' => '274880135180',
 		 'protocol' => 'RTP/AVP',
 		 'ptime' => '0',
 		 'tag' => '1',
 		 'type' => 'audio'
 	       },
-  'media-1' => {
+  'media-0' => {
 		 'desired_family' => 'IP4',
 		 'format_str' => '0 8',
 		 'index' => '1',
@@ -689,10 +940,10 @@ $json_exp = {
 		 'tag' => '0',
 		 'type' => 'audio'
 	       },
-  'medias-0' => [
+  'medias-1' => [
 		  '1'
 		],
-  'medias-1' => [
+  'medias-0' => [
 		  '0'
 		],
   'payload_types-0' => [
@@ -757,8 +1008,7 @@ $json_exp = {
 		  'advertised_endpoint' => '',
 		  'component' => '1',
 		  'endpoint' => '',
-		  'last_packet' => qr/^\d+$/,
-		  'media' => '0',
+		  'media' => '1',
 		  'ps_flags' => '65536',
 		  'rtcp_sibling' => '1',
 		  'sfd' => '0',
@@ -770,8 +1020,7 @@ $json_exp = {
 		  'advertised_endpoint' => '',
 		  'component' => '2',
 		  'endpoint' => '',
-		  'last_packet' => qr/^\d+$/,
-		  'media' => '0',
+		  'media' => '1',
 		  'ps_flags' => '131072',
 		  'rtcp_sibling' => '4294967295',
 		  'sfd' => '1',
@@ -783,8 +1032,7 @@ $json_exp = {
 		  'advertised_endpoint' => '198.51.100.14:6088',
 		  'component' => '1',
 		  'endpoint' => '198.51.100.14:6088',
-		  'last_packet' => qr/^\d+$/,
-		  'media' => '1',
+		  'media' => '0',
 		  'ps_flags' => '68222976',
 		  'rtcp_sibling' => '3',
 		  'sfd' => '2',
@@ -796,9 +1044,8 @@ $json_exp = {
 		  'advertised_endpoint' => '198.51.100.14:6089',
 		  'component' => '2',
 		  'endpoint' => '198.51.100.14:6089',
-		  'last_packet' => qr/^\d+$/,
-		  'media' => '1',
-		  'ps_flags' => '68288513',
+		  'media' => '0',
+		  'ps_flags' => '1179649',
 		  'rtcp_sibling' => '4294967295',
 		  'sfd' => '3',
 		  'stats-bytes' => '0',
@@ -817,11 +1064,11 @@ $json_exp = {
   'stream_sfds-3' => [
 		       '3'
 		     ],
-  'streams-0' => [
+  'streams-1' => [
 		   '0',
 		   '1'
 		 ],
-  'streams-1' => [
+  'streams-0' => [
 		   '2',
 		   '3'
 		 ],
@@ -884,7 +1131,6 @@ $json_exp = {
 	      'call_flags' => 1376256,
 	      'created' => qr/^\d+$/,
 	      'created_from' => qr//,
-	      'created_from_addr' => qr//,
 	      'deleted' => '0',
 	      'destroyed' => '0',
 	      'last_signal' => qr/^\d+$/,
@@ -922,38 +1168,38 @@ $json_exp = {
 		    '2',
 		    '3'
 		  ],
-  'maps-0' => [
+  'maps-1' => [
 		'0'
 	      ],
-  'maps-1' => [
+  'maps-0' => [
 		'1'
 	      ],
-  'media-0' => {
-		 'desired_family' => 'IP4',
-		 'format_str' => '0 8',
-		 'index' => '1',
-		 'logical_intf' => 'foo',
-		 'media_flags' => '2293772',
-		 'protocol' => 'RTP/AVP',
-		 'ptime' => '0',
-		 'tag' => '1',
-		 'type' => 'audio'
-	       },
   'media-1' => {
 		 'desired_family' => 'IP4',
 		 'format_str' => '0 8',
 		 'index' => '1',
 		 'logical_intf' => 'foo',
-		 'media_flags' => '65548',
+		 'media_flags' => '274880200716',
+		 'protocol' => 'RTP/AVP',
+		 'ptime' => '0',
+		 'tag' => '1',
+		 'type' => 'audio'
+	       },
+  'media-0' => {
+		 'desired_family' => 'IP4',
+		 'format_str' => '0 8',
+		 'index' => '1',
+		 'logical_intf' => 'foo',
+		 'media_flags' => '274877972492',
 		 'protocol' => 'RTP/AVP',
 		 'ptime' => '0',
 		 'tag' => '0',
 		 'type' => 'audio'
 	       },
-  'medias-0' => [
+  'medias-1' => [
 		  '1'
 		],
-  'medias-1' => [
+  'medias-0' => [
 		  '0'
 		],
   'payload_types-0' => [
@@ -1018,8 +1264,7 @@ $json_exp = {
 		  'advertised_endpoint' => '198.51.100.14:6090',
 		  'component' => '1',
 		  'endpoint' => '198.51.100.14:6090',
-		  'last_packet' => qr/^\d+$/,
-		  'media' => '0',
+		  'media' => '1',
 		  'ps_flags' => '1114112',
 		  'rtcp_sibling' => '1',
 		  'sfd' => '0',
@@ -1031,8 +1276,7 @@ $json_exp = {
 		  'advertised_endpoint' => '198.51.100.14:6091',
 		  'component' => '2',
 		  'endpoint' => '198.51.100.14:6091',
-		  'last_packet' => qr/^\d+$/,
-		  'media' => '0',
+		  'media' => '1',
 		  'ps_flags' => '1179649',
 		  'rtcp_sibling' => '4294967295',
 		  'sfd' => '1',
@@ -1044,8 +1288,7 @@ $json_exp = {
 		  'advertised_endpoint' => '198.51.100.14:6088',
 		  'component' => '1',
 		  'endpoint' => '198.51.100.14:6088',
-		  'last_packet' => qr/^\d+$/,
-		  'media' => '1',
+		  'media' => '0',
 		  'ps_flags' => '1114112',
 		  'rtcp_sibling' => '3',
 		  'sfd' => '2',
@@ -1057,8 +1300,7 @@ $json_exp = {
 		  'advertised_endpoint' => '198.51.100.14:6089',
 		  'component' => '2',
 		  'endpoint' => '198.51.100.14:6089',
-		  'last_packet' => qr/^\d+$/,
-		  'media' => '1',
+		  'media' => '0',
 		  'ps_flags' => '1179649',
 		  'rtcp_sibling' => '4294967295',
 		  'sfd' => '3',
@@ -1078,11 +1320,11 @@ $json_exp = {
   'stream_sfds-3' => [
 		       '3'
 		     ],
-  'streams-0' => [
+  'streams-1' => [
 		   '0',
 		   '1'
 		 ],
-  'streams-1' => [
+  'streams-0' => [
 		   '2',
 		   '3'
 		 ],
@@ -1147,7 +1389,6 @@ $json_exp = {
 		      'call_flags' => 1376256,
                       'created' => qr/^\d+$/,
                       'created_from' => qr//,
-                      'created_from_addr' => qr//,
                       'deleted' => '0',
                       'destroyed' => '0',
                       'last_signal' => qr/^\d+$/,
@@ -1209,10 +1450,10 @@ $json_exp = {
                             '6',
                             '7'
                           ],
-          'maps-0' => [
+          'maps-1' => [
                         '0'
                       ],
-          'maps-1' => [
+          'maps-0' => [
                         '1'
                       ],
           'maps-2' => [
@@ -1221,23 +1462,23 @@ $json_exp = {
           'maps-3' => [
                         '3'
                       ],
-          'media-0' => {
-                         'desired_family' => 'IP4',
-                         'format_str' => '0 8',
-                         'index' => '1',
-                         'logical_intf' => 'foo',
-                         'media_flags' => '2293772',
-                         'protocol' => 'RTP/AVP',
-                         'ptime' => '0',
-                         'tag' => '1',
-                         'type' => 'audio'
-                       },
           'media-1' => {
                          'desired_family' => 'IP4',
                          'format_str' => '0 8',
                          'index' => '1',
                          'logical_intf' => 'foo',
-                         'media_flags' => '65548',
+                         'media_flags' => '274880200716',
+                         'protocol' => 'RTP/AVP',
+                         'ptime' => '0',
+                         'tag' => '1',
+                         'type' => 'audio'
+                       },
+          'media-0' => {
+                         'desired_family' => 'IP4',
+                         'format_str' => '0 8',
+                         'index' => '1',
+                         'logical_intf' => 'foo',
+                         'media_flags' => '274877972492',
                          'protocol' => 'RTP/AVP',
                          'ptime' => '0',
                          'tag' => '0',
@@ -1248,7 +1489,7 @@ $json_exp = {
                          'format_str' => '0 8',
                          'index' => '1',
                          'logical_intf' => 'foo',
-                         'media_flags' => '2097156',
+                         'media_flags' => '274880004100',
                          'protocol' => 'RTP/AVP',
                          'ptime' => '0',
                          'tag' => '2',
@@ -1259,16 +1500,16 @@ $json_exp = {
                          'format_str' => '0 8',
                          'index' => '2',
                          'logical_intf' => 'foo',
-                         'media_flags' => '2097156',
+                         'media_flags' => '274880004100',
                          'protocol' => 'RTP/AVP',
                          'ptime' => '0',
                          'tag' => '2',
                          'type' => 'audio'
                        },
-          'medias-0' => [
+          'medias-1' => [
                           '1'
                         ],
-          'medias-1' => [
+          'medias-0' => [
                           '0'
                         ],
           'medias-2' => [
@@ -1390,8 +1631,7 @@ $json_exp = {
                           'advertised_endpoint' => '198.51.100.14:6090',
                           'component' => '1',
                           'endpoint' => '198.51.100.14:6090',
-                          'last_packet' => qr/^\d+$/,
-                          'media' => '0',
+                          'media' => '1',
                           'ps_flags' => '1114112',
                           'rtcp_sibling' => '1',
                           'sfd' => '0',
@@ -1403,9 +1643,8 @@ $json_exp = {
                           'advertised_endpoint' => '198.51.100.14:6091',
                           'component' => '2',
                           'endpoint' => '198.51.100.14:6091',
-                          'last_packet' => qr/^\d+$/,
-                          'media' => '0',
-                          'ps_flags' => '68288513',
+                          'media' => '1',
+                          'ps_flags' => '1179649',
                           'rtcp_sibling' => '4294967295',
                           'sfd' => '1',
                           'stats-bytes' => '0',
@@ -1416,8 +1655,7 @@ $json_exp = {
                           'advertised_endpoint' => '198.51.100.14:6088',
                           'component' => '1',
                           'endpoint' => '198.51.100.14:6088',
-                          'last_packet' => qr/^\d+$/,
-                          'media' => '1',
+                          'media' => '0',
                           'ps_flags' => '1114112',
                           'rtcp_sibling' => '3',
                           'sfd' => '2',
@@ -1429,9 +1667,8 @@ $json_exp = {
                           'advertised_endpoint' => '198.51.100.14:6089',
                           'component' => '2',
                           'endpoint' => '198.51.100.14:6089',
-                          'last_packet' => qr/^\d+$/,
-                          'media' => '1',
-                          'ps_flags' => '68288513',
+                          'media' => '0',
+                          'ps_flags' => '1179649',
                           'rtcp_sibling' => '4294967295',
                           'sfd' => '3',
                           'stats-bytes' => '0',
@@ -1442,7 +1679,6 @@ $json_exp = {
                           'advertised_endpoint' => '',
                           'component' => '1',
                           'endpoint' => '',
-                          'last_packet' => qr/^\d+$/,
                           'media' => '2',
                           'ps_flags' => '65536',
                           'rtcp_sibling' => '5',
@@ -1455,7 +1691,6 @@ $json_exp = {
                           'advertised_endpoint' => '',
                           'component' => '2',
                           'endpoint' => '',
-                          'last_packet' => qr/^\d+$/,
                           'media' => '2',
                           'ps_flags' => '131072',
                           'rtcp_sibling' => '4294967295',
@@ -1468,7 +1703,6 @@ $json_exp = {
                           'advertised_endpoint' => '',
                           'component' => '1',
                           'endpoint' => '',
-                          'last_packet' => qr/^\d+$/,
                           'media' => '3',
                           'ps_flags' => '65536',
                           'rtcp_sibling' => '7',
@@ -1481,7 +1715,6 @@ $json_exp = {
                           'advertised_endpoint' => '',
                           'component' => '2',
                           'endpoint' => '',
-                          'last_packet' => qr/^\d+$/,
                           'media' => '3',
                           'ps_flags' => '131072',
                           'rtcp_sibling' => '4294967295',
@@ -1514,11 +1747,11 @@ $json_exp = {
           'stream_sfds-7' => [
                                '7'
                              ],
-          'streams-0' => [
+          'streams-1' => [
                            '0',
                            '1'
                          ],
-          'streams-1' => [
+          'streams-0' => [
                            '2',
                            '3'
                          ],
@@ -1537,7 +1770,7 @@ $json_exp = {
                                  '0/1/0/0'
                                ],
           'media-subscriptions-2' => [
-                                 '1/0/0/0'
+                                 '0/0/0/0'
                                ],
           'tag-0' => {
                        'block_dtmf' => '0',
@@ -1589,12 +1822,12 @@ a=rtcp:PORT
 SDP
 
 
-$json_exp->{'media-0'}{media_flags} = '2293772';
-$json_exp->{'media-1'}{media_flags} = '65548';
+$json_exp->{'media-1'}{media_flags} = '274880200716';
+$json_exp->{'media-0'}{media_flags} = '274877972492';
 $json_exp->{'media-2'}{format_str} = '8';
-$json_exp->{'media-2'}{media_flags} = '2162692';
+$json_exp->{'media-2'}{media_flags} = '274880069636';
 $json_exp->{'media-3'}{format_str} = '8';
-$json_exp->{'media-3'}{media_flags} = '2162692';
+$json_exp->{'media-3'}{media_flags} = '274880069636';
 $json_exp->{'payload_types-2'}[0] = '8/PCMA/8000//0/20//';
 $#{$json_exp->{'payload_types-2'}} = 0;
 $json_exp->{'payload_types-3'}[0] = '8/PCMA/8000//0/20//';
@@ -1632,7 +1865,6 @@ SDP
 
 
 
-
 new_call;
 
 $json_exp = {
@@ -1642,7 +1874,6 @@ $json_exp = {
 	      'call_flags' => 0,
 	      'created' => qr//,
 	      'created_from' => qr//,
-	      'created_from_addr' => qr//,
 	      'deleted' => '0',
 	      'destroyed' => '0',
 	      'last_signal' => qr//,
@@ -1676,7 +1907,7 @@ $json_exp = {
 		 'format_str' => '0 8 9',
 		 'index' => '1',
 		 'logical_intf' => 'foo',
-		 'media_flags' => '65544',
+		 'media_flags' => '274877972488',
 		 'protocol' => 'RTP/AVP',
 		 'ptime' => '0',
 		 'tag' => '0',
@@ -1713,7 +1944,6 @@ $json_exp = {
 		  'advertised_endpoint' => '198.51.100.14:6042',
 		  'component' => '1',
 		  'endpoint' => '198.51.100.14:6042',
-		  'last_packet' => qr//,
 		  'media' => '0',
 		  'ps_flags' => '1114112',
 		  'rtcp_sibling' => '1',
@@ -1726,7 +1956,6 @@ $json_exp = {
 		  'advertised_endpoint' => '198.51.100.14:6043',
 		  'component' => '2',
 		  'endpoint' => '198.51.100.14:6043',
-		  'last_packet' => qr//,
 		  'media' => '0',
 		  'ps_flags' => '1179649',
 		  'rtcp_sibling' => '4294967295',
@@ -1786,7 +2015,6 @@ $json_exp = {
 	      'call_flags' => 0,
 	      'created' => qr//,
 	      'created_from' => qr//,
-	      'created_from_addr' => qr//,
 	      'deleted' => '0',
 	      'destroyed' => '0',
 	      'last_signal' => qr//,
@@ -1835,7 +2063,7 @@ $json_exp = {
 		 'format_str' => '0 8 9',
 		 'index' => '1',
 		 'logical_intf' => 'foo',
-		 'media_flags' => '65544',
+		 'media_flags' => '274877972488',
 		 'protocol' => 'RTP/AVP',
 		 'ptime' => '0',
 		 'tag' => '0',
@@ -1846,7 +2074,7 @@ $json_exp = {
 		 'format_str' => '0 8 9',
 		 'index' => '1',
 		 'logical_intf' => 'foo',
-		 'media_flags' => '2097156',
+		 'media_flags' => '274880004100',
 		 'protocol' => 'RTP/AVP',
 		 'ptime' => '0',
 		 'tag' => '1',
@@ -1914,7 +2142,6 @@ $json_exp = {
 		  'advertised_endpoint' => '198.51.100.14:6042',
 		  'component' => '1',
 		  'endpoint' => '198.51.100.14:6042',
-		  'last_packet' => qr//,
 		  'media' => '0',
 		  'ps_flags' => '1114112',
 		  'rtcp_sibling' => '1',
@@ -1927,9 +2154,8 @@ $json_exp = {
 		  'advertised_endpoint' => '198.51.100.14:6043',
 		  'component' => '2',
 		  'endpoint' => '198.51.100.14:6043',
-		  'last_packet' => qr//,
 		  'media' => '0',
-		  'ps_flags' => '68288513',
+		  'ps_flags' => '1179649',
 		  'rtcp_sibling' => '4294967295',
 		  'sfd' => '1',
 		  'stats-bytes' => '0',
@@ -1940,7 +2166,6 @@ $json_exp = {
 		  'advertised_endpoint' => '',
 		  'component' => '1',
 		  'endpoint' => '',
-		  'last_packet' => qr//,
 		  'media' => '1',
 		  'ps_flags' => '65536',
 		  'rtcp_sibling' => '3',
@@ -1953,7 +2178,6 @@ $json_exp = {
 		  'advertised_endpoint' => '',
 		  'component' => '2',
 		  'endpoint' => '',
-		  'last_packet' => qr//,
 		  'media' => '1',
 		  'ps_flags' => '131072',
 		  'rtcp_sibling' => '4294967295',
@@ -2027,7 +2251,6 @@ $json_exp = {
 	      'call_flags' => 0,
 	      'created' => qr//,
 	      'created_from' => qr//,
-	      'created_from_addr' => qr//,
 	      'deleted' => '0',
 	      'destroyed' => '0',
 	      'last_signal' => qr//,
@@ -2076,7 +2299,7 @@ $json_exp = {
 		 'format_str' => '0 8 9',
 		 'index' => '1',
 		 'logical_intf' => 'foo',
-		 'media_flags' => '65544',
+		 'media_flags' => '274877972488',
 		 'protocol' => 'RTP/AVP',
 		 'ptime' => '0',
 		 'tag' => '0',
@@ -2087,7 +2310,7 @@ $json_exp = {
 		 'format_str' => '0',
 		 'index' => '1',
 		 'logical_intf' => 'foo',
-		 'media_flags' => '2162692',
+		 'media_flags' => '274880069636',
 		 'protocol' => 'RTP/AVP',
 		 'ptime' => '0',
 		 'tag' => '1',
@@ -2155,7 +2378,6 @@ $json_exp = {
 		  'advertised_endpoint' => '198.51.100.14:6042',
 		  'component' => '1',
 		  'endpoint' => '198.51.100.14:6042',
-		  'last_packet' => qr//,
 		  'media' => '0',
 		  'ps_flags' => '1114112',
 		  'rtcp_sibling' => '1',
@@ -2168,7 +2390,6 @@ $json_exp = {
 		  'advertised_endpoint' => '198.51.100.14:6043',
 		  'component' => '2',
 		  'endpoint' => '198.51.100.14:6043',
-		  'last_packet' => qr//,
 		  'media' => '0',
 		  'ps_flags' => '1179649',
 		  'rtcp_sibling' => '4294967295',
@@ -2181,7 +2402,6 @@ $json_exp = {
 		  'advertised_endpoint' => '198.51.100.14:6044',
 		  'component' => '1',
 		  'endpoint' => '198.51.100.14:6044',
-		  'last_packet' => qr//,
 		  'media' => '1',
 		  'ps_flags' => '1114112',
 		  'rtcp_sibling' => '3',
@@ -2194,7 +2414,6 @@ $json_exp = {
 		  'advertised_endpoint' => '198.51.100.14:6045',
 		  'component' => '2',
 		  'endpoint' => '198.51.100.14:6045',
-		  'last_packet' => qr//,
 		  'media' => '1',
 		  'ps_flags' => '1179649',
 		  'rtcp_sibling' => '4294967295',
@@ -2267,7 +2486,6 @@ $json_exp = {
 	      'call_flags' => 0,
 	      'created' => qr//,
 	      'created_from' => qr//,
-	      'created_from_addr' => qr//,
 	      'deleted' => '0',
 	      'destroyed' => '0',
 	      'last_signal' => qr//,
@@ -2331,7 +2549,7 @@ $json_exp = {
 		 'format_str' => '0 8 9',
 		 'index' => '1',
 		 'logical_intf' => 'foo',
-		 'media_flags' => '65544',
+		 'media_flags' => '274877972488',
 		 'protocol' => 'RTP/AVP',
 		 'ptime' => '0',
 		 'tag' => '0',
@@ -2342,7 +2560,7 @@ $json_exp = {
 		 'format_str' => '0',
 		 'index' => '1',
 		 'logical_intf' => 'foo',
-		 'media_flags' => '2162692',
+		 'media_flags' => '274880069636',
 		 'protocol' => 'RTP/AVP',
 		 'ptime' => '0',
 		 'tag' => '1',
@@ -2353,7 +2571,7 @@ $json_exp = {
 		 'format_str' => '0 8 9',
 		 'index' => '1',
 		 'logical_intf' => 'foo',
-		 'media_flags' => '2097156',
+		 'media_flags' => '274880004100',
 		 'protocol' => 'RTP/AVP',
 		 'ptime' => '0',
 		 'tag' => '2',
@@ -2450,7 +2668,6 @@ $json_exp = {
 		  'advertised_endpoint' => '198.51.100.14:6042',
 		  'component' => '1',
 		  'endpoint' => '198.51.100.14:6042',
-		  'last_packet' => qr//,
 		  'media' => '0',
 		  'ps_flags' => '1114112',
 		  'rtcp_sibling' => '1',
@@ -2463,9 +2680,8 @@ $json_exp = {
 		  'advertised_endpoint' => '198.51.100.14:6043',
 		  'component' => '2',
 		  'endpoint' => '198.51.100.14:6043',
-		  'last_packet' => qr//,
 		  'media' => '0',
-		  'ps_flags' => '68288513',
+		  'ps_flags' => '1179649',
 		  'rtcp_sibling' => '4294967295',
 		  'sfd' => '1',
 		  'stats-bytes' => '0',
@@ -2476,7 +2692,6 @@ $json_exp = {
 		  'advertised_endpoint' => '198.51.100.14:6044',
 		  'component' => '1',
 		  'endpoint' => '198.51.100.14:6044',
-		  'last_packet' => qr//,
 		  'media' => '1',
 		  'ps_flags' => '1114112',
 		  'rtcp_sibling' => '3',
@@ -2489,7 +2704,6 @@ $json_exp = {
 		  'advertised_endpoint' => '198.51.100.14:6045',
 		  'component' => '2',
 		  'endpoint' => '198.51.100.14:6045',
-		  'last_packet' => qr//,
 		  'media' => '1',
 		  'ps_flags' => '1179649',
 		  'rtcp_sibling' => '4294967295',
@@ -2502,7 +2716,6 @@ $json_exp = {
 		  'advertised_endpoint' => '',
 		  'component' => '1',
 		  'endpoint' => '',
-		  'last_packet' => qr//,
 		  'media' => '2',
 		  'ps_flags' => '65536',
 		  'rtcp_sibling' => '5',
@@ -2515,7 +2728,6 @@ $json_exp = {
 		  'advertised_endpoint' => '',
 		  'component' => '2',
 		  'endpoint' => '',
-		  'last_packet' => qr//,
 		  'media' => '2',
 		  'ps_flags' => '131072',
 		  'rtcp_sibling' => '4294967295',
@@ -2604,7 +2816,7 @@ a=rtcp:PORT
 SDP
 
 $json_exp->{'media-2'}{format_str} = '0';
-$json_exp->{'media-2'}{media_flags} = '2162692';
+$json_exp->{'media-2'}{media_flags} = '274880069636';
 $json_exp->{'stream-1'}{ps_flags}  = '1179649';
 $json_exp->{'stream-4'}{advertised_endpoint} = '198.51.100.14:6046';
 $json_exp->{'stream-4'}{endpoint}  = '198.51.100.14:6046';
